@@ -16,6 +16,8 @@
  * ====================================================================
  */
 
+/* Tell swigutil_py.h that we're inside the implementation */
+#define SVN_SWIG_SWIGUTIL_PY_C
 
 #include <Python.h>
 
@@ -31,16 +33,12 @@
 #include "svn_string.h"
 #include "svn_opt.h"
 #include "svn_delta.h"
+#include "svn_auth.h"
+#include "svn_pools.h"
 
-#if SVN_SWIG_VERSION >= 103024
-#if SVN_SWIG_VERSION >= 103025
-#include <swiglabels.swg>
-#endif
-#include <swigrun.swg>
-#include <python/pyrun.swg>
-#include <runtime.swg>
-#endif
+#include "svn_private_config.h" /* for SVN_APR_INT64_T_PYCFMT */
 
+#include "swig_python_external_runtime.swg"
 #include "swigutil_py.h"
 
 
@@ -75,14 +73,6 @@ void svn_swig_py_release_py_lock(void)
   PyThreadState *thread_state;
 
   if (_saved_thread_key == NULL) {
-
-    /* This is ugly.  We call svn_swig_py_release_py_lock before executing any
-       subversion function.  Thus it gets called before any call to
-       apr_initialize in our script.  This means we have to call
-       apr_initialize ourselves, or otherwise we won't be able to
-       create our pool. */
-    apr_initialize();
-    
     /* Obviously, creating a top-level pool for this is pretty stupid. */
     apr_pool_create(&_saved_thread_pool, NULL);
     apr_threadkey_private_create(&_saved_thread_key, NULL, _saved_thread_pool);
@@ -105,6 +95,167 @@ void svn_swig_py_acquire_py_lock(void)
 }
 
 
+
+/*** Automatic Pool Management Functions ***/
+
+/* The application pool */
+static apr_pool_t *_global_pool = NULL;
+static PyObject *_global_svn_swig_py_pool = NULL;
+static char assertValid[] = "assert_valid";
+static char parentPool[] = "_parent_pool";
+static char wrap[] = "_wrap";
+static char unwrap[] = "_unwrap";
+static char setParentPool[] = "set_parent_pool";
+static char emptyTuple[] = "()";
+static char objectTuple[] = "(O)";
+
+
+apr_status_t svn_swig_py_initialize(void)
+{
+  apr_status_t status;
+
+  if ((status = apr_initialize()) != APR_SUCCESS)
+    return status;
+  if (atexit(apr_terminate) != 0)
+    return APR_EGENERAL;
+  return APR_SUCCESS;
+}
+
+/* Set the application pool */
+void svn_swig_py_set_application_pool(PyObject *py_pool, apr_pool_t *pool) 
+{
+  _global_pool = pool;
+  _global_svn_swig_py_pool = py_pool;
+}
+
+/* Clear the application pool */
+void svn_swig_py_clear_application_pool()
+{
+  _global_pool = NULL;
+  _global_svn_swig_py_pool = NULL;
+}
+
+/* Get the application pool */
+void svn_swig_get_application_pool(PyObject **py_pool, apr_pool_t **pool)
+{
+  *pool = _global_pool;
+  *py_pool = _global_svn_swig_py_pool;
+}
+
+/* Set the parent pool of a proxy object */
+static int proxy_set_pool(PyObject **proxy, PyObject *pool)
+{
+  PyObject *result;
+
+  if (*proxy != NULL) {
+    if (pool == NULL) {
+      if (PyObject_HasAttrString(*proxy, setParentPool)) {
+        result = PyObject_CallMethod(*proxy, setParentPool, emptyTuple);
+        if (result == NULL) {
+          return 1;
+        }
+        Py_DECREF(result);
+      }
+    } else {
+      result = PyObject_CallMethod(pool, wrap, objectTuple, *proxy);
+      Py_DECREF(*proxy);
+      *proxy = result;
+    }
+  }
+
+  return 0;
+}
+
+/* Wrapper for SWIG_TypeQuery */
+#define svn_swig_TypeQuery(x) SWIG_TypeQuery(x)
+
+/** Wrapper for SWIG_NewPointerObj */
+PyObject *svn_swig_NewPointerObj(void *obj, swig_type_info *type, 
+                                 PyObject *pool)
+{
+  PyObject *proxy = SWIG_NewPointerObj(obj, type, 0);
+
+  if (proxy == NULL) {
+    return NULL;
+  }
+  
+  if (proxy_set_pool(&proxy, pool)) {
+    Py_DECREF(proxy);
+    return NULL;
+  }
+
+  return proxy;
+}
+
+/** svn_swig_NewPointerObj, except a string is used to describe the type */
+static PyObject *svn_swig_NewPointerObjString(void *ptr, const char *type, 
+                                              PyObject *py_pool)
+{
+  swig_type_info *typeinfo = svn_swig_TypeQuery(type);
+  if (typeinfo == NULL) {
+    PyErr_SetString(PyExc_TypeError, "Cannot find required typeobject");
+    return NULL;
+  }
+  /* ### cache the swig_type_info at some point? */
+  return svn_swig_NewPointerObj(ptr, typeinfo, py_pool);
+}
+
+/** Wrapper for SWIG_ConvertPtr */
+int svn_swig_ConvertPtr(PyObject *input, void **obj, swig_type_info *type)
+{
+  if (PyObject_HasAttrString(input, assertValid)) {
+    PyObject *result = PyObject_CallMethod(input, assertValid, emptyTuple);
+    if (result == NULL) {
+      return 1;
+    }
+    Py_DECREF(result);
+  }
+  if (PyObject_HasAttrString(input, unwrap)) {
+    input = PyObject_CallMethod(input, unwrap, emptyTuple);
+    if (input == NULL) {
+      return 1;
+    }
+    Py_DECREF(input);
+  }
+  return SWIG_ConvertPtr(input, obj, type, SWIG_POINTER_EXCEPTION | 0);
+}
+
+/** svn_swig_ConvertPtr, except a string is used to describe the type */
+static int svn_swig_ConvertPtrString(PyObject *input,
+    void **obj, const char *type)
+{
+  return svn_swig_ConvertPtr(input, obj, svn_swig_TypeQuery(type));
+}
+
+/** Wrapper for SWIG_MustGetPtr */
+void *svn_swig_MustGetPtr(void *input, swig_type_info *type, int argnum,
+                          PyObject **py_pool)
+{
+  if (PyObject_HasAttrString(input, assertValid)) {
+    PyObject *result = PyObject_CallMethod(input, assertValid, emptyTuple);
+    if (result == NULL) {
+      return NULL;
+    }
+    Py_DECREF(result);
+  }
+  if (py_pool != NULL) {
+    if (PyObject_HasAttrString(input, parentPool)) {
+      *py_pool = PyObject_GetAttrString(input, parentPool);
+      Py_DECREF(*py_pool);
+    } else {
+      *py_pool = _global_svn_swig_py_pool;
+    }
+  }
+  if (PyObject_HasAttrString(input, unwrap)) {
+    input = PyObject_CallMethod(input, unwrap, emptyTuple);
+    if (input == NULL) {
+      return NULL;
+    }
+    Py_DECREF((PyObject *) input);
+  }
+  return SWIG_MustGetPtr(input, type, argnum, SWIG_POINTER_EXCEPTION | 0);
+}
+  
 
 /*** Custom SubversionException stuffs. ***/
 
@@ -174,41 +325,55 @@ void svn_swig_py_svn_exception(svn_error_t *err)
 
 /*** Helper/Conversion Routines ***/
 
-static PyObject *make_pointer(const char *typename, void *ptr)
+/* Functions for making Python wrappers around Subversion structs */
+static PyObject *make_ob_pool(void *pool)
 {
-  /* ### cache the swig_type_info at some point? */
-  return SWIG_NewPointerObj(ptr, SWIG_TypeQuery(typename), 0);
+  /* Return a brand new default pool to Python. This pool isn't
+   * normally used for anything. It's just here for compatibility
+   * with Subversion 1.2. */
+  apr_pool_t *new_pool = svn_pool_create(_global_pool);
+  PyObject *new_py_pool = svn_swig_NewPointerObj(new_pool,
+    svn_swig_TypeQuery("apr_pool_t *"), _global_svn_swig_py_pool);
+  (void) pool; /* Silence compiler warnings about unused parameter. */
+  return new_py_pool;
 }
-
-/* for use by the "O&" format specifier */
-static PyObject *make_ob_pool(void *ptr)
+static PyObject *make_ob_fs_root(svn_fs_root_t *ptr, PyObject *py_pool)
 {
-  return make_pointer("apr_pool_t *", ptr);
-}
-
-/* for use by the "O&" format specifier */
-static PyObject *make_ob_window(void *ptr)
-{
-  return make_pointer("svn_txdelta_window_t *", ptr);
-}
-
-/* for use by the "O&" format specifier */
-static PyObject *make_ob_status(void *ptr)
-{
-  return make_pointer("svn_wc_status_t *", ptr);
+  return svn_swig_NewPointerObjString(ptr, "svn_fs_root_t *", py_pool);
 } 
+/***/
 
-/* for use by the "O&" format specifier */
-static PyObject *make_ob_fs_root(void *ptr)
+/* Conversion from Python single objects (not hashes/lists/etc.) to
+   Subversion types. */
+static const char *make_string_from_ob(PyObject *ob, apr_pool_t *pool)
 {
-  return make_pointer("svn_fs_root_t *", ptr);
-} 
+  if (ob == Py_None)
+    return NULL;
+  if (! PyString_Check(ob)) {
+    PyErr_SetString(PyExc_TypeError, "not a string");
+    return NULL;
+  }
+  return apr_pstrdup(pool, PyString_AS_STRING(ob));
+}
+static svn_string_t *make_svn_string_from_ob(PyObject *ob, apr_pool_t *pool)
+{
+  if (ob == Py_None)
+    return NULL;
+  if (! PyString_Check(ob)) {
+    PyErr_SetString(PyExc_TypeError, "not a string");
+    return NULL;
+  }
+  return svn_string_create(PyString_AS_STRING(ob), pool);
+}
 
 
+/***/
+
 static PyObject *convert_hash(apr_hash_t *hash,
                               PyObject * (*converter_func)(void *value,
-                                                           void *ctx),
-                              void *ctx)
+                                                           void *ctx,
+                                                           PyObject *py_pool),
+                              void *ctx, PyObject *py_pool)
 {
     apr_hash_index_t *hi;
     PyObject *dict = PyDict_New();
@@ -222,7 +387,7 @@ static PyObject *convert_hash(apr_hash_t *hash,
         PyObject *value;
 
         apr_hash_this(hi, &key, NULL, &val);
-        value = (*converter_func)(val, ctx);
+        value = (*converter_func)(val, ctx, py_pool);
         if (value == NULL) {
             Py_DECREF(dict);
             return NULL;
@@ -239,23 +404,21 @@ static PyObject *convert_hash(apr_hash_t *hash,
     return dict;
 }
 
-static PyObject *convert_to_swigtype(void *value, void *ctx)
+static PyObject *convert_to_swigtype(void *value, void *ctx, PyObject *py_pool)
 {
   /* ctx is a 'swig_type_info *' */
-  return SWIG_NewPointerObj(value, ctx, 0);
+  return svn_swig_NewPointerObj(value, ctx, py_pool);
 }
 
-static PyObject *convert_svn_string_t(void *value, void *ctx)
+static PyObject *convert_svn_string_t(void *value, void *ctx,
+                                      PyObject *py_pool)
 {
   /* ctx is unused */
 
   const svn_string_t *s = value;
 
-  /* ### borrowing from value in the pool. or should we copy? note
-     ### that copying is "safest" */
-
   /* ### gotta cast this thing cuz Python doesn't use "const" */
-  return PyBuffer_FromMemory((void *)s->data, s->len);
+  return PyString_FromStringAndSize((void *)s->data, s->len);
 }
 
 static PyObject *convert_svn_client_commit_item_t(void *value, void *ctx)
@@ -319,12 +482,79 @@ static PyObject *convert_svn_client_commit_item_t(void *value, void *ctx)
 
 PyObject *svn_swig_py_prophash_to_dict(apr_hash_t *hash)
 {
-  return convert_hash(hash, convert_svn_string_t, NULL);
+  return convert_hash(hash, convert_svn_string_t, NULL, NULL);
 }
 
-PyObject *svn_swig_py_convert_hash(apr_hash_t *hash, swig_type_info *type)
+
+PyObject *svn_swig_py_locationhash_to_dict(apr_hash_t *hash)
 {
-  return convert_hash(hash, convert_to_swigtype, type);
+    /* Need special code for this because of the darned svn_revnum_t
+       keys. */
+    apr_hash_index_t *hi;
+    PyObject *dict = PyDict_New();
+
+    if (dict == NULL)
+        return NULL;
+
+    for (hi = apr_hash_first(NULL, hash); hi; hi = apr_hash_next(hi)) 
+      {
+        const void *k;
+        void *v;
+        PyObject *key, *value;
+
+        apr_hash_this(hi, &k, NULL, &v);
+        key = PyLong_FromLong(*(svn_revnum_t *)k);
+        if (key == NULL)
+          {
+            Py_DECREF(dict);
+            return NULL;
+          }
+        value = PyString_FromString((char *)v);
+        if (value == NULL) {
+            Py_DECREF(key);
+            Py_DECREF(dict);
+            return NULL;
+        }
+        if (PyDict_SetItem(dict, key, value) == -1) 
+          {
+            Py_DECREF(value);
+            Py_DECREF(dict);
+            return NULL;
+          }
+        Py_DECREF(value);
+        Py_DECREF(key);
+      }
+    return dict;
+}
+
+PyObject *svn_swig_py_convert_hash(apr_hash_t *hash, swig_type_info *type, 
+                                   PyObject *py_pool)
+{
+  return convert_hash(hash, convert_to_swigtype, type, py_pool);
+}  
+
+#define DECLARE_SWIG_CONSTRUCTOR(type, dup) \
+static PyObject *make_ob_##type(void *value) \
+{ \
+  apr_pool_t *new_pool = svn_pool_create(_global_pool); \
+  PyObject *new_py_pool = svn_swig_NewPointerObj(new_pool, \
+    svn_swig_TypeQuery("apr_pool_t *"), _global_svn_swig_py_pool); \
+  svn_##type##_t *new_value = dup(value, new_pool); \
+  return svn_swig_NewPointerObjString(new_value, "svn_" #type "_t *", \
+      new_py_pool); \
+}
+ 
+DECLARE_SWIG_CONSTRUCTOR(txdelta_window, svn_txdelta_window_dup)
+DECLARE_SWIG_CONSTRUCTOR(log_changed_path, svn_log_changed_path_dup)
+DECLARE_SWIG_CONSTRUCTOR(wc_status, svn_wc_dup_status)
+DECLARE_SWIG_CONSTRUCTOR(lock, svn_lock_dup)
+DECLARE_SWIG_CONSTRUCTOR(auth_ssl_server_cert_info,
+    svn_auth_ssl_server_cert_info_dup)
+
+static PyObject *convert_log_changed_path(void *value, void *ctx,
+                                          PyObject *py_pool)
+{
+  return make_ob_log_changed_path(value);
 }
 
 PyObject *svn_swig_py_c_strings_to_list(char **strings)
@@ -348,6 +578,86 @@ PyObject *svn_swig_py_c_strings_to_list(char **strings)
     return NULL;
 }
 
+apr_hash_t *svn_swig_py_stringhash_from_dict(PyObject *dict,
+                                             apr_pool_t *pool)
+{
+  apr_hash_t *hash;
+  PyObject *keys;
+  int i, num_keys;
+  
+  if (dict == Py_None)
+    return NULL;
+
+  if (!PyDict_Check(dict)) {
+    PyErr_SetString(PyExc_TypeError, "not a dictionary");
+    return NULL;
+  }
+
+  hash = apr_hash_make(pool);  
+  keys = PyDict_Keys(dict);
+  num_keys = PyList_Size(keys);
+  for (i = 0; i < num_keys; i++)
+    {
+      PyObject *key = PyList_GetItem(keys, i);
+      PyObject *value = PyDict_GetItem(dict, key);
+      const char *propname = make_string_from_ob(key, pool);
+      const char *propval = make_string_from_ob(value, pool);
+      Py_DECREF(key);
+      Py_DECREF(value);
+      if (! (propname && propval))
+        {
+          PyErr_SetString(PyExc_TypeError, 
+                          "dictionary keys/values aren't strings");
+          Py_DECREF(keys);
+          return NULL;
+        }
+      apr_hash_set(hash, propname, APR_HASH_KEY_STRING, propval);
+    }
+  Py_DECREF(keys);
+  return hash;
+}
+
+
+apr_hash_t *svn_swig_py_prophash_from_dict(PyObject *dict,
+                                           apr_pool_t *pool)
+{
+  apr_hash_t *hash;
+  PyObject *keys;
+  int i, num_keys;
+
+  if (dict == Py_None)
+    return NULL;
+
+  if (!PyDict_Check(dict)) {
+    PyErr_SetString(PyExc_TypeError, "not a dictionary");
+    return NULL;
+  }
+
+  hash = apr_hash_make(pool);  
+  keys = PyDict_Keys(dict);
+  num_keys = PyList_Size(keys);
+  for (i = 0; i < num_keys; i++)
+    {
+      PyObject *key = PyList_GetItem(keys, i);
+      PyObject *value = PyDict_GetItem(dict, key);
+      const char *propname = make_string_from_ob(key, pool);
+      svn_string_t *propval = make_svn_string_from_ob(value, pool);
+      Py_DECREF(key);
+      Py_DECREF(value);
+      if (! (propname && propval))
+        {
+          PyErr_SetString(PyExc_TypeError, 
+                          "dictionary keys/values aren't strings");
+          Py_DECREF(keys);
+          return NULL;
+        }
+      apr_hash_set(hash, propname, APR_HASH_KEY_STRING, propval);
+    }
+  Py_DECREF(keys);
+  return hash;
+}
+
+
 const apr_array_header_t *svn_swig_py_strings_to_array(PyObject *source,
                                                        apr_pool_t *pool)
 {
@@ -378,6 +688,45 @@ const apr_array_header_t *svn_swig_py_strings_to_array(PyObject *source,
     return temp;
 }
 
+
+const apr_array_header_t *svn_swig_py_revnums_to_array(PyObject *source,
+                                                       apr_pool_t *pool)
+{
+    int targlen;
+    apr_array_header_t *temp;
+
+    if (!PySequence_Check(source)) {
+        PyErr_SetString(PyExc_TypeError, "not a sequence");
+        return NULL;
+    }
+    targlen = PySequence_Length(source);
+    temp = apr_array_make(pool, targlen, sizeof(svn_revnum_t));
+    /* APR_ARRAY_IDX doesn't actually increment the array item count
+       (like, say, apr_array_push would). */
+    temp->nelts = targlen;
+    while (targlen--) {
+        PyObject *o = PySequence_GetItem(source, targlen);
+        if (o == NULL)
+            return NULL;
+        if (PyLong_Check(o)) {
+            APR_ARRAY_IDX(temp, targlen, svn_revnum_t) = 
+              (svn_revnum_t)PyLong_AsLong(o);
+        }
+        else if (PyInt_Check(o)) {
+            APR_ARRAY_IDX(temp, targlen, svn_revnum_t) = 
+              (svn_revnum_t)PyInt_AsLong(o);
+        }
+        else {
+            Py_DECREF(o);
+            PyErr_SetString(PyExc_TypeError, "not an integer type");
+            return NULL;
+        }
+        Py_DECREF(o);
+    }
+    return temp;
+}
+
+
 
 /*** apr_array_header_t conversions.  To create a new type of
      converter, simply copy-n-paste one of these function and tweak
@@ -402,6 +751,7 @@ PyObject *svn_swig_py_array_to_list(const apr_array_header_t *array)
     return NULL;
 }
 
+/* Formerly used by pre-1.0 APIs. Now unused
 PyObject *svn_swig_py_revarray_to_list(const apr_array_header_t *array)
 {
     PyObject *list = PyList_New(array->nelts);
@@ -420,6 +770,7 @@ PyObject *svn_swig_py_revarray_to_list(const apr_array_header_t *array)
     Py_DECREF(list);
     return NULL;
 }
+*/
 
 static PyObject *
 commit_item_array_to_list(const apr_array_header_t *array)
@@ -443,7 +794,7 @@ commit_item_array_to_list(const apr_array_header_t *array)
 
 
 
-/*** Callback Errors ***/
+/*** Errors ***/
 
 /* Return a Subversion error about a failed callback. */
 static svn_error_t *callback_exception_error(void)
@@ -461,6 +812,13 @@ static svn_error_t *callback_bad_return_error(const char *message)
                           "Python callback returned an invalid object");
 }
 
+/* Return a generic error about not being able to map types. */
+static svn_error_t *type_conversion_error(const char *datatype)
+{
+  return svn_error_createf(APR_EGENERAL, NULL,
+                           "Error converting object of type '%s'", datatype);
+}
+
 
 
 /*** Editor Wrapping ***/
@@ -802,8 +1160,8 @@ static svn_error_t *window_handler(svn_txdelta_window_t *window,
     {
       /* invoke the handler with the window */
       /* ### python doesn't have 'const' on the format */
-      result = PyObject_CallFunction(handler,
-                                     (char *)"O&", make_ob_window, window);
+      result = PyObject_CallFunction(handler, (char *)"O&", 
+        make_ob_txdelta_window, window);
     }
 
   if (result == NULL)
@@ -980,26 +1338,33 @@ void svn_swig_py_make_editor(const svn_delta_editor_t **editor,
 /*** Other Wrappers for SVN Functions ***/
 
 
-/* This is very hacky and gross.  */
 apr_file_t *svn_swig_py_make_file (PyObject *py_file,
                                    apr_pool_t *pool)
 {
   apr_file_t *apr_file = NULL;
+  apr_status_t apr_err;
 
   if (py_file == NULL || py_file == Py_None)
     return NULL;
 
-  if (PyString_Check(py_file))
+  if (PyString_Check (py_file))
     {
       /* input is a path -- just open an apr_file_t */
-      apr_file_open(&apr_file, PyString_AS_STRING(py_file),
-                    APR_CREATE | APR_READ | APR_WRITE,
-                    APR_OS_DEFAULT,
-                    pool);
+      char* fname = PyString_AS_STRING (py_file);
+      apr_err = apr_file_open (&apr_file, fname,
+                              APR_CREATE | APR_READ | APR_WRITE,
+                              APR_OS_DEFAULT, pool);
+      if (apr_err)
+        {
+          char buf[256];
+          apr_strerror (apr_err, buf, sizeof(buf));
+          PyErr_Format (PyExc_IOError, "apr_file_open failed: %s: '%s'",
+                        buf, fname);
+          return NULL;
+        }
     }
   else if (PyFile_Check (py_file))
     {
-      apr_status_t status;
       FILE *file;
       apr_os_file_t osfile;
 
@@ -1010,14 +1375,97 @@ apr_file_t *svn_swig_py_make_file (PyObject *py_file,
 #else
       osfile = (apr_os_file_t)fileno(file);
 #endif
-      status = apr_os_file_put (&apr_file, &osfile, O_CREAT | O_WRONLY, pool);
-      if (status)
-        return NULL;
+      apr_err = apr_os_file_put (&apr_file, &osfile, O_CREAT | O_WRONLY, pool);
+      if (apr_err)
+        {
+          char buf[256];
+          apr_strerror (apr_err, buf, sizeof(buf));
+          PyErr_Format (PyExc_IOError, "apr_os_file_put failed: %s", buf);
+          return NULL;
+        }
     }
   return apr_file;
 }
 
 
+static svn_error_t *
+read_handler_pyio (void *baton, char *buffer, apr_size_t *len)
+{
+  PyObject *result;
+  PyObject *py_io = baton;
+  apr_size_t bytes;
+  svn_error_t *err = SVN_NO_ERROR;
+
+  svn_swig_py_acquire_py_lock();
+  if ((result = PyObject_CallMethod(py_io, (char *)"read",
+                                    (char *)"i", *len)) == NULL)
+    {
+      err = callback_exception_error();
+    }
+  else if (PyString_Check(result))
+    {
+      bytes = PyString_GET_SIZE(result);
+      if (bytes > *len)
+        {
+          err = callback_bad_return_error("Too many bytes");
+        }
+      else
+        {
+          /* Writeback, in case this was a short read, indicating EOF */
+          *len = bytes;
+          memcpy(buffer, PyString_AS_STRING(result), *len);
+        }
+    }
+  else
+    {
+      err = callback_bad_return_error("Not a string");
+    }
+  Py_XDECREF(result);
+  svn_swig_py_release_py_lock();
+
+  return err;
+}
+
+static svn_error_t *
+write_handler_pyio (void *baton, const char *data, apr_size_t *len)
+{
+  PyObject *result;
+  PyObject *py_io = baton;
+  svn_error_t *err = SVN_NO_ERROR;
+
+  if (data != NULL)
+    {
+      svn_swig_py_acquire_py_lock();
+      if ((result = PyObject_CallMethod(py_io, (char *)"write",
+                                        (char *)"s#", data, *len)) == NULL)
+        {
+          err = callback_exception_error();
+        }
+      Py_XDECREF(result);
+      svn_swig_py_release_py_lock();
+    }
+
+  return err;
+}
+
+svn_stream_t *
+svn_swig_py_make_stream (PyObject *py_io, apr_pool_t *pool)
+{
+  svn_stream_t *stream;
+
+  /* Borrow the caller's reference to py_io - this is safe only because the
+   * caller must have a reference in order to pass the object into the 
+   * bindings, and we will be finished with the py_io object before we return
+   * to python. I.e. DO NOT STORE AWAY THE RESULTING svn_stream_t * for use
+   * over multiple calls into the bindings. */
+  stream = svn_stream_create (py_io, pool);
+  svn_stream_set_read (stream, read_handler_pyio);
+  svn_stream_set_write (stream, write_handler_pyio);
+
+  return stream;
+}
+
+
 void svn_swig_py_notify_func(void *baton,
                              const char *path,
                              svn_wc_notify_action_t action,
@@ -1073,7 +1521,7 @@ void svn_swig_py_status_func(void *baton,
 
   svn_swig_py_acquire_py_lock();
   if ((result = PyObject_CallFunction(function, (char *)"sO&", path,
-                                      make_ob_status, status)) == NULL)
+                                      make_ob_wc_status, status)) == NULL)
     {
       err = callback_exception_error();
     }
@@ -1129,6 +1577,36 @@ svn_error_t *svn_swig_py_cancel_func(void *cancel_baton)
   return err;
 }
 
+svn_error_t *svn_swig_py_fs_get_locks_func (void *baton, 
+                                            svn_lock_t *lock,
+                                            apr_pool_t *pool)
+{
+  PyObject *function = baton;
+  PyObject *result;
+  svn_error_t *err = SVN_NO_ERROR;
+
+  if (function == NULL || function == Py_None)
+    return SVN_NO_ERROR;
+
+  svn_swig_py_acquire_py_lock();
+  
+  if ((result = PyObject_CallFunction(function, (char *)"O&O&",
+                                      make_ob_lock, lock,
+                                      make_ob_pool, pool)) == NULL)
+    {
+      err = callback_exception_error();
+    }
+  else
+    {
+      /* The callback shouldn't be returning anything. */
+      if (result != Py_None)
+        err = callback_bad_return_error("Not None");
+      Py_DECREF(result);
+    }
+
+  svn_swig_py_release_py_lock();
+  return err;
+}
 
 svn_error_t *svn_swig_py_get_commit_log_func(const char **log_msg,
                                              const char **tmp_file,
@@ -1205,18 +1683,31 @@ svn_error_t *svn_swig_py_repos_authz_func(svn_boolean_t *allowed,
 {
   PyObject *function = baton;
   PyObject *result;
+  PyObject *py_pool, *py_root;
   svn_error_t *err = SVN_NO_ERROR;
 
+  *allowed = TRUE;
+
   if (function == NULL || function == Py_None)
     return SVN_NO_ERROR;
 
-  *allowed = TRUE;
-
   svn_swig_py_acquire_py_lock();
+  
+  py_pool = make_ob_pool(pool);
+  if (py_pool == NULL) {
+    err = callback_exception_error();
+    goto finished;
+  }
+  py_root = make_ob_fs_root(root, py_pool);
+  if (py_root == NULL) {
+    Py_DECREF(py_pool);
+    err = callback_exception_error();
+    goto finished; 
+  }
+
   if ((result = PyObject_CallFunction(function, 
-                                      (char *)"O&sO&", 
-                                      make_ob_fs_root, root,
-                                      path, make_ob_pool, pool)) == NULL)
+                                      (char *)"OsO", 
+                                      py_root, path, py_pool)) == NULL)
     {
       err = callback_exception_error();
     }
@@ -1230,6 +1721,9 @@ svn_error_t *svn_swig_py_repos_authz_func(svn_boolean_t *allowed,
         err = callback_bad_return_error("Not an integer");
       Py_DECREF(result);
     }
+  Py_DECREF(py_root);
+  Py_DECREF(py_pool);
+finished:
   svn_swig_py_release_py_lock();
   return err;
 }
@@ -1275,7 +1769,7 @@ svn_error_t *svn_swig_py_log_receiver(void *baton,
                                       apr_pool_t *pool)
 {
   PyObject *receiver = baton;
-  PyObject *result;
+  PyObject *result, *py_pool;
   PyObject *chpaths;
   svn_error_t *err = SVN_NO_ERROR;
  
@@ -1284,10 +1778,16 @@ svn_error_t *svn_swig_py_log_receiver(void *baton,
 
   svn_swig_py_acquire_py_lock();
 
+  py_pool = make_ob_pool(pool);
+  if (py_pool == NULL) {
+    err = callback_exception_error();
+    goto finished;
+  }
+
   if (changed_paths)
     {
-      swig_type_info *tinfo = SWIG_TypeQuery("svn_log_changed_path_t *");
-      chpaths = svn_swig_py_convert_hash (changed_paths, tinfo);
+      chpaths = convert_hash(changed_paths, convert_log_changed_path,
+                             NULL, NULL);
     }
   else
     {
@@ -1296,9 +1796,9 @@ svn_error_t *svn_swig_py_log_receiver(void *baton,
     }
 
   if ((result = PyObject_CallFunction(receiver, 
-                                      (char *)"OlsssO&", 
+                                      (char *)"OlsssO", 
                                       chpaths, rev, author, date, msg, 
-                                      make_ob_pool, pool)) == NULL)
+                                      py_pool)) == NULL)
     {
       err = callback_exception_error();
     }
@@ -1310,6 +1810,303 @@ svn_error_t *svn_swig_py_log_receiver(void *baton,
     }
 
   Py_DECREF(chpaths);
+  Py_DECREF(py_pool);
+finished:
   svn_swig_py_release_py_lock();
   return err;
 }
+
+
+svn_error_t *svn_swig_py_client_blame_receiver_func(void *baton,
+                                                    apr_int64_t line_no,
+                                                    svn_revnum_t revision,
+                                                    const char *author,
+                                                    const char *date,
+                                                    const char *line,
+                                                    apr_pool_t *pool)
+{
+  PyObject *receiver = baton;
+  PyObject *result;
+  svn_error_t *err = SVN_NO_ERROR;
+ 
+  if ((receiver == NULL) || (receiver == Py_None))
+    return SVN_NO_ERROR;
+
+  svn_swig_py_acquire_py_lock();
+
+  if ((result = PyObject_CallFunction(receiver, 
+                                      (char *)
+                                      (SVN_APR_INT64_T_PYCFMT "lsssO&"), 
+                                      line_no, revision, author, date, line, 
+                                      make_ob_pool, pool)) == NULL)
+    {
+      err = callback_exception_error();
+    }
+  else
+    {
+      if (result != Py_None)
+        err = callback_bad_return_error("Not None");
+      Py_DECREF(result);
+    }
+
+  svn_swig_py_release_py_lock();
+  return err;
+}
+
+
+svn_error_t *
+svn_swig_py_auth_simple_prompt_func (svn_auth_cred_simple_t **cred,
+                                     void *baton,
+                                     const char *realm,
+                                     const char *username,
+                                     svn_boolean_t may_save,
+                                     apr_pool_t *pool)
+{
+  PyObject *function = baton;
+  PyObject *result;
+  svn_auth_cred_simple_t *creds = NULL;
+  svn_error_t *err = SVN_NO_ERROR;
+
+  if ((function == NULL) || (function == Py_None))
+    return SVN_NO_ERROR;
+
+  svn_swig_py_acquire_py_lock();
+
+  if ((result = PyObject_CallFunction(function, 
+                                      (char *)"sslO&", 
+                                      realm, username, may_save, 
+                                      make_ob_pool, pool)) == NULL)
+    {
+      err = callback_exception_error();
+    }
+  else
+    {
+      if (result != Py_None)
+        {
+          svn_auth_cred_simple_t *tmp_creds = NULL;
+          if (svn_swig_ConvertPtrString(result, (void **)&tmp_creds, 
+                "svn_auth_cred_simple_t *"))
+            {
+              err = type_conversion_error("svn_auth_cred_simple_t *");
+            }
+          else
+            {
+              creds = apr_pcalloc(pool, sizeof (*creds));
+              creds->username = tmp_creds->username ? \
+                apr_pstrdup(pool, tmp_creds->username) : NULL;
+              creds->password = tmp_creds->password ? \
+                apr_pstrdup(pool, tmp_creds->password) : NULL;
+              creds->may_save = tmp_creds->may_save;
+            }
+        }
+      Py_DECREF(result);
+    }
+  svn_swig_py_release_py_lock();
+  *cred = creds;
+  return err;
+}
+
+svn_error_t *
+svn_swig_py_auth_username_prompt_func (svn_auth_cred_username_t **cred,
+                                       void *baton,
+                                       const char *realm,
+                                       svn_boolean_t may_save,
+                                       apr_pool_t *pool)
+{
+  PyObject *function = baton;
+  PyObject *result;
+  svn_auth_cred_username_t *creds = NULL;
+  svn_error_t *err = SVN_NO_ERROR;
+
+  if ((function == NULL) || (function == Py_None))
+    return SVN_NO_ERROR;
+
+  svn_swig_py_acquire_py_lock();
+
+  if ((result = PyObject_CallFunction(function, 
+                                      (char *)"slO&", 
+                                      realm, may_save, 
+                                      make_ob_pool, pool)) == NULL)
+    {
+      err = callback_exception_error();
+    }
+  else
+    {
+      if (result != Py_None)
+        {
+          svn_auth_cred_username_t *tmp_creds = NULL;
+          if (svn_swig_ConvertPtrString(result, (void **)&tmp_creds, 
+                "svn_auth_cred_username_t *"))
+            {
+              err = type_conversion_error("svn_auth_cred_username_t *");
+            }
+          else
+            {
+              creds = apr_pcalloc(pool, sizeof (*creds));
+              creds->username = tmp_creds->username ? \
+                apr_pstrdup(pool, tmp_creds->username) : NULL;
+              creds->may_save = tmp_creds->may_save;
+            }
+        }
+      Py_DECREF(result);
+    }
+  svn_swig_py_release_py_lock();
+  *cred = creds;
+  return err;
+}
+
+
+svn_error_t *
+svn_swig_py_auth_ssl_server_trust_prompt_func(
+    svn_auth_cred_ssl_server_trust_t **cred,
+    void *baton,
+    const char *realm,
+    apr_uint32_t failures,
+    const svn_auth_ssl_server_cert_info_t *cert_info,
+    svn_boolean_t may_save,
+    apr_pool_t *pool)
+{
+  PyObject *function = baton;
+  PyObject *result;
+  svn_auth_cred_ssl_server_trust_t *creds = NULL;
+  svn_error_t *err = SVN_NO_ERROR;
+
+  if ((function == NULL) || (function == Py_None))
+    return SVN_NO_ERROR;
+
+  svn_swig_py_acquire_py_lock();
+
+  if ((result = PyObject_CallFunction(function, (char *)"slO&lO&", 
+                  realm, failures, make_ob_auth_ssl_server_cert_info,
+                  cert_info, may_save, make_ob_pool, pool)) == NULL)
+    {
+      err = callback_exception_error();
+    }
+  else
+    {
+      if (result != Py_None)
+        {
+          svn_auth_cred_ssl_server_trust_t *tmp_creds = NULL;
+          if (svn_swig_ConvertPtrString
+              (result, (void **)&tmp_creds, 
+               "svn_auth_cred_ssl_server_trust_t *"))
+            {
+              err = type_conversion_error
+                ("svn_auth_cred_ssl_server_trust_t *");
+            }
+          else
+            {
+              creds = apr_pcalloc(pool, sizeof (*creds));
+              *creds = *tmp_creds;
+            }
+        }
+      Py_DECREF(result);
+    }
+
+  svn_swig_py_release_py_lock();
+  *cred = creds;
+  return err;
+}
+
+svn_error_t *
+svn_swig_py_auth_ssl_client_cert_prompt_func(
+    svn_auth_cred_ssl_client_cert_t **cred,
+    void *baton,
+    const char *realm,
+    svn_boolean_t may_save,
+    apr_pool_t *pool)
+{
+  PyObject *function = baton;
+  PyObject *result;
+  svn_auth_cred_ssl_client_cert_t *creds = NULL;
+  svn_error_t *err = SVN_NO_ERROR;
+
+  if ((function == NULL) || (function == Py_None))
+    return SVN_NO_ERROR;
+
+  svn_swig_py_acquire_py_lock();
+
+  if ((result = PyObject_CallFunction(function, 
+                                      (char *)"slO&", 
+                                      realm, may_save, 
+                                      make_ob_pool, pool)) == NULL)
+    {
+      err = callback_exception_error();
+    }
+  else
+    {
+      if (result != Py_None)
+        {
+          svn_auth_cred_ssl_client_cert_t *tmp_creds = NULL;
+          if (svn_swig_ConvertPtrString
+              (result, (void **)&tmp_creds, 
+               "svn_auth_cred_ssl_client_cert_t *"))
+            {
+              err = type_conversion_error("svn_auth_cred_ssl_client_cert_t *");
+            }
+          else
+            {
+              creds = apr_pcalloc(pool, sizeof (*creds));
+              creds->cert_file = tmp_creds->cert_file ? \
+                apr_pstrdup(pool, tmp_creds->cert_file) : NULL;
+              creds->may_save = tmp_creds->may_save;
+            }
+        }
+      Py_DECREF(result);
+    }
+  svn_swig_py_release_py_lock();
+  *cred = creds;
+  return err;
+}
+
+svn_error_t *
+svn_swig_py_auth_ssl_client_cert_pw_prompt_func(
+    svn_auth_cred_ssl_client_cert_pw_t **cred,
+    void *baton,
+    const char *realm,
+    svn_boolean_t may_save,
+    apr_pool_t *pool)
+{
+  PyObject *function = baton;
+  PyObject *result;
+  svn_auth_cred_ssl_client_cert_pw_t *creds = NULL;
+  svn_error_t *err = SVN_NO_ERROR;
+
+  if ((function == NULL) || (function == Py_None))
+    return SVN_NO_ERROR;
+
+  svn_swig_py_acquire_py_lock();
+
+  if ((result = PyObject_CallFunction(function, 
+                                      (char *)"slO&", 
+                                      realm, may_save, 
+                                      make_ob_pool, pool)) == NULL)
+    {
+      err = callback_exception_error();
+    }
+  else
+    {
+      if (result != Py_None)
+        {
+          svn_auth_cred_ssl_client_cert_pw_t *tmp_creds = NULL;
+          if (svn_swig_ConvertPtrString
+              (result, (void **)&tmp_creds, 
+               "svn_auth_cred_ssl_client_cert_pw_t *"))
+            {
+              err = type_conversion_error
+                ("svn_auth_cred_ssl_client_cert_pw_t *");
+            }
+          else
+            {
+              creds = apr_pcalloc(pool, sizeof (*creds));
+              creds->password = tmp_creds->password ? \
+                apr_pstrdup(pool, tmp_creds->password) : NULL;
+              creds->may_save = tmp_creds->may_save;
+            }
+        }
+      Py_DECREF(result);
+    }
+  svn_swig_py_release_py_lock();
+  *cred = creds;
+  return err;
+}
