@@ -41,7 +41,6 @@ svn_cl__add (apr_getopt_t *os,
 {
   svn_cl__opt_state_t *opt_state = ((svn_cl__cmd_baton_t *) baton)->opt_state;
   svn_client_ctx_t *ctx = ((svn_cl__cmd_baton_t *) baton)->ctx;
-  svn_error_t *err;
   apr_array_header_t *targets;
   int i;
   apr_pool_t *subpool;
@@ -63,18 +62,14 @@ svn_cl__add (apr_getopt_t *os,
 
       svn_pool_clear (subpool);
       SVN_ERR (svn_cl__check_cancel (ctx->cancel_baton));
-      err = svn_client_add2 (target, (! opt_state->nonrecursive), 
-                             opt_state->force, ctx, subpool);
-      if (err)
-        {
-          if (err->apr_err == SVN_ERR_ENTRY_EXISTS)
-            {
-              svn_handle_warning (stderr, err);
-              svn_error_clear (err);
-            }
-          else
-            return err;
-        }
+      SVN_ERR (svn_cl__try 
+               (svn_client_add3 (target, (! opt_state->nonrecursive), 
+                                 opt_state->force, opt_state->no_ignore,
+                                 ctx, subpool),
+                NULL, opt_state->quiet,
+                SVN_ERR_ENTRY_EXISTS,
+                SVN_ERR_WC_PATH_NOT_FOUND,
+                SVN_NO_ERROR));
     }
 
   svn_pool_destroy (subpool);
