@@ -141,10 +141,12 @@ const apr_getopt_option_t svn_cl__options[] =
                       N_("disable automatic properties")},
     {"native-eol",    svn_cl__native_eol_opt, 1,
                       N_("use a different EOL marker than the standard\n"
-                      "                             system marker for files "
-                      "with a native svn:eol-style\n"
-                      "                             property.  ARG may be one "
-                      "of 'LF', 'CR', 'CRLF'")},
+                         "                             "
+                         "system marker for files with the svn:eol-style\n"
+                         "                             "
+                         "property set to 'native'.\n"
+                         "                             "
+                         "ARG may be one of 'LF', 'CR', 'CRLF'")},
     {"limit",         svn_cl__limit_opt, 1,
                       N_("maximum number of log entries")},
     {"no-unlock",     svn_cl__no_unlock_opt, 0,
@@ -170,7 +172,15 @@ const apr_getopt_option_t svn_cl__options[] =
                              svn_cl__auth_password_opt, \
                              svn_cl__no_auth_cache_opt, \
                              svn_cl__non_interactive_opt
-/* Options for giving a log message.  (Some of these also have other uses.) */
+/* Options for giving a log message.  (Some of these also have other uses.) 
+ *
+ * In theory, we should include svn_cl__non_interactive_opt here too,
+ * because all the log-message-taking commands have the potential to
+ * pop up an editor, and svn_cl__non_interactive_opt is the way to
+ * prevent that.  But so far, any command that includes these options
+ * also includes SVN_CL__AUTH_OPTIONS, which of course already
+ * contains svn_cl__non_interactive_opt, so we get it for free.
+ */
 #define SVN_CL__LOG_MSG_OPTIONS 'm', 'F', \
                                 svn_cl__force_log_opt, \
                                 svn_cl__editor_cmd_opt, \
@@ -183,7 +193,8 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "them for addition to repository.  They will be added in next commit.\n"
        "usage: add PATH...\n"),
     {svn_cl__targets_opt, 'N', 'q', svn_cl__config_dir_opt,
-     svn_cl__force_opt, svn_cl__autoprops_opt, svn_cl__no_autoprops_opt} },
+     svn_cl__force_opt, svn_cl__no_ignore_opt, svn_cl__autoprops_opt, 
+     svn_cl__no_autoprops_opt} },
 
   { "blame", svn_cl__blame, {"praise", "annotate", "ann"},
     N_("Output the content of specified files or\n"
@@ -192,7 +203,8 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "\n"
        "  If specified, REV determines in which revision the target is first\n"
        "  looked up.\n"),
-    {'r', 'v', SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
+    {'r', 'v', svn_cl__incremental_opt, svn_cl__xml_opt, SVN_CL__AUTH_OPTIONS,
+     svn_cl__config_dir_opt} },
 
   { "cat", svn_cl__cat, {0},
     N_("Output the content of specified files or URLs.\n"
@@ -320,7 +332,7 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "PATH1 is used\n"
        "     for the local directory name. If REV is not specified,"
        " all local\n"
-       "     changes will be preserved, but files not under version "
+       "     changes will be preserved.  Files not under version "
        "control will\n"
        "     not be copied.\n"
        "\n"
@@ -345,10 +357,13 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "usage: import [PATH] URL\n"
        "\n"
        "  Recursively commit a copy of PATH to URL.\n"
-       "  If PATH is omitted '.' is assumed.  Parent directories are created\n"
-       "  as necessary in the repository.\n"),
+       "  If PATH is omitted '.' is assumed.\n"
+       "  Parent directories are created as necessary in the repository.\n"
+       "  If PATH is a directory, the contents of the directory are added\n"
+       "  directly under URL.\n"),
     {'q', 'N', svn_cl__autoprops_opt, svn_cl__no_autoprops_opt,
-     SVN_CL__LOG_MSG_OPTIONS, SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
+     SVN_CL__LOG_MSG_OPTIONS, svn_cl__no_ignore_opt, SVN_CL__AUTH_OPTIONS, 
+     svn_cl__config_dir_opt} },
  
   { "info", svn_cl__info, {0},
     N_("Display information about a local or remote item.\n"
@@ -356,8 +371,8 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "\n"
        "  Print information about each TARGET (default: '.')\n"
        "  TARGET may be either a working-copy path or URL.\n"),
-    {'r', 'R', svn_cl__targets_opt, SVN_CL__AUTH_OPTIONS, 
-     svn_cl__config_dir_opt} },
+    {'r', 'R', svn_cl__targets_opt, svn_cl__incremental_opt, svn_cl__xml_opt,
+     SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
  
   { "list", svn_cl__ls, {"ls"},
     N_("List directory entries in the repository.\n"
@@ -374,10 +389,11 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
        "current\n"
        "  working directory.\n"
        "\n"
-       "  With --verbose, the following fields show the status of the item:\n"
+       "  With --verbose, the following fields will be shown for each item:\n"
        "\n"
        "    Revision number of the last commit\n"
        "    Author of the last commit\n"
+       "    If locked, the letter 'O'.  (Use 'svn info URL' to see details)\n"
        "    Size (in bytes)\n"
        "    Date and time of the last commit\n"),
     {'r', 'v', 'R', svn_cl__incremental_opt, svn_cl__xml_opt,
@@ -487,34 +503,37 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
      SVN_CL__LOG_MSG_OPTIONS, SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt} },
   
   { "propdel", svn_cl__propdel, {"pdel", "pd"},
-    N_("Remove PROPNAME from files, dirs, or revisions.\n"
+    N_("Remove a property from files, dirs, or revisions.\n"
        "usage: 1. propdel PROPNAME [PATH...]\n"
-       "       2. propdel PROPNAME --revprop -r REV [URL]\n"
+       "       2. propdel PROPNAME --revprop -r REV [TARGET]\n"
        "\n"
        "  1. Removes versioned props in working copy.\n"
-       "  2. Removes unversioned remote prop on repos revision.\n"),
+       "  2. Removes unversioned remote prop on repos revision.\n"
+       "     TARGET only determines which repository to access.\n"),
     {'q', 'R', 'r', svn_cl__revprop_opt, SVN_CL__AUTH_OPTIONS,
      svn_cl__config_dir_opt} },
   
   { "propedit", svn_cl__propedit, {"pedit", "pe"},
-    N_("Edit property PROPNAME with an external editor on targets.\n"
+    N_("Edit a property with an external editor.\n"
        "usage: 1. propedit PROPNAME PATH...\n"
-       "       2. propedit PROPNAME --revprop -r REV [URL]\n"
+       "       2. propedit PROPNAME --revprop -r REV [TARGET]\n"
        "\n"
        "  1. Edits versioned props in working copy.\n"
-       "  2. Edits unversioned remote prop on repos revision.\n"),
+       "  2. Edits unversioned remote prop on repos revision.\n"
+       "     TARGET only determines which repository to access.\n"),
     {'r', svn_cl__revprop_opt, SVN_CL__AUTH_OPTIONS,
      svn_cl__encoding_opt, svn_cl__editor_cmd_opt, svn_cl__force_opt,
      svn_cl__config_dir_opt} },
   
   { "propget", svn_cl__propget, {"pget", "pg"},
-    N_("Print value of PROPNAME on files, dirs, or revisions.\n"
+    N_("Print the value of a property on files, dirs, or revisions.\n"
        "usage: 1. propget PROPNAME [TARGET[@REV]...]\n"
-       "       2. propget PROPNAME --revprop -r REV [URL]\n"
+       "       2. propget PROPNAME --revprop -r REV [TARGET]\n"
        "\n"
-       "  1. Prints versioned prop. If specified, REV determines in which\n"
+       "  1. Prints versioned props. If specified, REV determines in which\n"
        "     revision the target is first looked up.\n"
        "  2. Prints unversioned remote prop on repos revision.\n"
+       "     TARGET only determines which repository to access.\n"
        "\n"
        "  By default, this subcommand will add an extra newline to the end\n"
        "  of the property values so that the output looks pretty.  Also,\n"
@@ -528,22 +547,26 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
   { "proplist", svn_cl__proplist, {"plist", "pl"},
     N_("List all properties on files, dirs, or revisions.\n"
        "usage: 1. proplist [TARGET[@REV]...]\n"
-       "       2. proplist --revprop -r REV [URL]\n"
+       "       2. proplist --revprop -r REV [TARGET]\n"
        "\n"
        "  1. Lists versioned props. If specified, REV determines in which\n"
        "     revision the target is first looked up.\n"
-       "  2. Lists unversioned remote props on repos revision.\n"),
+       "  2. Lists unversioned remote props on repos revision.\n"
+       "     TARGET only determines which repository to access.\n"),
     {'v', 'R', 'r', 'q', svn_cl__revprop_opt, SVN_CL__AUTH_OPTIONS,
      svn_cl__config_dir_opt} },
 
   { "propset", svn_cl__propset, {"pset", "ps"},
-    N_("Set PROPNAME to PROPVAL on files, dirs, or revisions.\n"
-       "usage: 1. propset PROPNAME [PROPVAL | -F VALFILE] PATH...\n"
-       "       2. propset PROPNAME --revprop -r REV [PROPVAL | -F VALFILE] "
-       "[URL]\n"
+    N_("Set the value of a property on files, dirs, or revisions.\n"
+       "usage: 1. propset PROPNAME PROPVAL PATH...\n"
+       "       2. propset PROPNAME --revprop -r REV PROPVAL [TARGET]\n"
        "\n"
        "  1. Creates a versioned, local propchange in working copy.\n"
        "  2. Creates an unversioned, remote propchange on repos revision.\n"
+       "     TARGET only determines which repository to access.\n"
+       "\n"
+       "  The value may be provided with the --file option instead of "
+       "PROPVAL.\n"
        "\n"
        "  Note: svn recognizes the following special versioned properties\n"
        "  but will store any arbitrary properties set:\n"
@@ -620,7 +643,6 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
      "      'A' Added\n"
      "      'C' Conflicted\n"
      "      'D' Deleted\n"
-     "      'G' Merged\n"
      "      'I' Ignored\n"
      "      'M' Modified\n"
      "      'R' Replaced\n"
@@ -679,8 +701,9 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
      "    A  +         965       687 joe          wc/qax.c\n"
      "                 965       687 joe          wc/zig.c\n"
      "    Status against revision:   981\n"),
-    { 'u', 'v', 'N', 'q', svn_cl__no_ignore_opt, SVN_CL__AUTH_OPTIONS, 
-      svn_cl__config_dir_opt, svn_cl__ignore_externals_opt} },
+    { 'u', 'v', 'N', 'q', svn_cl__no_ignore_opt, svn_cl__incremental_opt,
+      svn_cl__xml_opt, SVN_CL__AUTH_OPTIONS, svn_cl__config_dir_opt,
+      svn_cl__ignore_externals_opt} },
   
   { "switch", svn_cl__switch, {"sw"},
     N_("Update the working copy to a different URL.\n"
@@ -742,18 +765,6 @@ const svn_opt_subcommand_desc_t svn_cl__cmd_table[] =
 };
 
 
-/* Standard error handler */
-static int
-error_exit (svn_error_t *err, FILE *stream, svn_boolean_t fatal,
-            apr_pool_t *pool)
-{
-  svn_handle_error (err, stderr, fatal);
-  svn_error_clear (err);
-  svn_pool_destroy (pool);
-  return EXIT_FAILURE;
-}
-
-
 /* Version compatibility check */
 static svn_error_t *
 check_lib_versions (void)
@@ -809,8 +820,8 @@ main (int argc, const char * const *argv)
   apr_getopt_t *os;  
   svn_cl__opt_state_t opt_state = { { 0 } };
   svn_client_ctx_t *ctx;
-  int received_opts[SVN_OPT_MAX_OPTIONS];
-  int i, num_opts = 0;
+  apr_array_header_t *received_opts;
+  int i;
   const svn_opt_subcommand_desc_t *subcommand = NULL;
   const char *dash_m_arg = NULL, *dash_F_arg = NULL;
   const char *path_utf8;
@@ -834,15 +845,27 @@ main (int argc, const char * const *argv)
   pool = svn_pool_create_ex (NULL, allocator);
   apr_allocator_owner_set (allocator, pool);
 
+  received_opts = apr_array_make (pool, SVN_OPT_MAX_OPTIONS, sizeof (int));
+
   /* Check library versions */
   err = check_lib_versions ();
   if (err)
-    return error_exit (err, stderr, FALSE, pool);
+    return svn_cmdline_handle_exit_error (err, pool, "svn: ");
+
+#if defined(WIN32) || defined(__CYGWIN__)
+  /* Set the working copy administrative directory name. */
+  if (getenv ("SVN_ASP_DOT_NET_HACK"))
+    {
+      err = svn_wc_set_adm_dir ("_svn", pool);
+      if (err)
+        return svn_cmdline_handle_exit_error (err, pool, "svn: ");
+    }
+#endif
 
   /* Initialize the RA library. */
   err = svn_ra_initialize (pool);
   if (err)
-    return error_exit (err, stderr, FALSE, pool);
+    return svn_cmdline_handle_exit_error (err, pool, "svn: ");
 
   /* Begin processing arguments. */
   opt_state.start_revision.kind = svn_opt_revision_unspecified;
@@ -876,8 +899,7 @@ main (int argc, const char * const *argv)
         }
 
       /* Stash the option code in an array before parsing it. */
-      received_opts[num_opts] = opt_id;
-      num_opts++;
+      APR_ARRAY_PUSH (received_opts, int) = opt_id;
 
       switch (opt_id) {
       case svn_cl__limit_opt:
@@ -886,13 +908,9 @@ main (int argc, const char * const *argv)
           opt_state.limit = strtol (opt_arg, &end, 10);
           if (end == opt_arg || *end != '\0')
             {
-              /* ### See r14699 on trunk for the error message that
-                     should be used here.  We removed it for 1.2.x in
-                     order to avoid slipping an untranslated string
-                     into the release at the last minute. */
-              err = svn_error_create (SVN_ERR_CL_ARG_PARSING_ERROR,
-                                      NULL, NULL);
-              return error_exit (err, stderr, FALSE, pool);
+              err = svn_error_create (SVN_ERR_CL_ARG_PARSING_ERROR, NULL,
+                                      _("Non-numeric limit argument given"));
+              return svn_cmdline_handle_exit_error (err, pool, "svn: ");
             }
         }
         break;
@@ -910,7 +928,7 @@ main (int argc, const char * const *argv)
               (SVN_ERR_CL_ARG_PARSING_ERROR, NULL,
                _("Multiple revision arguments encountered; "
                  "try '-r M:N' instead of '-r M -r N'"));
-            return error_exit (err, stderr, FALSE, pool);
+            return svn_cmdline_handle_exit_error (err, pool, "svn: ");
           }
         if (svn_opt_parse_revision (&(opt_state.start_revision),
                                     &(opt_state.end_revision),
@@ -922,7 +940,7 @@ main (int argc, const char * const *argv)
                 (SVN_ERR_CL_ARG_PARSING_ERROR, NULL,
                  _("Syntax error in revision argument '%s'"),
                  utf8_opt_arg);
-            return error_exit (err, stderr, FALSE, pool);
+            return svn_cmdline_handle_exit_error (err, pool, "svn: ");
           }
         break;
       case 'v':
@@ -947,7 +965,7 @@ main (int argc, const char * const *argv)
           err = svn_stringbuf_from_file (&(opt_state.filedata),
                                          utf8_opt_arg, pool);
         if (err)
-          return error_exit (err, stderr, FALSE, pool);
+          return svn_cmdline_handle_exit_error (err, pool, "svn: ");
         dash_F_arg = opt_arg;
         break;
       case svn_cl__targets_opt:
@@ -965,7 +983,7 @@ main (int argc, const char * const *argv)
           if (! err)
             err = svn_utf_stringbuf_to_utf8 (&buffer_utf8, buffer, pool);
           if (err)
-            return error_exit (err, stdout, FALSE, pool);
+            return svn_cmdline_handle_exit_error (err, pool, "svn: ");
           opt_state.targets = svn_cstring_split (buffer_utf8->data, "\n\r",
                                                  TRUE, pool);
         }
@@ -996,13 +1014,13 @@ main (int argc, const char * const *argv)
         err = svn_utf_cstring_to_utf8 (&opt_state.auth_username,
                                        opt_arg, pool);
         if (err)
-          return error_exit (err, stdout, FALSE, pool);
+          return svn_cmdline_handle_exit_error (err, pool, "svn: ");
         break;
       case svn_cl__auth_password_opt:
         err = svn_utf_cstring_to_utf8 (&opt_state.auth_password,
                                        opt_arg, pool);
         if (err)
-          return error_exit (err, stdout, FALSE, pool);
+          return svn_cmdline_handle_exit_error (err, pool, "svn: ");
         break;
       case svn_cl__encoding_opt:
         opt_state.encoding = apr_pstrdup (pool, opt_arg);
@@ -1043,7 +1061,7 @@ main (int argc, const char * const *argv)
       case 'x':
         err = svn_utf_cstring_to_utf8 (&opt_state.extensions, opt_arg, pool);
         if (err)
-          return error_exit (err, stderr, FALSE, pool);
+          return svn_cmdline_handle_exit_error (err, pool, "svn: ");
         break;
       case svn_cl__diff_cmd_opt:
         opt_state.diff_cmd = apr_pstrdup (pool, opt_arg);
@@ -1070,7 +1088,7 @@ main (int argc, const char * const *argv)
             err = svn_error_create (SVN_ERR_CL_MUTUALLY_EXCLUSIVE_ARGS, NULL,
                                     _("--auto-props and --no-auto-props are "
                                       "mutually exclusive"));
-            return error_exit (err, stderr, FALSE, pool);
+            return svn_cmdline_handle_exit_error (err, pool, "svn: ");
           }
         opt_state.autoprops = TRUE;
         break;
@@ -1080,7 +1098,7 @@ main (int argc, const char * const *argv)
             err = svn_error_create (SVN_ERR_CL_MUTUALLY_EXCLUSIVE_ARGS, NULL,
                                     _("--auto-props and --no-auto-props are "
                                       "mutually exclusive"));
-            return error_exit (err, stderr, FALSE, pool);
+            return svn_cmdline_handle_exit_error (err, pool, "svn: ");
           }
         opt_state.no_autoprops = TRUE;
         break;
@@ -1096,11 +1114,12 @@ main (int argc, const char * const *argv)
                 (SVN_ERR_CL_ARG_PARSING_ERROR, NULL,
                  _("Syntax error in native-eol argument '%s'"),
                  utf8_opt_arg);
-            svn_handle_error (err, stderr, FALSE);
+            svn_handle_error2 (err, stderr, FALSE, "svn: ");
             svn_error_clear (err);
             svn_pool_destroy (pool);
             return EXIT_FAILURE;
           }
+        break;
       case svn_cl__no_unlock_opt:
         opt_state.no_unlock = TRUE;
         break;
@@ -1123,7 +1142,7 @@ main (int argc, const char * const *argv)
      init function anyway.  Thoughts?  */
   err = svn_config_ensure (opt_state.config_dir, pool);
   if (err)
-    return error_exit (err, stderr, FALSE, pool);
+    return svn_cmdline_handle_exit_error (err, pool, "svn: ");
 
   /* If the user asked for help, then the rest of the arguments are
      the names of subcommands to get help on (if any), or else they're
@@ -1155,7 +1174,7 @@ main (int argc, const char * const *argv)
               const char *first_arg_utf8;
               err = svn_utf_cstring_to_utf8 (&first_arg_utf8, first_arg, pool);
               if (err)
-                return error_exit (err, stderr, FALSE, pool);
+                return svn_cmdline_handle_exit_error (err, pool, "svn: ");
               svn_error_clear
                 (svn_cmdline_fprintf (stderr, pool,
                                       _("Unknown command: '%s'\n"),
@@ -1168,9 +1187,9 @@ main (int argc, const char * const *argv)
     }
 
   /* Check that the subcommand wasn't passed any inappropriate options. */
-  for (i = 0; i < num_opts; i++)
+  for (i = 0; i < received_opts->nelts; i++)
     {
-      opt_id = received_opts[i];
+      opt_id = APR_ARRAY_IDX (received_opts, i, int);
 
       /* All commands implicitly accept --help, so just skip over this
          when we see it. Note that we don't want to include this option
@@ -1188,7 +1207,7 @@ main (int argc, const char * const *argv)
           if ((err = svn_utf_cstring_to_utf8 (&optstr_utf8, optstr, pool))
               || (err = svn_utf_cstring_to_utf8 (&cmdname_utf8,
                                                  subcommand->name, pool)))
-            return error_exit (err, stderr, FALSE, pool);
+            return svn_cmdline_handle_exit_error (err, pool, "svn: ");
           svn_error_clear
             (svn_cmdline_fprintf
              (stderr, pool, _("Subcommand '%s' doesn't accept option '%s'\n"
@@ -1225,16 +1244,20 @@ main (int argc, const char * const *argv)
           if ((err == SVN_NO_ERROR) && e)
             {
               if (subcommand->cmd_func != svn_cl__lock)
-                err = svn_error_create 
-                  (SVN_ERR_CL_LOG_MESSAGE_IS_VERSIONED_FILE, NULL,
-                   _("Log message file is a versioned file; "
-                     "use '--force-log' to override"));
+                {
+                  err = svn_error_create 
+                    (SVN_ERR_CL_LOG_MESSAGE_IS_VERSIONED_FILE, NULL,
+                     _("Log message file is a versioned file; "
+                       "use '--force-log' to override"));
+                }
               else
-                err = svn_error_create 
-                  (SVN_ERR_CL_LOG_MESSAGE_IS_VERSIONED_FILE, NULL,
-                   _("Lock comment file is a versioned file; "
-                     "use '--force-log' to override"));
-              return error_exit (err, stderr, FALSE, pool);
+                {
+                  err = svn_error_create 
+                    (SVN_ERR_CL_LOG_MESSAGE_IS_VERSIONED_FILE, NULL,
+                     _("Lock comment file is a versioned file; "
+                       "use '--force-log' to override"));
+                }
+              return svn_cmdline_handle_exit_error (err, pool, "svn: ");
             }
           if (err)
             svn_error_clear (err);
@@ -1249,16 +1272,20 @@ main (int argc, const char * const *argv)
                         APR_FINFO_MIN, pool) == APR_SUCCESS)
             {
               if (subcommand->cmd_func != svn_cl__lock)
-                err = svn_error_create 
-                  (SVN_ERR_CL_LOG_MESSAGE_IS_PATHNAME, NULL,
-                   _("The log message is a pathname "
-                     "(was -F intended?); use '--force-log' to override"));
+                {
+                  err = svn_error_create 
+                    (SVN_ERR_CL_LOG_MESSAGE_IS_PATHNAME, NULL,
+                     _("The log message is a pathname "
+                       "(was -F intended?); use '--force-log' to override"));
+                }
               else
-                err = svn_error_create 
-                  (SVN_ERR_CL_LOG_MESSAGE_IS_PATHNAME, NULL,
-                   _("The lock comment is a pathname "
-                     "(was -F intended?); use '--force-log' to override"));
-              return error_exit (err, stderr, FALSE, pool);
+                {
+                  err = svn_error_create 
+                    (SVN_ERR_CL_LOG_MESSAGE_IS_PATHNAME, NULL,
+                     _("The lock comment is a pathname "
+                       "(was -F intended?); use '--force-log' to override"));
+                  return svn_cmdline_handle_exit_error (err, pool, "svn: ");
+                }
             }
         }
     }
@@ -1273,23 +1300,23 @@ main (int argc, const char * const *argv)
       if (opt_state.end_revision.kind != svn_opt_revision_unspecified)
         {
           err = svn_error_create (SVN_ERR_CLIENT_REVISION_RANGE, NULL, NULL);
-          return error_exit (err, stderr, FALSE, pool);
+          return svn_cmdline_handle_exit_error (err, pool, "svn: ");
         }
     }
 
   /* Create a client context object. */
   command_baton.opt_state = &opt_state;
   if ((err = svn_client_create_context (&ctx, pool)))
-    return error_exit (err, stderr, FALSE, pool);
+    return svn_cmdline_handle_exit_error (err, pool, "svn: ");
   command_baton.ctx = ctx;
 
   if ((err = svn_config_get_config (&(ctx->config),
                                     opt_state.config_dir, pool)))
-    return error_exit (err, stderr, FALSE, pool);
+    return svn_cmdline_handle_exit_error (err, pool, "svn: ");
 
   cfg = apr_hash_get (ctx->config, SVN_CONFIG_CATEGORY_CONFIG,
                       APR_HASH_KEY_STRING);
-  
+
   /* Update the options in the config */
   /* XXX: Only diff_cmd for now, overlay rest later and stop passing
      opt_state altogether? */
@@ -1322,8 +1349,8 @@ main (int argc, const char * const *argv)
                          SVN_CONFIG_OPTION_NO_UNLOCK, TRUE);
 
   /* Set the log message callback function.  Note that individual
-     subcommands will populate the ctx->log_msg_baton */
-  ctx->log_msg_func = svn_cl__get_log_message;
+     subcommands will populate the ctx->log_msg_baton2 */
+  ctx->log_msg_func2 = svn_cl__get_log_message;
 
   /* Authentication set-up. */
   {
@@ -1332,7 +1359,7 @@ main (int argc, const char * const *argv)
 
     /* The whole list of registered providers */
     apr_array_header_t *providers
-      = apr_array_make (pool, 10, sizeof (svn_auth_provider_object_t *));
+      = apr_array_make (pool, 11, sizeof (svn_auth_provider_object_t *));
 
     /* The main disk-caching auth providers, for both
        'username/password' creds and 'username' creds.  */
@@ -1410,7 +1437,7 @@ main (int argc, const char * const *argv)
                                     SVN_CONFIG_SECTION_AUTH,
                                     SVN_CONFIG_OPTION_STORE_PASSWORDS,
                                     TRUE)))
-      svn_handle_error (err, stderr, TRUE);
+      svn_handle_error2 (err, stderr, TRUE, "svn: ");
     if (! store_password_val)
       svn_auth_set_parameter(ab, SVN_AUTH_PARAM_DONT_STORE_PASSWORDS, "");
 
@@ -1421,7 +1448,7 @@ main (int argc, const char * const *argv)
                                     SVN_CONFIG_SECTION_AUTH,
                                     SVN_CONFIG_OPTION_STORE_AUTH_CREDS,
                                     TRUE)))
-      svn_handle_error (err, stderr, TRUE);
+      svn_handle_error2 (err, stderr, TRUE, "svn: ");
     if (opt_state.no_auth_cache || ! store_password_val)
       svn_auth_set_parameter(ab, SVN_AUTH_PARAM_NO_AUTH_CACHE, "");
   }
@@ -1460,7 +1487,7 @@ main (int argc, const char * const *argv)
         svn_opt_subcommand_help (subcommand->name, svn_cl__cmd_table,
                                  svn_cl__options, pool);
       else
-        svn_handle_error (err, stderr, FALSE);
+        svn_handle_error2 (err, stderr, FALSE, "svn: ");
 
       /* Tell the user about 'svn cleanup' if any error on the stack
          was about locked working copies. */
@@ -1484,7 +1511,7 @@ main (int argc, const char * const *argv)
          This makes sure that output is not silently lost. */
       err = svn_cmdline_fflush (stdout);
       if (err)
-        return error_exit (err, stdout, FALSE, pool);
+        return svn_cmdline_handle_exit_error (err, pool, "svn: ");
 
       svn_pool_destroy (pool);
       return EXIT_SUCCESS;
