@@ -28,6 +28,7 @@
 #include <ctype.h>
 #include <assert.h>
 
+#include <apr_errno.h>
 #include <apr_strings.h>
 #include <apr_tables.h>
 #include <apr_general.h>
@@ -42,6 +43,7 @@
 #include "svn_utf.h"
 #include "svn_subst.h"
 #include "svn_config.h"
+#include "svn_xml.h"
 #include "svn_private_config.h"
 #include "cl.h"
 
@@ -49,7 +51,7 @@
 
 
 svn_error_t *
-svn_cl__print_commit_info (svn_client_commit_info_t *commit_info,
+svn_cl__print_commit_info (svn_commit_info_t *commit_info,
                            apr_pool_t *pool)
 {
   if ((commit_info) 
@@ -129,9 +131,9 @@ svn_cl__edit_externally (svn_string_t **edited_contents /* UTF-8! */,
   if (as_text)
     {
       const char *translated;
-      SVN_ERR (svn_subst_translate_cstring (contents->data, &translated,
-                                            APR_EOL_STR, FALSE,
-                                            NULL, FALSE, pool));
+      SVN_ERR (svn_subst_translate_cstring2 (contents->data, &translated,
+                                             APR_EOL_STR, FALSE,
+                                             NULL, FALSE, pool));
       translated_contents = svn_string_create ("", pool);
       if (encoding)
         SVN_ERR (svn_utf_cstring_from_utf8_ex (&translated_contents->data,
@@ -169,6 +171,27 @@ svn_cl__edit_externally (svn_string_t **edited_contents /* UTF-8! */,
      PREFIX. */
   err = svn_io_open_unique_file (&tmp_file, &tmpfile_name,
                                  prefix, ".tmp", FALSE, pool);
+
+  if (err && APR_STATUS_IS_EACCES (err->apr_err))
+    {
+      const char *temp_dir_apr;
+
+      svn_error_clear (err);
+
+      SVN_ERR (svn_io_temp_dir (&base_dir, pool));
+
+      SVN_ERR (svn_path_cstring_from_utf8 (&temp_dir_apr, base_dir, pool));
+      apr_err = apr_filepath_set (temp_dir_apr, pool);
+      if (apr_err)
+        {
+          return svn_error_wrap_apr
+            (apr_err, _("Can't change working directory to '%s'"), base_dir);
+        }
+
+      err = svn_io_open_unique_file (&tmp_file, &tmpfile_name,
+                                     prefix, ".tmp", FALSE, pool);
+    }
+
   if (err)
     goto cleanup2;
 
@@ -272,6 +295,8 @@ svn_cl__edit_externally (svn_string_t **edited_contents /* UTF-8! */,
       /* Only report remove error if there was no previous error. */
       if (! err && err2)
         err = err2;
+      else
+        svn_error_clear (err2);
     }
 
  cleanup2:
@@ -280,9 +305,9 @@ svn_cl__edit_externally (svn_string_t **edited_contents /* UTF-8! */,
   apr_err = apr_filepath_set (old_cwd, pool);
   if (apr_err)
     {
-      svn_handle_error (svn_error_wrap_apr
-                        (apr_err, _("Can't restore working directory")),
-                        stderr, TRUE /* fatal */);
+      svn_handle_error2 (svn_error_wrap_apr
+                         (apr_err, _("Can't restore working directory")),
+                         stderr, TRUE /* fatal */, "svn: ");
     }
 
   return err;
@@ -296,6 +321,7 @@ struct log_msg_baton
   const char *message_encoding; /* the locale/encoding of the message. */
   const char *base_dir; /* the base directory for an external edit. UTF-8! */
   const char *tmpfile_left; /* the tmpfile left by an external edit. UTF-8! */
+  svn_boolean_t non_interactive; /* if true, don't pop up an editor */
   apr_hash_t *config; /* client configuration hash */
   svn_boolean_t keep_locks; /* Keep repository locks? */
   apr_pool_t *pool; /* a pool. */
@@ -348,6 +374,7 @@ svn_cl__make_log_msg_baton (void **baton,
   lmb->tmpfile_left = NULL;
   lmb->config = config;
   lmb->keep_locks = opt_state->no_unlock;
+  lmb->non_interactive = opt_state->non_interactive;
   lmb->pool = pool;
   *baton = lmb;
   return SVN_NO_ERROR;
@@ -427,17 +454,17 @@ truncate_buffer_at_prefix (apr_size_t *new_len,
         }
     }
 
-  return;
+  /* NOTREACHED */
 }
 
 
 #define EDITOR_EOF_PREFIX  _("--This line, and those below, will be ignored--")
 
-/* This function is of type svn_client_get_commit_log_t. */
+/* This function is of type svn_client_get_commit_log2_t. */
 svn_error_t *
 svn_cl__get_log_message (const char **log_msg,
                          const char **tmp_file,
-                         apr_array_header_t *commit_items,
+                         const apr_array_header_t *commit_items,
                          void *baton,
                          apr_pool_t *pool)
 {
@@ -453,10 +480,7 @@ svn_cl__get_log_message (const char **log_msg,
   *tmp_file = NULL;
   if (lmb->message)
     {
-      svn_string_t *log_msg_string = svn_string_create ("", pool);
-
-      log_msg_string->data = lmb->message;
-      log_msg_string->len = strlen (lmb->message);
+      svn_string_t *log_msg_string = svn_string_create (lmb->message, pool);
 
       SVN_ERR (svn_subst_translate_string (&log_msg_string, log_msg_string,
                                            lmb->message_encoding, pool));
@@ -488,8 +512,8 @@ svn_cl__get_log_message (const char **log_msg,
 
       for (i = 0; i < commit_items->nelts; i++)
         {
-          svn_client_commit_item_t *item
-            = ((svn_client_commit_item_t **) commit_items->elts)[i];
+          svn_client_commit_item2_t *item
+            = APR_ARRAY_IDX(commit_items, i, svn_client_commit_item2_t *);
           const char *path = item->path;
           char text_mod = '_', prop_mod = ' ', unlock = ' ';
 
@@ -534,11 +558,22 @@ svn_cl__get_log_message (const char **log_msg,
       msg_string->len = tmp_message->len;
 
       /* Use the external edit to get a log message. */
-      err = svn_cl__edit_externally (&msg_string, &lmb->tmpfile_left,
-                                     lmb->editor_cmd, lmb->base_dir,
-                                     msg_string, "svn-commit",
-                                     lmb->config, TRUE, lmb->message_encoding,
-                                     pool);
+      if (! lmb->non_interactive)
+        {
+          err = svn_cl__edit_externally (&msg_string, &lmb->tmpfile_left,
+                                         lmb->editor_cmd, lmb->base_dir,
+                                         msg_string, "svn-commit",
+                                         lmb->config, TRUE,
+                                         lmb->message_encoding,
+                                         pool);
+        }
+      else /* non_interactive flag says we can't pop up an editor, so error */
+        {
+          return svn_error_create
+            (SVN_ERR_CL_INSUFFICIENT_ARGS, NULL,
+             _("Cannot invoke editor to get log message "
+               "when non-interactive"));
+        }
 
       /* Dup the tmpfile path into its baton's pool. */
       *tmp_file = lmb->tmpfile_left = apr_pstrdup (lmb->pool, 
@@ -666,3 +701,70 @@ svn_cl__error_checked_fputs (const char *string, FILE* stream)
 
   return SVN_NO_ERROR;
 }
+
+
+svn_error_t *
+svn_cl__try (svn_error_t *err,
+             svn_boolean_t *success,
+             svn_boolean_t quiet,
+             ...)
+{
+  if (err)
+    {
+      apr_status_t apr_err;
+      va_list ap;
+
+      if (success)
+        *success = FALSE;
+
+      va_start (ap, quiet);
+      while ((apr_err = va_arg (ap, apr_status_t)) != SVN_NO_ERROR)
+        {
+          if (err->apr_err == apr_err)
+            {
+              if (! quiet)
+                svn_handle_warning (stderr, err);
+              svn_error_clear (err);
+              return SVN_NO_ERROR;
+            }
+        }
+      va_end (ap);
+    }
+  else if (success)
+    {
+      *success = TRUE;
+    }
+
+  return err;
+}
+
+
+void
+svn_cl__xml_tagged_cdata (svn_stringbuf_t **sb,
+                          apr_pool_t *pool,
+                          const char *tagname,
+                          const char *string)
+{
+  if (string)
+    {
+      svn_xml_make_open_tag (sb, pool, svn_xml_protect_pcdata,
+                             tagname, NULL);
+      svn_xml_escape_cdata_cstring (sb, string, pool);
+      svn_xml_make_close_tag (sb, pool, tagname);
+    }
+}
+
+
+const char *
+svn_cl__node_kind_str (svn_node_kind_t kind)
+{
+  switch (kind)
+    {
+    case svn_node_dir:
+      return "dir";
+    case svn_node_file:
+      return "file";
+    default:
+      return "";
+    }
+}
