@@ -159,6 +159,8 @@ store_locks_callback (void *baton,
  *
  * If *COMMON_PARENT is a URL, then the values are a pointer to
  * SVN_INVALID_REVNUM (allocated in pool) if DO_LOCK, else "".
+ *
+ * TARGETS may not be empty.
  */
 static svn_error_t *
 organize_lock_targets (const char **common_parent,
@@ -210,7 +212,8 @@ organize_lock_targets (const char **common_parent,
           const char *target = ((const char **) (rel_targets->elts))[i];
           apr_hash_set (rel_targets_ret, svn_path_uri_decode (target, pool),
                         APR_HASH_KEY_STRING,
-                        do_lock ? (void *) invalid_revnum : (void *) "");
+                        do_lock ? (const void *) invalid_revnum
+                                : (const void *) "");
         }
       *rel_fs_paths_p = NULL;
     }
@@ -412,6 +415,9 @@ svn_client_lock (const apr_array_header_t *targets,
   struct lock_baton cb;
   svn_boolean_t is_url;
 
+  if (apr_is_empty_array (targets))
+    return SVN_NO_ERROR;
+
   /* Enforce that the comment be xml-escapable. */
   if (comment)
     {
@@ -433,7 +439,7 @@ svn_client_lock (const apr_array_header_t *targets,
     url = entry->url;
 
   /* Open an RA session to the common parent of TARGETS. */
-  SVN_ERR (svn_client__open_ra_session
+  SVN_ERR (svn_client__open_ra_session_internal
            (&ra_session, url, is_url ? NULL : common_parent,
             adm_access, NULL, FALSE, FALSE, ctx, pool));
 
@@ -468,6 +474,9 @@ svn_client_unlock (const apr_array_header_t *targets,
   struct lock_baton cb;
   svn_boolean_t is_url;
 
+  if (apr_is_empty_array (targets))
+    return SVN_NO_ERROR;
+
   SVN_ERR (organize_lock_targets (&common_parent, &entry, &adm_access,
                                   &path_tokens, &urls_to_paths, targets, 
                                   FALSE, break_lock, ctx, pool));
@@ -480,7 +489,7 @@ svn_client_unlock (const apr_array_header_t *targets,
     url = entry->url;
 
   /* Open an RA session. */
-  SVN_ERR (svn_client__open_ra_session
+  SVN_ERR (svn_client__open_ra_session_internal
            (&ra_session, url,
             svn_path_is_url (common_parent) ? NULL : common_parent,
             adm_access, NULL, FALSE, FALSE, ctx, pool));
