@@ -18,13 +18,13 @@
 #include <string.h>
 #include <assert.h>
 
-#include "svn_pools.h"
 #include "svn_path.h"
 #include "svn_time.h"
 #include "svn_error.h"
-#include "svn_md5.h"
 #include "svn_fs.h"
+#include "svn_props.h"
 #include "svn_pools.h"
+#include "svn_md5.h"
 
 #include "dag.h"
 #include "err.h"
@@ -36,6 +36,8 @@
 
 #include "../libsvn_fs/fs-loader.h"
 
+#include "svn_private_config.h"
+
 
 /* Initializing a filesystem.  */
 
@@ -272,8 +274,8 @@ svn_fs_fs__dag_walk_predecessors (dag_node_t *node,
       if (callback)
         SVN_ERR (callback (baton, this_node, &done, iterpool));
     }
-  apr_pool_destroy (iterpool);
-  apr_pool_destroy (last_iterpool);
+  svn_pool_destroy (iterpool);
+  svn_pool_destroy (last_iterpool);
 
   return SVN_NO_ERROR;
 }
@@ -375,19 +377,19 @@ make_entry (dag_node_t **child_p,
   if (! svn_path_is_single_path_component (name))
     return svn_error_createf
       (SVN_ERR_FS_NOT_SINGLE_PATH_COMPONENT, NULL,
-       "Attempted to create a node with an illegal name '%s'", name);
+       _("Attempted to create a node with an illegal name '%s'"), name);
 
   /* Make sure that parent is a directory */
   if (parent->kind != svn_node_dir)
     return svn_error_create
       (SVN_ERR_FS_NOT_DIRECTORY, NULL,
-       "Attempted to create entry in non-directory parent");
+       _("Attempted to create entry in non-directory parent"));
 
   /* Check that the parent is mutable. */
   if (! svn_fs_fs__dag_check_mutable (parent, txn_id))
     return svn_error_createf
       (SVN_ERR_FS_NOT_MUTABLE, NULL,
-       "Attempted to clone child of non-mutable node");
+       _("Attempted to clone child of non-mutable node"));
 
   /* Create the new node's NODE-REVISION */
   memset (&new_noderev, 0, sizeof (new_noderev));
@@ -431,7 +433,7 @@ svn_fs_fs__dag_dir_entries (apr_hash_t **entries,
 
   if (noderev->kind != svn_node_dir)
     return svn_error_create (SVN_ERR_FS_NOT_DIRECTORY, NULL,
-                             "Can't get entries of non-directory");
+                             _("Can't get entries of non-directory"));
 
   return svn_fs_fs__rep_contents_dir (entries, node->fs, noderev, pool);
 }
@@ -449,13 +451,13 @@ svn_fs_fs__dag_set_entry (dag_node_t *node,
   if (node->kind != svn_node_dir)
     return svn_error_create
       (SVN_ERR_FS_NOT_DIRECTORY, NULL,
-       "Attempted to set entry in non-directory node");
+       _("Attempted to set entry in non-directory node"));
   
   /* Check it's mutable. */
   if (! svn_fs_fs__dag_check_mutable (node, txn_id))
     return svn_error_create
       (SVN_ERR_FS_NOT_DIRECTORY, NULL,
-       "Attempted to set entry in immutable node");
+       _("Attempted to set entry in immutable node"));
 
   return set_entry (node, entry_name, id, kind, txn_id, pool);
 }
@@ -938,8 +940,21 @@ svn_fs_fs__dag_finalize_edits (dag_node_t *file,
                                const char *txn_id, 
                                apr_pool_t *pool)
 {
-  /* A big no-op for FSFS. */
-  
+  unsigned char digest[APR_MD5_DIGESTSIZE];
+  const char *hex;
+
+  if (checksum)
+    {
+      SVN_ERR (svn_fs_fs__dag_file_checksum (digest, file, pool));
+      hex = svn_md5_digest_to_cstring (digest, pool);
+      if (hex && strcmp (checksum, hex) != 0)
+        return svn_error_createf (SVN_ERR_CHECKSUM_MISMATCH, NULL,
+                                  _("Checksum mismatch, file '%s':\n"
+                                    "   expected:  %s\n"
+                                    "     actual:  %s\n"),
+                                  file->created_path, checksum, hex);
+    }
+
   return SVN_NO_ERROR;
 }
 
@@ -1017,7 +1032,7 @@ svn_fs_fs__dag_copy (dag_node_t *to_node,
       /* Reserve a copy ID for this new copy. */
       SVN_ERR (svn_fs_fs__reserve_copy_id (&copy_id, fs, txn_id, pool));
 
-      /* Create a successor with it's predecessor pointing at the copy
+      /* Create a successor with its predecessor pointing at the copy
          source. */
       to_noderev->predecessor_id = svn_fs_fs__id_copy (src_id, pool);
       if (to_noderev->predecessor_count != -1)
