@@ -34,6 +34,7 @@
 #include "svn_error.h"
 #include "svn_pools.h"
 #include "svn_ra_svn.h"
+#include "svn_private_config.h"
 
 #include "ra_svn.h"
 
@@ -74,7 +75,7 @@ svn_error_t *svn_ra_svn_set_capabilities(svn_ra_svn_conn_t *conn,
       item = &APR_ARRAY_IDX(list, i, svn_ra_svn_item_t);
       if (item->kind != SVN_RA_SVN_WORD)
         return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
-                                "Capability entry is not a word");
+                                _("Capability entry is not a word"));
       word = apr_pstrdup(conn->pool, item->u.word);
       apr_hash_set(conn->capabilities, word, APR_HASH_KEY_STRING, word);
     }
@@ -156,7 +157,7 @@ static svn_error_t *writebuf_output(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
       else
         status = apr_file_write(conn->out_file, data, &count);
       if (status)
-        return svn_error_wrap_apr(status, "Can't write to connection");
+        return svn_error_wrap_apr(status, _("Can't write to connection"));
       if (count == 0)
         {
           if (!subpool)
@@ -203,6 +204,9 @@ static svn_error_t *writebuf_write(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
   return SVN_NO_ERROR;
 }
 
+static svn_error_t *writebuf_printf(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
+                                    const char *fmt, ...)
+    __attribute__ ((format (printf, 3, 4)));
 static svn_error_t *writebuf_printf(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
                                     const char *fmt, ...)
 {
@@ -246,10 +250,10 @@ static svn_error_t *readbuf_input(svn_ra_svn_conn_t *conn, char *data,
   if (conn->sock && conn->block_handler)
     apr_socket_timeout_set(conn->sock, 0);
   if (status && !APR_STATUS_IS_EOF(status))
-    return svn_error_wrap_apr(status, "Can't read from connection");
+    return svn_error_wrap_apr(status, _("Can't read from connection"));
   if (*len == 0)
     return svn_error_create(SVN_ERR_RA_SVN_CONNECTION_CLOSED, NULL,
-                            "Connection closed unexpectedly");
+                            _("Connection closed unexpectedly"));
   return SVN_NO_ERROR;
 }
 
@@ -259,7 +263,7 @@ static svn_error_t *readbuf_fill(svn_ra_svn_conn_t *conn, apr_pool_t *pool)
   apr_size_t len;
 
   assert(conn->read_ptr == conn->read_end);
-  writebuf_flush(conn, pool);
+  SVN_ERR(writebuf_flush(conn, pool));
   len = sizeof(conn->read_buf);
   SVN_ERR(readbuf_input(conn, conn->read_buf, &len));
   conn->read_ptr = conn->read_buf;
@@ -298,7 +302,7 @@ static svn_error_t *readbuf_read(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
   /* Read large chunks directly into buffer. */
   while (end - data > (apr_ssize_t)sizeof(conn->read_buf))
     {
-      writebuf_flush(conn, pool);
+      SVN_ERR(writebuf_flush(conn, pool));
       count = end - data;
       SVN_ERR(readbuf_input(conn, data, &count));
       data += count;
@@ -493,7 +497,7 @@ static svn_error_t *read_string(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
    * so check for wrapping */
   if (((apr_size_t) len) < len) 
     return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
-                            "String length larger than maximum");
+                            _("String length larger than maximum"));
 
   while (len)
     {
@@ -516,15 +520,23 @@ static svn_error_t *read_string(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
 }
 
 /* Given the first non-whitespace character FIRST_CHAR, read an item
- * into the already allocated structure ITEM. */
+ * into the already allocated structure ITEM.  LEVEL should be set
+ * to 0 for the first call and is used to enforce a recurssion limit
+ * on the parser. */
 static svn_error_t *read_item(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
-                              svn_ra_svn_item_t *item, char first_char)
+                              svn_ra_svn_item_t *item, char first_char,
+                              int level)
 {
   char c = first_char;
   apr_uint64_t val, prev_val=0;
   svn_stringbuf_t *str;
   svn_ra_svn_item_t *listitem;
 
+  if (++level >= 64)
+    return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
+                            _("Too many nested items"));
+  
+  
   /* Determine the item type and read it in.  Make sure that c is the
    * first character at the end of the item so we can test to make
    * sure it's whitespace. */
@@ -541,7 +553,7 @@ static svn_error_t *read_item(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
           val = val * 10 + (c - '0');
           if ((val / 10) != prev_val) /* val wrapped past maximum value */
             return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
-                                    "Number is larger than maximum"); 
+                                    _("Number is larger than maximum"));
         }
       if (c == ':')
         {
@@ -581,14 +593,14 @@ static svn_error_t *read_item(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
           if (c == ')')
             break;
           listitem = apr_array_push(item->u.list);
-          SVN_ERR(read_item(conn, pool, listitem, c));
+          SVN_ERR(read_item(conn, pool, listitem, c, level));
         }
       SVN_ERR(readbuf_getchar(conn, pool, &c));
     }
 
   if (!svn_iswhitespace(c))
     return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
-                            "Malformed network data");
+                            _("Malformed network data"));
   return SVN_NO_ERROR;
 }
 
@@ -601,7 +613,7 @@ svn_error_t *svn_ra_svn_read_item(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
    * the work.  This makes sense because of the way lists are read. */
   *item = apr_palloc(pool, sizeof(**item));
   SVN_ERR(readbuf_getchar_skip_whitespace(conn, pool, &c));
-  return read_item(conn, pool, *item, c);
+  return read_item(conn, pool, *item, c, 0);
 }
 
 svn_error_t *svn_ra_svn_skip_leading_garbage(svn_ra_svn_conn_t *conn,
@@ -679,6 +691,9 @@ static svn_error_t *vparse_tuple(apr_array_header_t *list, apr_pool_t *pool,
             case 'l':
               *va_arg(*ap, apr_array_header_t **) = NULL;
               break;
+            case 'n':
+              *va_arg(*ap, apr_uint64_t *) = SVN_RA_SVN_UNSPECIFIED_NUMBER;
+              break;
             case '(':
               list_level++;
               break;
@@ -693,7 +708,7 @@ static svn_error_t *vparse_tuple(apr_array_header_t *list, apr_pool_t *pool,
     }
   if (**fmt && **fmt != ')')
     return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
-                            "Malformed network data");
+                            _("Malformed network data"));
   return SVN_NO_ERROR;
 }
 
@@ -720,7 +735,7 @@ svn_error_t *svn_ra_svn_read_tuple(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
   SVN_ERR(svn_ra_svn_read_item(conn, pool, &item));
   if (item->kind != SVN_RA_SVN_LIST)
     return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
-                            "Malformed network data");
+                            _("Malformed network data"));
   va_start(ap, fmt);
   err = vparse_tuple(item->u.list, pool, &fmt, &ap);
   va_end(ap);
@@ -729,17 +744,51 @@ svn_error_t *svn_ra_svn_read_tuple(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
 
 /* --- READING AND WRITING COMMANDS AND RESPONSES --- */
 
+svn_error_t *svn_ra_svn__handle_failure_status(apr_array_header_t *params,
+                                               apr_pool_t *pool)
+{
+  const char *message, *file;
+  svn_error_t *err = NULL;
+  svn_ra_svn_item_t *elt;
+  int i;
+  apr_uint64_t apr_err, line;
+  apr_pool_t *subpool = svn_pool_create(pool);
+
+  if (params->nelts == 0)
+    return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
+                            _("Empty error list"));
+
+  /* Rebuild the error list from the end, to avoid reversing the order. */
+  for (i = params->nelts - 1; i >= 0; i--)
+    {
+      svn_pool_clear(subpool);
+      elt = &((svn_ra_svn_item_t *) params->elts)[i];
+      if (elt->kind != SVN_RA_SVN_LIST)
+        return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
+                                _("Malformed error list"));
+      SVN_ERR(svn_ra_svn_parse_tuple(elt->u.list, subpool, "nccn", &apr_err,
+                                      &message, &file, &line));
+      /* The message field should have been optional, but we can't
+         easily change that, so "" means a nonexistent message. */
+      if (!*message)
+        message = NULL;
+      err = svn_error_create(apr_err, err, message);
+      err->file = apr_pstrdup(err->pool, file);
+      err->line = line;
+    }
+
+  svn_pool_destroy(subpool);
+  return err;
+}
+
 svn_error_t *svn_ra_svn_read_cmd_response(svn_ra_svn_conn_t *conn,
                                           apr_pool_t *pool,
                                           const char *fmt, ...)
 {
   va_list ap;
-  const char *status, *message, *file;
+  const char *status;
   apr_array_header_t *params;
   svn_error_t *err;
-  svn_ra_svn_item_t *elt;
-  int i;
-  apr_uint64_t apr_err, line;
 
   SVN_ERR(svn_ra_svn_read_tuple(conn, pool, "wl", &status, &params));
   if (strcmp(status, "success") == 0)
@@ -751,32 +800,12 @@ svn_error_t *svn_ra_svn_read_cmd_response(svn_ra_svn_conn_t *conn,
     }
   else if (strcmp(status, "failure") == 0)
     {
-      /* Rebuild the error list from the end, to avoid reversing the order. */
-      if (params->nelts == 0)
-        return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
-                                "Empty error list");
-      err = NULL;
-      for (i = params->nelts - 1; i >= 0; i--)
-        {
-          elt = &((svn_ra_svn_item_t *) params->elts)[i];
-          if (elt->kind != SVN_RA_SVN_LIST)
-            return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
-                                    "Malformed error list");
-          SVN_ERR(svn_ra_svn_parse_tuple(elt->u.list, pool, "nccn", &apr_err,
-                                          &message, &file, &line));
-          /* The message field should have been optional, but we can't
-             easily change that, so "" means a nonexistent message. */
-          if (!*message)
-            message = NULL;
-          err = svn_error_create(apr_err, err, message);
-          err->file = apr_pstrdup(err->pool, file);
-          err->line = line;
-        }
-      return err;
+      return svn_ra_svn__handle_failure_status(params, pool);
     }
 
   return svn_error_createf(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
-                           "Unknown status '%s' in command response", status);
+                           _("Unknown status '%s' in command response"),
+                           status);
 }
 
 svn_error_t *svn_ra_svn_handle_commands(svn_ra_svn_conn_t *conn,
@@ -804,7 +833,7 @@ svn_error_t *svn_ra_svn_handle_commands(svn_ra_svn_conn_t *conn,
       else
         {
           err = svn_error_createf(SVN_ERR_RA_SVN_UNKNOWN_CMD, NULL,
-                                  "Unknown command '%s'", cmdname);
+                                  _("Unknown command '%s'"), cmdname);
           err = svn_error_create(SVN_ERR_RA_SVN_CMD_ERR, err, NULL);
         }
 
