@@ -1,7 +1,7 @@
-/* commit.c --- editor for commiting changes to a filesystem.
+/* commit.c --- editor for committing changes to a filesystem.
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -29,6 +29,8 @@
 #include "svn_fs.h"
 #include "svn_repos.h"
 #include "svn_md5.h"
+#include "svn_props.h"
+#include "svn_private_config.h"
 
 
 
@@ -48,8 +50,12 @@ struct edit_baton
   const char *log_msg;
 
   /* Callback to run when the commit is done. */
-  svn_repos_commit_callback_t callback;
-  void *callback_baton;
+  svn_commit_callback_t commit_callback;
+  void *commit_callback_baton;
+
+  /* Callback to check authorizations on paths. */
+  svn_repos_authz_callback_t authz_callback;
+  void *authz_baton;
 
   /* The already-open svn repository to commit to. */
   svn_repos_t *repos;
@@ -57,6 +63,9 @@ struct edit_baton
   /* URL to the root of the open repository. */
   const char *repos_url;
 
+  /* The name of the repository (here for convenience). */
+  const char *repos_name;
+
   /* The filesystem associated with the REPOS above (here for
      convenience). */
   svn_fs_t *fs;
@@ -64,17 +73,21 @@ struct edit_baton
   /* Location in fs where the edit will begin. */
   const char *base_path;
 
-  /** Created during the edit: **/
+  /* Does this set of interfaces 'own' the commit transaction? */
+  svn_boolean_t txn_owner;
 
-  /* svn transaction associated with this edit (created in open_root). */
+  /* svn transaction associated with this edit (created in
+     open_root, or supplied by the public API caller). */
   svn_fs_txn_t *txn;
 
-  /* The object representing the root directory of the svn txn. */
-  svn_fs_root_t *txn_root;
+  /** Filled in during open_root: **/
 
   /* The name of the transaction. */
   const char *txn_name;
 
+  /* The object representing the root directory of the svn txn. */
+  svn_fs_root_t *txn_root;
+
   /** Filled in when the edit is closed: **/
 
   /* The new revision created by this commit. */
@@ -103,7 +116,6 @@ struct file_baton
 {
   struct edit_baton *edit_baton;
   const char *path; /* the -absolute- path to this file in the fs */
-  apr_pool_t *pool; /* my personal pool, in which I am allocated. */
 };
 
 
@@ -113,10 +125,38 @@ static svn_error_t *
 out_of_date (const char *path, const char *txn_name)
 {
   return svn_error_createf (SVN_ERR_FS_TXN_OUT_OF_DATE, NULL,
-                            "out of date: `%s' in txn `%s'", path, txn_name);
+                            _("Out of date: '%s' in transaction '%s'"),
+                            path, txn_name);
 }
 
 
+
+/* If EDITOR_BATON contains a valid authz callback, verify that the
+   REQUIRED access to PATH in ROOT is authorized.  Return an error
+   appropriate for throwing out of the commit editor with SVN_ERR.  If
+   no authz callback is present in EDITOR_BATON, then authorize all
+   paths.  Use POOL for temporary allocation only. */
+static svn_error_t *
+check_authz (struct edit_baton *editor_baton, const char *path,
+             svn_fs_root_t *root, svn_repos_authz_access_t required,
+             apr_pool_t *pool)
+{
+  if (editor_baton->authz_callback)
+    {
+      svn_boolean_t allowed;
+
+      editor_baton->authz_callback (required, &allowed, root, path,
+                                    editor_baton->authz_baton, pool);
+      if (!allowed)
+        return svn_error_create(required & svn_authz_write ?
+                                SVN_ERR_AUTHZ_UNWRITABLE :
+                                SVN_ERR_AUTHZ_UNREADABLE,
+                                NULL, "Access denied");
+    }
+
+  return SVN_NO_ERROR;
+}
+
 
 /*** Editor functions ***/
 
@@ -135,17 +175,42 @@ open_root (void *edit_baton,
      dateness checks.  */
   SVN_ERR (svn_fs_youngest_rev (&youngest, eb->fs, eb->pool));
 
-  /* Begin a subversion transaction, cache its name, and get its
-     root object. */
-  SVN_ERR (svn_repos_fs_begin_txn_for_commit (&(eb->txn), 
-                                              eb->repos, 
-                                              youngest,
-                                              eb->user, 
-                                              eb->log_msg,
-                                              eb->pool));
-  SVN_ERR (svn_fs_txn_root (&(eb->txn_root), eb->txn, eb->pool));
+  /* Unless we've been instructed to use a specific transaction, we'll
+     make our own. */
+  if (eb->txn_owner)
+    {
+      SVN_ERR (svn_repos_fs_begin_txn_for_commit (&(eb->txn),
+                                                  eb->repos, 
+                                                  youngest,
+                                                  eb->user, 
+                                                  eb->log_msg,
+                                                  eb->pool));
+    }
+  else /* Even if we aren't the owner of the transaction, we might
+          have been instructed to set some properties. */
+    {
+      svn_string_t propval;
+      if (eb->user)
+        {
+          propval.data = eb->user;
+          propval.len = strlen (eb->user);
+          SVN_ERR (svn_fs_change_txn_prop (eb->txn, SVN_PROP_REVISION_AUTHOR,
+                                           &propval, pool));
+        }
+      if (eb->log_msg)
+        {
+          propval.data = eb->log_msg;
+          propval.len = strlen (eb->log_msg);
+          SVN_ERR (svn_fs_change_txn_prop (eb->txn, SVN_PROP_REVISION_LOG,
+                                           &propval, pool));
+        }
+    }
   SVN_ERR (svn_fs_txn_name (&(eb->txn_name), eb->txn, eb->pool));
-  
+  SVN_ERR (svn_fs_txn_root (&(eb->txn_root), eb->txn, eb->pool));
+
+  /* Check read access to root */
+  SVN_ERR(check_authz (eb, "/", eb->txn_root, svn_authz_read, pool));
+
   /* Create a root dir baton.  The `base_path' field is an -absolute-
      path in the filesystem, upon which all further editor paths are
      based. */
@@ -173,11 +238,21 @@ delete_entry (const char *path,
   struct edit_baton *eb = parent->edit_baton;
   svn_node_kind_t kind;
   svn_revnum_t cr_rev;
+  svn_repos_authz_access_t required = svn_authz_write;
   const char *full_path = svn_path_join (eb->base_path, path, pool);
 
   /* Check PATH in our transaction.  */
   SVN_ERR (svn_fs_check_path (&kind, eb->txn_root, full_path, pool));
 
+  /* Deletion requires a recursive write access, as well as write
+     access to the parent directory. */
+  if (kind == svn_node_dir)
+    required |= svn_authz_recursive;
+  SVN_ERR (check_authz (eb, full_path, eb->txn_root,
+                        required, pool));
+  SVN_ERR (check_authz (eb, parent->path, eb->txn_root,
+                        svn_authz_write, pool));
+
   /* If PATH doesn't exist in the txn, that's fine (merge
      allows this). */
   if (kind == svn_node_none)
@@ -192,7 +267,7 @@ delete_entry (const char *path,
   /* This routine is a mindless wrapper.  We call svn_fs_delete_tree
      because that will delete files and recursively delete
      directories.  */
-  return svn_fs_delete_tree (eb->txn_root, full_path, pool);
+  return svn_fs_delete (eb->txn_root, full_path, pool);
 }
 
 
@@ -217,7 +292,7 @@ add_directory (const char *path,
   if (copy_path && (! SVN_IS_VALID_REVNUM (copy_revision)))
     return svn_error_createf 
       (SVN_ERR_FS_GENERAL, NULL,
-       "add_dir `%s': got copy_path, but no copy_rev", full_path);
+       _("Got source path but no source revision for '%s'"), full_path);
 
   if (copy_path)
     {
@@ -226,6 +301,14 @@ add_directory (const char *path,
       svn_node_kind_t kind;
       int repos_url_len;
 
+      /* Copy requires recursive write access to the destination path
+         and write access to the parent path. */
+      SVN_ERR (check_authz (eb, full_path, eb->txn_root,
+                            svn_authz_write | svn_authz_recursive,
+                            subpool));
+      SVN_ERR (check_authz (eb, pb->path, eb->txn_root,
+                            svn_authz_write, subpool));
+
       /* Check PATH in our transaction.  Make sure it does not exist
          unless its parent directory was copied (in which case, the
          thing might have been copied in as well), else return an
@@ -241,7 +324,7 @@ add_directory (const char *path,
       if (strncmp (copy_path, eb->repos_url, repos_url_len) != 0)
         return svn_error_createf 
           (SVN_ERR_FS_GENERAL, NULL,
-           "add_dir `%s': copy_url is from different repo", full_path);
+           _("Source url '%s' is from different repository"), copy_path);
 
       fs_path = apr_pstrdup (subpool, copy_path + repos_url_len);
 
@@ -249,6 +332,13 @@ add_directory (const char *path,
          repository to make the copy from. */      
       SVN_ERR (svn_fs_revision_root (&copy_root, eb->fs,
                                      copy_revision, subpool));
+
+      /* Copy also requires recursive read access to the source
+         path. */
+      SVN_ERR(check_authz (eb, fs_path, copy_root,
+                           svn_authz_read | svn_authz_recursive,
+                           subpool));
+
       SVN_ERR (svn_fs_copy (copy_root, fs_path,
                             eb->txn_root, full_path, subpool));
       was_copied = TRUE;
@@ -257,7 +347,13 @@ add_directory (const char *path,
     {
       /* No ancestry given, just make a new directory.  We don't
          bother with an out-of-dateness check here because
-         svn_fs_make_dir will error out if PATH already exists.  */      
+         svn_fs_make_dir will error out if PATH already exists.
+         Verify write access to the full path and the parent
+         directory. */
+      SVN_ERR (check_authz (eb, full_path, eb->txn_root,
+                            svn_authz_write, subpool));
+      SVN_ERR (check_authz (eb, pb->path, eb->txn_root,
+                            svn_authz_write, subpool));
       SVN_ERR (svn_fs_make_dir (eb->txn_root, full_path, subpool));
     }
 
@@ -292,11 +388,17 @@ open_directory (const char *path,
   svn_node_kind_t kind;
   const char *full_path = svn_path_join (eb->base_path, path, pool);
 
-  /* Check PATH in our transaction.  Make sure it does not exist,
-     else return an out-of-dateness error. */
+  /* Check for read authorization. */
+  SVN_ERR (check_authz (eb, full_path, eb->txn_root,
+                        svn_authz_read, pool));
+
+  /* Check PATH in our transaction.  If it does not exist,
+     return a 'Path not present' error. */
   SVN_ERR (svn_fs_check_path (&kind, eb->txn_root, full_path, pool));
   if (kind == svn_node_none)
-    return out_of_date (full_path, eb->txn_name);
+    return svn_error_createf (SVN_ERR_FS_NOT_DIRECTORY, NULL,
+                              _("Path '%s' not present"),
+                              path);
 
   /* Build a new dir baton for this directory */
   new_dirb = apr_pcalloc (pool, sizeof (*new_dirb));
@@ -304,7 +406,7 @@ open_directory (const char *path,
   new_dirb->parent = pb;
   new_dirb->pool = pool;
   new_dirb->path = full_path;
-  new_dirb->was_copied = FALSE;
+  new_dirb->was_copied = pb->was_copied;
   new_dirb->base_rev = base_revision;
 
   *child_baton = new_dirb;
@@ -320,12 +422,18 @@ apply_textdelta (void *file_baton,
                  void **handler_baton)
 {
   struct file_baton *fb = file_baton;
+
+  /* Check for write authorization. */
+  SVN_ERR (check_authz (fb->edit_baton, fb->path,
+                        fb->edit_baton->txn_root,
+                        svn_authz_write, pool));
+
   return svn_fs_apply_textdelta (handler, handler_baton, 
                                  fb->edit_baton->txn_root, 
                                  fb->path,
                                  base_checksum,
                                  NULL,
-                                 fb->pool);
+                                 pool);
 }
 
 
@@ -349,7 +457,7 @@ add_file (const char *path,
   if (copy_path && (! SVN_IS_VALID_REVNUM (copy_revision)))
     return svn_error_createf 
       (SVN_ERR_FS_GENERAL, NULL,
-       "add_file `%s': got copy_path, but no copy_rev", full_path);
+       _("Got source path but no source revision for '%s'"), full_path);
 
   if (copy_path)
     {      
@@ -358,6 +466,13 @@ add_file (const char *path,
       svn_node_kind_t kind;
       int repos_url_len;
 
+      /* Copy requires recursive write to the destination path and
+         parent path. */
+      SVN_ERR (check_authz (eb, full_path, eb->txn_root,
+                           svn_authz_write, subpool));
+      SVN_ERR (check_authz (eb, pb->path, eb->txn_root,
+                            svn_authz_write, subpool));
+
       /* Check PATH in our transaction.  Make sure it does not exist
          unless its parent directory was copied (in which case, the
          thing might have been copied in as well), else return an
@@ -373,7 +488,7 @@ add_file (const char *path,
       if (strncmp (copy_path, eb->repos_url, repos_url_len) != 0)
             return svn_error_createf 
               (SVN_ERR_FS_GENERAL, NULL,
-               "add_file `%s': copy_url is from different repo", full_path);
+               _("Source url '%s' is from different repository"), copy_path);
       
       fs_path = apr_pstrdup (subpool, copy_path + repos_url_len);
 
@@ -381,6 +496,11 @@ add_file (const char *path,
          repository to make the copy from. */      
       SVN_ERR (svn_fs_revision_root (&copy_root, eb->fs,
                                      copy_revision, subpool));
+
+      /* Copy also requires read access to the source */
+      SVN_ERR(check_authz (eb, fs_path, copy_root,
+                           svn_authz_read, subpool));
+
       SVN_ERR (svn_fs_copy (copy_root, fs_path, 
                             eb->txn_root, full_path, subpool));
     }
@@ -389,7 +509,12 @@ add_file (const char *path,
       /* No ancestry given, just make a new, empty file.  Note that we
          don't perform an existence check here like the copy-from case
          does -- that's because svn_fs_make_file() already errors out
-         if the file already exists.  */
+         if the file already exists.  Verify write access to the full
+         path and to the parent. */
+      SVN_ERR (check_authz (eb, full_path, eb->txn_root, svn_authz_write,
+                            subpool));
+      SVN_ERR (check_authz (eb, pb->path, eb->txn_root, svn_authz_write,
+                            subpool));
       SVN_ERR (svn_fs_make_file (eb->txn_root, full_path, subpool));
     }
 
@@ -399,7 +524,6 @@ add_file (const char *path,
   /* Build a new file baton */
   new_fb = apr_pcalloc (pool, sizeof (*new_fb));
   new_fb->edit_baton = eb;
-  new_fb->pool = pool;
   new_fb->path = full_path;
 
   *file_baton = new_fb;
@@ -420,23 +544,32 @@ open_file (const char *path,
   struct dir_baton *pb = parent_baton;
   struct edit_baton *eb = pb->edit_baton;
   svn_revnum_t cr_rev;
+  apr_pool_t *subpool = svn_pool_create (pool);
   const char *full_path = svn_path_join (eb->base_path, path, pool);
 
+  /* Check for read authorization. */
+  SVN_ERR (check_authz (eb, full_path, eb->txn_root,
+                        svn_authz_read, subpool));
+
   /* Get this node's creation revision (doubles as an existence check). */
-  SVN_ERR (svn_fs_node_created_rev (&cr_rev, eb->txn_root, full_path, pool));
+  SVN_ERR (svn_fs_node_created_rev (&cr_rev, eb->txn_root, full_path, 
+                                    subpool));
   
   /* If the node our caller has is an older revision number than the
      one in our transaction, return an out-of-dateness error. */
-  if (base_revision < cr_rev)
+  if (SVN_IS_VALID_REVNUM(base_revision) && (base_revision < cr_rev))
     return out_of_date (full_path, eb->txn_name);
 
   /* Build a new file baton */
   new_fb = apr_pcalloc (pool, sizeof (*new_fb));
   new_fb->edit_baton = eb;
-  new_fb->pool = pool;
   new_fb->path = full_path;
-  
+
   *file_baton = new_fb;
+
+  /* Destory the work subpool. */
+  svn_pool_destroy (subpool);
+
   return SVN_NO_ERROR;
 }
 
@@ -450,6 +583,11 @@ change_file_prop (void *file_baton,
 {
   struct file_baton *fb = file_baton;
   struct edit_baton *eb = fb->edit_baton;
+
+  /* Check for write authorization. */
+  SVN_ERR (check_authz (eb, fb->path, eb->txn_root,
+                        svn_authz_write, pool));
+
   return svn_repos_fs_change_node_prop (eb->txn_root, fb->path, 
                                         name, value, pool);
 }
@@ -464,21 +602,21 @@ close_file (void *file_baton,
 
   if (text_checksum)
     {
-      unsigned char digest[MD5_DIGESTSIZE];
+      unsigned char digest[APR_MD5_DIGESTSIZE];
       const char *hex_digest;
 
       SVN_ERR (svn_fs_file_md5_checksum
                (digest, fb->edit_baton->txn_root, fb->path, pool));
       hex_digest = svn_md5_digest_to_cstring (digest, pool);
-      
-      if (strcmp (text_checksum, hex_digest) != 0)
+
+      if (hex_digest && strcmp (text_checksum, hex_digest) != 0)
         {
           return svn_error_createf
             (SVN_ERR_CHECKSUM_MISMATCH, NULL,
-             "close_file: checksum mismatch for resulting fulltext\n"
-             "(%s):\n"
-             "   expected checksum:  %s\n"
-             "   actual checksum:    %s\n",
+             _("Checksum mismatch for resulting fulltext\n"
+               "(%s):\n"
+               "   expected checksum:  %s\n"
+               "   actual checksum:    %s\n"),
              fb->path, text_checksum, hex_digest);
         }
     }
@@ -496,6 +634,10 @@ change_dir_prop (void *dir_baton,
   struct dir_baton *db = dir_baton;
   struct edit_baton *eb = db->edit_baton;
 
+  /* Check for write authorization. */
+  SVN_ERR (check_authz (eb, db->path, eb->txn_root,
+                        svn_authz_write, pool));
+
   if (SVN_IS_VALID_REVNUM(db->base_rev))
     {
       /* Subversion rule:  propchanges can only happen on a directory
@@ -513,6 +655,7 @@ change_dir_prop (void *dir_baton,
 }
 
 
+
 static svn_error_t *
 close_edit (void *edit_baton,
             apr_pool_t *pool)
@@ -522,10 +665,20 @@ close_edit (void *edit_baton,
   svn_error_t *err;
   const char *conflict;
 
-  /* Commit. */
-  err = svn_repos_fs_commit_txn (&conflict, eb->repos, &new_revision, eb->txn);
+  /* If no transaction has been created (ie. if open_root wasn't
+     called before close_edit), abort the operation here with an
+     error. */
+  if (! eb->txn)
+    return svn_error_create(SVN_ERR_REPOS_BAD_ARGS, NULL,
+                            "No valid transaction supplied to close_edit");
 
-  if (err)
+  /* Commit. */
+  err = svn_repos_fs_commit_txn (&conflict, eb->repos, 
+                                 &new_revision, eb->txn, pool);
+
+  /* We want to abort the transaction *unless* the error code tells us
+     the commit succeeded and something just went wrong in post-commit. */
+  if (err && (err->apr_err != SVN_ERR_REPOS_POST_COMMIT_HOOK_FAILED))
     {
       /* ### todo: we should check whether it really was a conflict,
          and return the conflict info if so? */
@@ -540,66 +693,109 @@ close_edit (void *edit_baton,
          So, in a nutshell: svn commits are an all-or-nothing deal.
          Each commit creates a new fs txn which either succeeds or is
          aborted completely.  No second chances;  the user simply
-         needs to update and commit again  :) */
+         needs to update and commit again  :)
 
-      /* We don't care about the error return here, we want to return the
-         orignal error. There's likely something seriously wrong already, and
-         we don't want to cover it up.  */
-      svn_fs_abort_txn (eb->txn);
+         We ignore the possible error result from svn_fs_abort_txn();
+         it's more important to return the original error. */
+      svn_error_clear (svn_fs_abort_txn (eb->txn, pool));
       return err;
     }
-
+  else if (err)
+    {
+      /* ### TODO: ra_local is the only RA layer that currently
+         understands SVN_ERR_REPOS_POST_COMMIT_HOOK_FAILED.  And as of
+         at least r12960, svn_repos_fs_commit_txn() would never return
+         that anyway.  If someone who knows ra_svn better can add
+         handling for this special case, this whole "else if" block
+         can go away (again).  */ 
+      svn_error_clear(err);
+      err = SVN_NO_ERROR;
+    }
+  
   /* Pass new revision information to the caller's callback. */
   {
     svn_string_t *date, *author;
+    svn_error_t *err2;
 
-    SVN_ERR (svn_fs_revision_prop (&date, svn_repos_fs (eb->repos),
-                                   new_revision, SVN_PROP_REVISION_DATE,
-                                   eb->pool));
+    /* Even if there was a post-commit hook failure, it's more serious
+       if one of the calls here fails, so we explicitly check for errors
+       here, while saving the possible post-commit error for later. */
 
-    SVN_ERR (svn_fs_revision_prop (&author, svn_repos_fs (eb->repos),
-                                   new_revision, SVN_PROP_REVISION_AUTHOR,
-                                   eb->pool));
+    err2 = svn_fs_revision_prop (&date, svn_repos_fs (eb->repos),
+                                new_revision, SVN_PROP_REVISION_DATE,
+                                eb->pool);
+    if (! err2)
+      err2 =  svn_fs_revision_prop (&author, svn_repos_fs (eb->repos),
+                                    new_revision, SVN_PROP_REVISION_AUTHOR,
+                                    eb->pool);
 
-    SVN_ERR ((*eb->callback) (new_revision, 
+    if (! err2)
+      err2 = (*eb->commit_callback) (new_revision, 
                               date ? date->data : NULL, 
                               author ? author->data : NULL,
-                              eb->callback_baton));
+                              eb->commit_callback_baton);
+    if (err2)
+      {
+        svn_error_clear (err);
+        return err2;
+      }
+
   }
 
-  return SVN_NO_ERROR;
+  return err;
 }
 
 
-
 static svn_error_t *
 abort_edit (void *edit_baton,
             apr_pool_t *pool)
 {
   struct edit_baton *eb = edit_baton;
-  return (eb->txn ? svn_fs_abort_txn (eb->txn) : SVN_NO_ERROR);
+  if ((! eb->txn) || (! eb->txn_owner))
+    return SVN_NO_ERROR;
+  return svn_fs_abort_txn (eb->txn, pool);
 }
 
 
 
 
-/*** Public interface. ***/
+/*** Public interfaces. ***/
 
 svn_error_t *
-svn_repos_get_commit_editor (const svn_delta_editor_t **editor,
-                             void **edit_baton,
-                             svn_repos_t *repos,
-                             const char *repos_url,
-                             const char *base_path,
-                             const char *user,
-                             const char *log_msg,
-                             svn_repos_commit_callback_t callback,
-                             void *callback_baton,
-                             apr_pool_t *pool)
+svn_repos_get_commit_editor3 (const svn_delta_editor_t **editor,
+                              void **edit_baton,
+                              svn_repos_t *repos,
+                              svn_fs_txn_t *txn,
+                              const char *repos_url,
+                              const char *base_path,
+                              const char *user,
+                              const char *log_msg,
+                              svn_commit_callback_t callback,
+                              void *callback_baton,
+                              svn_repos_authz_callback_t authz_callback,
+                              void *authz_baton,
+                              apr_pool_t *pool)
 {
-  svn_delta_editor_t *e = svn_delta_default_editor (pool);
+  svn_delta_editor_t *e;
   apr_pool_t *subpool = svn_pool_create (pool);
-  struct edit_baton *eb = apr_pcalloc (subpool, sizeof (*eb));
+  struct edit_baton *eb;
+
+  /* Do a global authz access lookup.  Users with no write access
+     whatsoever to the repository don't get a commit editor. */
+  if (authz_callback)
+    {
+      svn_boolean_t allowed;
+
+      authz_callback (svn_authz_write, &allowed, NULL, NULL,
+                      authz_baton, pool);
+      if (!allowed)
+        return svn_error_create(SVN_ERR_AUTHZ_UNWRITABLE, NULL,
+                                "Not authorized to open a commit editor.");
+    }
+
+  /* Allocate the structures. */
+  e = svn_delta_default_editor (pool);
+  eb = apr_pcalloc (subpool, sizeof (*eb));
 
   /* Set up the editor. */
   e->open_root         = open_root;
@@ -619,16 +815,60 @@ svn_repos_get_commit_editor (const svn_delta_editor_t **editor,
   eb->pool = subpool;
   eb->user = user ? apr_pstrdup (subpool, user) : NULL;
   eb->log_msg = apr_pstrdup (subpool, log_msg);
-  eb->callback = callback;
-  eb->callback_baton = callback_baton;
+  eb->commit_callback = callback;
+  eb->commit_callback_baton = callback_baton;
+  eb->authz_callback = authz_callback;
+  eb->authz_baton = authz_baton;
   eb->base_path = apr_pstrdup (subpool, base_path);
   eb->repos = repos;
   eb->repos_url = repos_url;
+  eb->repos_name = svn_path_basename (svn_repos_path (repos, subpool),
+                                      subpool);
   eb->fs = svn_repos_fs (repos);
-  eb->txn = NULL;
+  eb->txn = txn;
+  eb->txn_owner = txn ? FALSE : TRUE;
 
   *edit_baton = eb;
   *editor = e;
   
   return SVN_NO_ERROR;
 }
+
+
+svn_error_t *
+svn_repos_get_commit_editor2 (const svn_delta_editor_t **editor,
+                              void **edit_baton,
+                              svn_repos_t *repos,
+                              svn_fs_txn_t *txn,
+                              const char *repos_url,
+                              const char *base_path,
+                              const char *user,
+                              const char *log_msg,
+                              svn_commit_callback_t callback,
+                              void *callback_baton,
+                              apr_pool_t *pool)
+{
+  return svn_repos_get_commit_editor3 (editor, edit_baton, repos, txn,
+                                       repos_url, base_path, user,
+                                       log_msg, callback, callback_baton,
+                                       NULL, NULL, pool);
+}
+
+
+svn_error_t *
+svn_repos_get_commit_editor (const svn_delta_editor_t **editor,
+                             void **edit_baton,
+                             svn_repos_t *repos,
+                             const char *repos_url,
+                             const char *base_path,
+                             const char *user,
+                             const char *log_msg,
+                             svn_commit_callback_t callback,
+                             void *callback_baton,
+                             apr_pool_t *pool)
+{
+  return svn_repos_get_commit_editor3 (editor, edit_baton, repos, NULL,
+                                       repos_url, base_path, user,
+                                       log_msg,  callback, callback_baton,
+                                       NULL, NULL, pool);
+}
