@@ -32,6 +32,7 @@
 #include "svn_xml.h"
 #include "svn_path.h"
 #include "svn_dav.h"
+#include "svn_props.h"
 
 #include "dav_svn.h"
 #include <http_request.h>
@@ -224,7 +225,7 @@ static void add_to_path_map(apr_hash_t *hash,
   const char *repos_path = linkpath ? linkpath : norm_path;
 
   /* now, geez, put the path in the map already! */
-  apr_hash_set(hash, path, APR_HASH_KEY_STRING, (void *)repos_path);
+  apr_hash_set(hash, path, APR_HASH_KEY_STRING, repos_path);
 }
 
 
@@ -342,26 +343,6 @@ svn_stream_t * dav_svn_make_base64_output_stream(apr_bucket_brigade *bb,
 }
 
 
-static svn_error_t * send_xml(update_ctx_t *uc, const char *fmt, ...)
-{
-  apr_status_t apr_err;
-  va_list ap;
-
-  va_start(ap, fmt);
-  apr_err = apr_brigade_vprintf(uc->bb, ap_filter_flush, 
-                                uc->output, fmt, ap);
-  va_end(ap);
-  if (apr_err)
-    return svn_error_create(apr_err, 0, NULL);
-  /* ### check for an aborted connection, since the brigade functions
-     don't appear to be return useful errors when the connection is
-     dropped. */
-  if (uc->output->c->aborted)
-    return svn_error_create(SVN_ERR_APMOD_CONNECTION_ABORTED, 0, NULL);
-  return SVN_NO_ERROR;
-}
-
-
 /* Get the real filesystem PATH for BATON, and return the value
    allocated from POOL.  This function juggles the craziness of
    updates, switches, and updates of switched things. */
@@ -387,8 +368,9 @@ static svn_error_t * send_vsn_url(item_baton_t *baton, apr_pool_t *pool)
                            DAV_SVN_BUILD_URI_VERSION,
                            revision, path, 0 /* add_href */, pool);
   
-  return send_xml(baton->uc, "<D:checked-in><D:href>%s</D:href></D:checked-in>"
-                  DEBUG_CR, apr_xml_quote_string (pool, href, 1));
+  return dav_svn__send_xml(baton->uc->bb, baton->uc->output,
+                           "<D:checked-in><D:href>%s</D:href></D:checked-in>"
+                           DEBUG_CR, apr_xml_quote_string (pool, href, 1));
 }
 
 static svn_error_t * absent_helper(svn_boolean_t is_dir,
@@ -404,7 +386,7 @@ static svn_error_t * absent_helper(svn_boolean_t is_dir,
                                      "<S:absent-%s name=\"%s\"/>" DEBUG_CR,
                                      DIR_OR_FILE(is_dir),
                                      svn_path_basename(path, pool));
-      SVN_ERR( send_xml(uc, "%s", elt) );
+      SVN_ERR( dav_svn__send_xml(uc->bb, uc->output, "%s", elt) );
     }
 
   return SVN_NO_ERROR;
@@ -444,8 +426,9 @@ static svn_error_t * add_helper(svn_boolean_t is_dir,
 
   if (uc->resource_walk)
     {
-      SVN_ERR( send_xml(child->uc, "<S:resource path=\"%s\">" DEBUG_CR, 
-                        apr_xml_quote_string(pool, child->path3, 1)) );
+      SVN_ERR( dav_svn__send_xml(child->uc->bb, child->uc->output,
+                                 "<S:resource path=\"%s\">" DEBUG_CR, 
+                                 apr_xml_quote_string(pool, child->path3, 1)) );
     }
   else
     {
@@ -517,13 +500,14 @@ static svn_error_t * add_helper(svn_boolean_t is_dir,
          like format string insert placeholders.  For example,
          "this%20dir" is a valid printf() format string that means
          "this[insert an integer of width 20 here]ir". */
-      SVN_ERR( send_xml(child->uc, "%s", elt) );
+      SVN_ERR( dav_svn__send_xml(child->uc->bb, child->uc->output, "%s", elt) );
     }
 
   SVN_ERR( send_vsn_url(child, pool) );
 
   if (uc->resource_walk)
-    SVN_ERR( send_xml(child->uc, "</S:resource>" DEBUG_CR) );
+    SVN_ERR( dav_svn__send_xml(child->uc->bb, child->uc->output,
+                               "</S:resource>" DEBUG_CR) );
 
   *child_baton = child;
 
@@ -540,9 +524,10 @@ static svn_error_t * open_helper(svn_boolean_t is_dir,
   item_baton_t *child = make_child_baton(parent, path, pool);
   const char *qname = apr_xml_quote_string(pool, child->name, 1);
 
-  SVN_ERR( send_xml(child->uc, "<S:open-%s name=\"%s\""
-                    " rev=\"%ld\">" DEBUG_CR,
-                    DIR_OR_FILE(is_dir), qname, base_revision));
+  SVN_ERR( dav_svn__send_xml(child->uc->bb, child->uc->output,
+                             "<S:open-%s name=\"%s\""
+                             " rev=\"%ld\">" DEBUG_CR,
+                             DIR_OR_FILE(is_dir), qname, base_revision));
   SVN_ERR( send_vsn_url(child, pool) );
   *child_baton = child;
   return SVN_NO_ERROR;
@@ -564,24 +549,27 @@ static svn_error_t * close_helper(svn_boolean_t is_dir, item_baton_t *baton)
         {
           /* We already XML-escaped the property name in change_xxx_prop. */
           qname = ((const char **)(baton->removed_props->elts))[i];
-          SVN_ERR( send_xml(baton->uc, "<S:remove-prop name=\"%s\"/>" 
-                            DEBUG_CR, qname) );
+          SVN_ERR( dav_svn__send_xml(baton->uc->bb, baton->uc->output,
+                                     "<S:remove-prop name=\"%s\"/>" 
+                                     DEBUG_CR, qname) );
         }
     }
 
   if ((! baton->uc->send_all) && baton->changed_props && (! baton->added))
     {
       /* Tell the client to fetch all the props */
-      SVN_ERR( send_xml(baton->uc, "<S:fetch-props/>" DEBUG_CR) );
+      SVN_ERR( dav_svn__send_xml(baton->uc->bb, baton->uc->output,
+                                 "<S:fetch-props/>" DEBUG_CR) );
     }
 
-  SVN_ERR( send_xml(baton->uc, "<S:prop>") );
+  SVN_ERR( dav_svn__send_xml(baton->uc->bb, baton->uc->output, "<S:prop>") );
 
   /* Both modern and non-modern clients need the checksum... */
   if (baton->text_checksum)
     {
-      SVN_ERR( send_xml(baton->uc, "<V:md5-checksum>%s</V:md5-checksum>", 
-                        baton->text_checksum) );
+      SVN_ERR( dav_svn__send_xml(baton->uc->bb, baton->uc->output,
+                                 "<V:md5-checksum>%s</V:md5-checksum>", 
+                                 baton->text_checksum) );
     }
 
   /* ...but only non-modern clients want the 3 CR-related properties
@@ -599,29 +587,34 @@ static svn_error_t * close_helper(svn_boolean_t is_dir, item_baton_t *baton)
       /* ### special knowledge: svn_repos_dir_delta will never send
        *removals* of the commit-info "entry props". */
       if (baton->committed_rev)
-        SVN_ERR( send_xml(baton->uc, "<D:version-name>%s</D:version-name>",
-                          baton->committed_rev) );
+        SVN_ERR( dav_svn__send_xml(baton->uc->bb, baton->uc->output,
+                                   "<D:version-name>%s</D:version-name>",
+                                   baton->committed_rev) );
       
       if (baton->committed_date)
-        SVN_ERR( send_xml(baton->uc, "<D:creationdate>%s</D:creationdate>",
-                          baton->committed_date) );
+        SVN_ERR( dav_svn__send_xml(baton->uc->bb, baton->uc->output,
+                                   "<D:creationdate>%s</D:creationdate>",
+                                   baton->committed_date) );
       
       if (baton->last_author)
-        SVN_ERR( send_xml(baton->uc,
-                          "<D:creator-displayname>%s</D:creator-displayname>",
-                          baton->last_author) );
+        SVN_ERR( dav_svn__send_xml(baton->uc->bb, baton->uc->output,
+                                   "<D:creator-displayname>%s"
+                                   "</D:creator-displayname>",
+                                   baton->last_author) );
 
     }
 
   /* Close unconditionally, because we sent checksum unconditionally. */
-  SVN_ERR( send_xml(baton->uc, "</S:prop>\n") );
+  SVN_ERR( dav_svn__send_xml(baton->uc->bb, baton->uc->output, "</S:prop>\n") );
     
   if (baton->added)
-    SVN_ERR( send_xml(baton->uc, "</S:add-%s>" DEBUG_CR, 
-                      DIR_OR_FILE(is_dir)) );
+    SVN_ERR( dav_svn__send_xml(baton->uc->bb, baton->uc->output,
+                               "</S:add-%s>" DEBUG_CR, 
+                               DIR_OR_FILE(is_dir)) );
   else
-    SVN_ERR( send_xml(baton->uc, "</S:open-%s>" DEBUG_CR, 
-                      DIR_OR_FILE(is_dir)) );
+    SVN_ERR( dav_svn__send_xml(baton->uc->bb, baton->uc->output,
+                               "</S:open-%s>" DEBUG_CR, 
+                               DIR_OR_FILE(is_dir)) );
   return SVN_NO_ERROR;
 }
 
@@ -629,19 +622,20 @@ static svn_error_t * close_helper(svn_boolean_t is_dir, item_baton_t *baton)
 /* Send the opening tag of the update-report if it hasn't been sent
    already.
 
-   Note: because send_xml does not return an error, this function
+   Note: because dav_svn__send_xml does not return an error, this function
    never returns error either.  However, its prototype anticipates a
-   day when send_xml() can return error. */
+   day when dav_svn__send_xml() can return error. */
 static svn_error_t * maybe_start_update_report(update_ctx_t *uc)
 {
   if ((! uc->resource_walk) && (! uc->started_update))
     {
-      SVN_ERR( send_xml(uc,
-                        DAV_XML_HEADER DEBUG_CR
-                        "<S:update-report xmlns:S=\"" SVN_XML_NAMESPACE "\" "
-                        "xmlns:V=\"" SVN_DAV_PROP_NS_DAV "\" "
-                        "xmlns:D=\"DAV:\" %s>" DEBUG_CR,
-                        uc->send_all ? "send-all=\"true\"" : "") );
+      SVN_ERR( dav_svn__send_xml(uc->bb, uc->output,
+                                 DAV_XML_HEADER DEBUG_CR
+                                 "<S:update-report xmlns:S=\""
+                                 SVN_XML_NAMESPACE "\" "
+                                 "xmlns:V=\"" SVN_DAV_PROP_NS_DAV "\" "
+                                 "xmlns:D=\"DAV:\" %s>" DEBUG_CR,
+                                 uc->send_all ? "send-all=\"true\"" : "") );
       
       uc->started_update = TRUE;
     }
@@ -659,8 +653,9 @@ static svn_error_t * upd_set_target_revision(void *edit_baton,
   SVN_ERR( maybe_start_update_report(uc) );
 
   if (! uc->resource_walk)
-    SVN_ERR( send_xml(uc, "<S:target-revision rev=\"%ld"
-                      "\"/>" DEBUG_CR, target_revision) );
+    SVN_ERR( dav_svn__send_xml(uc->bb, uc->output,
+                               "<S:target-revision rev=\"%ld"
+                               "\"/>" DEBUG_CR, target_revision) );
 
   return SVN_NO_ERROR;
 }
@@ -689,12 +684,14 @@ static svn_error_t * upd_open_root(void *edit_baton,
   if (uc->resource_walk)
     {
       const char *qpath = apr_xml_quote_string(pool, b->path3, 1);
-      SVN_ERR( send_xml(uc, "<S:resource path=\"%s\">" DEBUG_CR, qpath) );
+      SVN_ERR( dav_svn__send_xml(uc->bb, uc->output,
+                                 "<S:resource path=\"%s\">" DEBUG_CR, qpath) );
     }
   else    
     {
-      SVN_ERR( send_xml(uc, "<S:open-directory rev=\"%ld\">"
-                        DEBUG_CR, base_revision) );
+      SVN_ERR( dav_svn__send_xml(uc->bb, uc->output,
+                                 "<S:open-directory rev=\"%ld\">"
+                                 DEBUG_CR, base_revision) );
     }
 
   /* Only transmit the root directory's Version Resource URL if
@@ -703,7 +700,8 @@ static svn_error_t * upd_open_root(void *edit_baton,
     SVN_ERR( send_vsn_url(b, pool) );
 
   if (uc->resource_walk)
-    SVN_ERR( send_xml(uc, "</S:resource>" DEBUG_CR) );
+    SVN_ERR( dav_svn__send_xml(uc->bb, uc->output,
+                               "</S:resource>" DEBUG_CR) );
 
   return SVN_NO_ERROR;
 }
@@ -716,7 +714,8 @@ static svn_error_t * upd_delete_entry(const char *path,
   item_baton_t *parent = parent_baton;
   const char *qname = apr_xml_quote_string(pool, 
                                            svn_path_basename(path, pool), 1);
-  return send_xml(parent->uc, "<S:delete-entry name=\"%s\"/>" DEBUG_CR, qname);
+  return dav_svn__send_xml(parent->uc->bb, parent->uc->output,
+                           "<S:delete-entry name=\"%s\"/>" DEBUG_CR, qname);
 }
 
 static svn_error_t * upd_add_directory(const char *path,
@@ -770,30 +769,34 @@ static svn_error_t * upd_change_xxx_prop(void *baton,
     {
       if (value)
         {
-          const svn_string_t *qval;
+          const char *qval;
           
           if (svn_xml_is_xml_safe(value->data, value->len))
             {
               svn_stringbuf_t *tmp = NULL;
               svn_xml_escape_cdata_string(&tmp, value, pool);
-              qval = svn_string_create (tmp->data, pool);
-              SVN_ERR( send_xml(b->uc, "<S:set-prop name=\"%s\">", qname) );
+              qval = tmp->data;
+              SVN_ERR( dav_svn__send_xml(b->uc->bb, b->uc->output,
+                                         "<S:set-prop name=\"%s\">", qname) );
             }
           else
             {
-              qval = svn_base64_encode_string(value, pool);
-              SVN_ERR( send_xml(b->uc, 
-                                "<S:set-prop name=\"%s\" encoding=\"base64\">" 
-                                DEBUG_CR, qname) );
+              qval = svn_base64_encode_string(value, pool)->data;
+              SVN_ERR( dav_svn__send_xml(b->uc->bb, b->uc->output,
+                                         "<S:set-prop name=\"%s\" "
+                                         "encoding=\"base64\">" DEBUG_CR,
+                                         qname) );
             }
           
-          SVN_ERR( send_xml(b->uc, "%s", qval->data) );
-          SVN_ERR( send_xml(b->uc, "</S:set-prop>" DEBUG_CR) );
+          SVN_ERR( dav_svn__send_xml(b->uc->bb, b->uc->output, "%s", qval) );
+          SVN_ERR( dav_svn__send_xml(b->uc->bb, b->uc->output,
+                                     "</S:set-prop>" DEBUG_CR) );
         }
       else  /* value is null, so this is a prop removal */
         {
-          SVN_ERR( send_xml(b->uc, "<S:remove-prop name=\"%s\"/>" DEBUG_CR, 
-                            qname) );
+          SVN_ERR( dav_svn__send_xml(b->uc->bb, b->uc->output,
+                                     "<S:remove-prop name=\"%s\"/>" DEBUG_CR,
+                                     qname) );
         }
     }
   else  /* don't do inline response, just cache prop names for close_helper */
@@ -896,13 +899,13 @@ static svn_error_t * window_handler(svn_txdelta_window_t *window, void *baton)
   if (! wb->seen_first_window)
     {
       wb->seen_first_window = TRUE;
-      SVN_ERR( send_xml(wb->uc, "<S:txdelta>") );
+      SVN_ERR( dav_svn__send_xml(wb->uc->bb, wb->uc->output, "<S:txdelta>") );
     }
 
   SVN_ERR( wb->handler(window, wb->handler_baton) );
 
   if (window == NULL)
-    SVN_ERR( send_xml(wb->uc, "</S:txdelta>") );
+    SVN_ERR( dav_svn__send_xml(wb->uc->bb, wb->uc->output, "</S:txdelta>") );
 
   return SVN_NO_ERROR;
 }
@@ -926,19 +929,23 @@ static svn_error_t * upd_apply_textdelta(void *file_baton,
                                          void **handler_baton)
 {
   item_baton_t *file = file_baton;
-  struct window_handler_baton *wb = apr_palloc(file->pool, sizeof(*wb));
+  struct window_handler_baton *wb;
   svn_stream_t *base64_stream;
 
-  if (file->uc->resource_walk)
+  /* Store the base checksum and the fact the file's text changed. */
+  file->base_checksum = apr_pstrdup(file->pool, base_checksum);
+  file->text_changed = TRUE;
+
+  /* If this is a resource walk, or if we're not in "send-all" mode,
+     we don't actually want to transmit text-deltas. */
+  if (file->uc->resource_walk || (! file->uc->send_all))
     {
       *handler = dummy_window_handler;
       *handler_baton = NULL;
       return SVN_NO_ERROR;
     }
 
-  file->base_checksum = apr_pstrdup(file->pool, base_checksum);
-  file->text_changed = TRUE;
-
+  wb = apr_palloc(file->pool, sizeof(*wb));
   wb->seen_first_window = FALSE;
   wb->uc = file->uc;
   base64_stream = dav_svn_make_base64_output_stream(wb->uc->bb, wb->uc->output,
@@ -960,7 +967,8 @@ static svn_error_t * upd_close_file(void *file_baton,
 {
   item_baton_t *file = file_baton;
 
-  file->text_checksum = apr_pstrdup(file->pool, text_checksum);
+  file->text_checksum = text_checksum ?
+    apr_pstrdup(file->pool, text_checksum) : NULL;
 
   /* If we are not in "send all" mode, and this file is not a new
      addition or didn't otherwise have changed text, tell the client
@@ -972,7 +980,7 @@ static svn_error_t * upd_close_file(void *file_baton,
                          file->base_checksum ? " base-checksum=\"" : "",
                          file->base_checksum ? file->base_checksum : "",
                          file->base_checksum ? "\"" : "");
-      SVN_ERR( send_xml(file->uc, elt) );
+      SVN_ERR( dav_svn__send_xml(file->uc->bb, file->uc->output, elt) );
     }
 
   return close_helper(FALSE /* is_dir */, file);
@@ -990,6 +998,20 @@ static svn_error_t * upd_close_edit(void *edit_baton,
 }
 
 
+/* Return a specific error associated with the contents of TAGNAME
+   being malformed.  Use pool for allocations.  */
+static dav_error *
+malformed_element_error(const char *tagname,
+                        apr_pool_t *pool)
+{
+  const char *errstr = apr_pstrcat(pool, "The request's '", tagname, 
+                                   "' element is malformed; there "
+                                   "is a problem with the client.", NULL);
+  return dav_svn__new_error_tag(pool, HTTP_BAD_REQUEST, 0, errstr,
+                                SVN_DAV_ERROR_NAMESPACE, SVN_DAV_ERROR_TAG);
+}
+
+
 dav_error * dav_svn__update_report(const dav_resource *resource,
                                    const apr_xml_doc *doc,
                                    ap_filter_t *output)
@@ -1000,6 +1022,8 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
   update_ctx_t uc = { 0 };
   svn_revnum_t revnum = SVN_INVALID_REVNUM;
   int ns;
+  int entry_counter = 0;
+  svn_boolean_t entry_is_empty = FALSE;
   svn_error_t *serr;
   dav_error *derr = NULL;
   apr_status_t apr_err;
@@ -1007,6 +1031,7 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
   const char *dst_path = NULL;
   const dav_svn_repos *repos = resource->info->repos;
   const char *target = "";
+  svn_boolean_t text_deltas = TRUE;
   svn_boolean_t recurse = TRUE;
   svn_boolean_t resource_walk = FALSE;
   svn_boolean_t ignore_ancestry = FALSE;
@@ -1019,22 +1044,23 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
 
   if (resource->info->restype != DAV_SVN_RESTYPE_VCC)
     {
-      return dav_new_error_tag(resource->pool, HTTP_CONFLICT, 0,
-                               "This report can only be run against a VCC.",
-                               SVN_DAV_ERROR_NAMESPACE,
-                               SVN_DAV_ERROR_TAG);
+      return dav_svn__new_error_tag(resource->pool, HTTP_CONFLICT, 0,
+                                    "This report can only be run against "
+                                    "a VCC.",
+                                    SVN_DAV_ERROR_NAMESPACE,
+                                    SVN_DAV_ERROR_TAG);
     }
 
   ns = dav_svn_find_ns(doc->namespaces, SVN_XML_NAMESPACE);
   if (ns == -1)
     {
-      return dav_new_error_tag(resource->pool, HTTP_BAD_REQUEST, 0,
-                               "The request does not contain the 'svn:' "
-                               "namespace, so it is not going to have an "
-                               "svn:target-revision element. That element "
-                               "is required.",
-                               SVN_DAV_ERROR_NAMESPACE,
-                               SVN_DAV_ERROR_TAG);
+      return dav_svn__new_error_tag(resource->pool, HTTP_BAD_REQUEST, 0,
+                                    "The request does not contain the 'svn:' "
+                                    "namespace, so it is not going to have an "
+                                    "svn:target-revision element. That element "
+                                    "is required.",
+                                    SVN_DAV_ERROR_NAMESPACE,
+                                    SVN_DAV_ERROR_TAG);
     }
   
   /* Look to see if client wants a report with props and textdeltas
@@ -1060,168 +1086,93 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
          Thus, the check for non-empty cdata in each of these cases
          cannot be moved to the top of the loop, because then it would
          wrongly catch other elements that do allow empty cdata. */ 
-
+      const char *cdata;
+      
       if (child->ns == ns && strcmp(child->name, "target-revision") == 0)
         {
-          if (! child->first_cdata.first)
-            return dav_new_error_tag(resource->pool, HTTP_BAD_REQUEST, 0,
-              "The request's 'target-revision' element contains empty cdata; "
-              "there is a problem with the client.",
-              SVN_DAV_ERROR_NAMESPACE,
-              SVN_DAV_ERROR_TAG);
-
-          /* ### assume no white space, no child elems, etc */
-          revnum = SVN_STR_TO_REV(child->first_cdata.first->text);
+          cdata = dav_xml_get_cdata(child, resource->pool, 1);
+          if (! *cdata)
+            return malformed_element_error(child->name, resource->pool);
+          revnum = SVN_STR_TO_REV(cdata);
         }
-
       if (child->ns == ns && strcmp(child->name, "src-path") == 0)
         {
-          /* ### assume no white space, no child elems, etc */
           dav_svn_uri_info this_info;
-
-          if (! child->first_cdata.first)
-            return dav_new_error_tag(resource->pool, HTTP_BAD_REQUEST, 0,
-              "The request's 'src-path' element contains empty cdata; "
-              "there is a problem with the client.",
-              SVN_DAV_ERROR_NAMESPACE,
-              SVN_DAV_ERROR_TAG);
-
-          /* split up the 1st public URL. */
-          if ((derr = dav_svn__test_canonical
-               (child->first_cdata.first->text, resource->pool)))
+          cdata = dav_xml_get_cdata(child, resource->pool, 0);
+          if (! *cdata)
+            return malformed_element_error(child->name, resource->pool);
+          if ((derr = dav_svn__test_canonical(cdata, resource->pool)))
             return derr;
-          serr = dav_svn_simple_parse_uri(&this_info, resource,
-                                          child->first_cdata.first->text,
-                                          resource->pool);
-          if (serr != NULL)
-            {
-              return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
-                                         "Could not parse src-path URL.",
-                                         resource->pool);
-            }
+          if ((serr = dav_svn_simple_parse_uri(&this_info, resource,
+                                               cdata, 
+                                               resource->pool)))
+            return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
+                                       "Could not parse 'src-path' URL.",
+                                       resource->pool);
           src_path = this_info.repos_path;
         }
-
       if (child->ns == ns && strcmp(child->name, "dst-path") == 0)
         {
-          /* ### assume no white space, no child elems, etc */
           dav_svn_uri_info this_info;
-
-          if (! child->first_cdata.first)
-            return dav_new_error_tag(resource->pool, HTTP_BAD_REQUEST, 0,
-              "The request's 'dst-path' element contains empty cdata; "
-              "there is a problem with the client.  See "
-              "http://subversion.tigris.org/issues/show_bug.cgi?id=1055",
-              SVN_DAV_ERROR_NAMESPACE,
-              SVN_DAV_ERROR_TAG);
-
-          /* split up the 2nd public URL. */
-          if ((derr = dav_svn__test_canonical
-               (child->first_cdata.first->text, resource->pool)))
+          cdata = dav_xml_get_cdata(child, resource->pool, 0);
+          if (! *cdata)
+            return malformed_element_error(child->name, resource->pool);
+          if ((derr = dav_svn__test_canonical(cdata, resource->pool)))
             return derr;
-          serr = dav_svn_simple_parse_uri(&this_info, resource,
-                                          child->first_cdata.first->text,
-                                          resource->pool);
-          if (serr != NULL)
-            {
-              return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
-                                         "Could not parse dst-path URL.",
-                                         resource->pool);
-            }
+          if ((serr = dav_svn_simple_parse_uri(&this_info, resource,
+                                               cdata, 
+                                               resource->pool)))
+            return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
+                                       "Could not parse 'dst-path' URL.",
+                                       resource->pool);
           dst_path = this_info.repos_path;
         }
-
       if (child->ns == ns && strcmp(child->name, "update-target") == 0)
         {
-          if (! child->first_cdata.first)
-            return dav_new_error_tag(resource->pool, HTTP_BAD_REQUEST, 0,
-              "The request's 'update-target' element contains empty cdata; "
-              "there is a problem with the client.",
-              SVN_DAV_ERROR_NAMESPACE,
-              SVN_DAV_ERROR_TAG);
-
-          /* ### assume no white space, no child elems, etc */
-          if ((derr = dav_svn__test_canonical
-               (child->first_cdata.first->text, resource->pool)))
+          cdata = dav_xml_get_cdata(child, resource->pool, 0);
+          if ((derr = dav_svn__test_canonical(cdata, resource->pool)))
             return derr;
-          target = child->first_cdata.first->text;
+          target = cdata;
         }
       if (child->ns == ns && strcmp(child->name, "recursive") == 0)
         {
-          if (! child->first_cdata.first)
-            return dav_new_error_tag(resource->pool, HTTP_BAD_REQUEST, 0,
-              "The request's 'recursive' element contains empty cdata; "
-              "there is a problem with the client.",
-              SVN_DAV_ERROR_NAMESPACE,
-              SVN_DAV_ERROR_TAG);
-
-          /* ### assume no white space, no child elems, etc */
-          if (strcmp(child->first_cdata.first->text, "no") == 0)
+          cdata = dav_xml_get_cdata(child, resource->pool, 1);
+          if (! *cdata)
+            return malformed_element_error(child->name, resource->pool);
+          if (strcmp(cdata, "no") == 0)
             recurse = FALSE;
         }
       if (child->ns == ns && strcmp(child->name, "ignore-ancestry") == 0)
         {
-          if (! child->first_cdata.first)
-            return dav_new_error_tag(resource->pool, HTTP_BAD_REQUEST, 0,
-              "The request's 'ignore-ancestry' element contains empty cdata; "
-              "there is a problem with the client.",
-              SVN_DAV_ERROR_NAMESPACE,
-              SVN_DAV_ERROR_TAG);
-
-          /* ### assume no white space, no child elems, etc */
-          ignore_ancestry = TRUE;
-          if (strcmp(child->first_cdata.first->text, "no") == 0)
-            ignore_ancestry = FALSE;
+          cdata = dav_xml_get_cdata(child, resource->pool, 1);
+          if (! *cdata)
+            return malformed_element_error(child->name, resource->pool);
+          if (strcmp(cdata, "no") != 0)
+            ignore_ancestry = TRUE;
         }
       if (child->ns == ns && strcmp(child->name, "resource-walk") == 0)
         {
-          if (! child->first_cdata.first)
-            return dav_new_error_tag(resource->pool, HTTP_BAD_REQUEST, 0,
-              "The request's 'resource-walk' element contains empty cdata; "
-              "there is a problem with the client.",
-              SVN_DAV_ERROR_NAMESPACE,
-              SVN_DAV_ERROR_TAG);
-
-          /* ### assume no white space, no child elems, etc */
-          if (strcmp(child->first_cdata.first->text, "no") != 0)
+          cdata = dav_xml_get_cdata(child, resource->pool, 1);
+          if (! *cdata)
+            return malformed_element_error(child->name, resource->pool);
+          if (strcmp(cdata, "no") != 0)
             resource_walk = TRUE;
         }
-    }
-
-  if (revnum == SVN_INVALID_REVNUM)
-    {
-      serr = svn_fs_youngest_rev(&revnum, repos->fs, resource->pool);
-      if (serr != NULL)
+      if (child->ns == ns && strcmp(child->name, "text-deltas") == 0)
         {
-          return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
-                                     "Could not determine the youngest "
-                                     "revision for the update process.",
-                                     resource->pool);
+          cdata = dav_xml_get_cdata(child, resource->pool, 1);
+          if (! *cdata)
+            return malformed_element_error(child->name, resource->pool);
+          if (strcmp(cdata, "no") == 0)
+            text_deltas = FALSE;
         }
     }
-
-  editor = svn_delta_default_editor(resource->pool);
-  editor->set_target_revision = upd_set_target_revision;
-  editor->open_root = upd_open_root;
-  editor->delete_entry = upd_delete_entry;
-  editor->add_directory = upd_add_directory;
-  editor->open_directory = upd_open_directory;
-  editor->change_dir_prop = upd_change_xxx_prop;
-  editor->close_directory = upd_close_directory;
-  editor->absent_directory = upd_absent_directory;
-  editor->add_file = upd_add_file;
-  editor->open_file = upd_open_file;
-  editor->apply_textdelta = upd_apply_textdelta;
-  editor->change_file_prop = upd_change_xxx_prop;
-  editor->close_file = upd_close_file;
-  editor->absent_file = upd_absent_file;
-  editor->close_edit = upd_close_edit;
-
+          
   /* If the client never sent a <src-path> element, it's old and
      sending a style of report that we no longer allow. */
   if (! src_path)
     {
-      return dav_new_error_tag
+      return dav_svn__new_error_tag
         (resource->pool, HTTP_BAD_REQUEST, 0,
          "The request did not contain the '<src-path>' element.\n"
          "This may indicate that your client is too old.",
@@ -1229,6 +1180,17 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
          SVN_DAV_ERROR_TAG);
     }
 
+  /* If a revision for this operation was not dictated to us, this
+     means "update to whatever the current HEAD is now". */
+  if (revnum == SVN_INVALID_REVNUM)
+    {
+      if ((serr = svn_fs_youngest_rev(&revnum, repos->fs, resource->pool)))
+        return dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
+                                   "Could not determine the youngest "
+                                   "revision for the update process.",
+                                   resource->pool);
+    }
+
   uc.resource = resource;
   uc.output = output;  
   uc.anchor = src_path;
@@ -1269,15 +1231,37 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
                                  resource->pool);
     }
 
+  /* If the client did *not* request 'send-all' mode, then we will be
+     sending only a "skelta" of the difference, which will not need to
+     contain actual text deltas. */
+  if (! uc.send_all)
+    text_deltas = FALSE;
+
   /* When we call svn_repos_finish_report, it will ultimately run
      dir_delta() between REPOS_PATH/TARGET and TARGET_PATH.  In the
      case of an update or status, these paths should be identical.  In
      the case of a switch, they should be different. */
+  editor = svn_delta_default_editor(resource->pool);
+  editor->set_target_revision = upd_set_target_revision;
+  editor->open_root = upd_open_root;
+  editor->delete_entry = upd_delete_entry;
+  editor->add_directory = upd_add_directory;
+  editor->open_directory = upd_open_directory;
+  editor->change_dir_prop = upd_change_xxx_prop;
+  editor->close_directory = upd_close_directory;
+  editor->absent_directory = upd_absent_directory;
+  editor->add_file = upd_add_file;
+  editor->open_file = upd_open_file;
+  editor->apply_textdelta = upd_apply_textdelta;
+  editor->change_file_prop = upd_change_xxx_prop;
+  editor->close_file = upd_close_file;
+  editor->absent_file = upd_absent_file;
+  editor->close_edit = upd_close_edit;
   if ((serr = svn_repos_begin_report(&rbaton, revnum, repos->username, 
                                      repos->repos, 
                                      src_path, target,
                                      dst_path,
-                                     uc.send_all,
+                                     text_deltas,
                                      recurse,
                                      ignore_ancestry,
                                      editor, &uc,
@@ -1303,9 +1287,12 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
             const char *path;
             svn_revnum_t rev = SVN_INVALID_REVNUM;
             const char *linkpath = NULL;
+            const char *locktoken = NULL;
             svn_boolean_t start_empty = FALSE;
             apr_xml_attr *this_attr = child->attr;
 
+            entry_counter++;
+
             while (this_attr)
               {
                 if (! strcmp(this_attr->name, "rev"))
@@ -1313,7 +1300,9 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
                 else if (! strcmp(this_attr->name, "linkpath"))
                   linkpath = this_attr->value;
                 else if (! strcmp(this_attr->name, "start-empty"))
-                  start_empty = TRUE;
+                  start_empty = entry_is_empty = TRUE;
+                else if (! strcmp(this_attr->name, "lock-token"))
+                  locktoken = this_attr->value;
 
                 this_attr = this_attr->next;
               }
@@ -1332,14 +1321,14 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
               }
 
             /* get cdata, stripping whitespace */
-            path = dav_xml_get_cdata(child, subpool, 1);
+            path = dav_xml_get_cdata(child, subpool, 0);
             
             if (! linkpath)
-              serr = svn_repos_set_path(rbaton, path, rev,
-                                        start_empty, subpool);
+              serr = svn_repos_set_path2(rbaton, path, rev,
+                                         start_empty, locktoken, subpool);
             else
-              serr = svn_repos_link_path(rbaton, path, linkpath, rev,
-                                         start_empty, subpool);
+              serr = svn_repos_link_path2(rbaton, path, linkpath, rev,
+                                          start_empty, locktoken, subpool);
             if (serr != NULL)
               {
                 derr = dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
@@ -1365,7 +1354,7 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
         else if (strcmp(child->name, "missing") == 0)
           {
             /* get cdata, stripping whitespace */
-            const char *path = dav_xml_get_cdata(child, subpool, 1);
+            const char *path = dav_xml_get_cdata(child, subpool, 0);
             serr = svn_repos_delete_path(rbaton, path, subpool);
             if (serr != NULL)
               {
@@ -1379,6 +1368,58 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
           }
       }
 
+  /* Try to deduce what sort of client command is being run,, then
+     make this guess available to apache's logging subsystem. */
+  {
+    const char *action, *spath;
+
+    if (target)
+      spath = svn_path_join(src_path, target, resource->pool);
+    else
+      spath = src_path;
+
+    /* If a second path was passed to svn_repos_dir_delta(), then it
+       must have been switch, diff, or merge.  */
+    if (dst_path)
+      {
+        /* diff/merge don't ask for inline text-deltas. */
+        if (uc.send_all)
+          action = apr_psprintf(resource->pool,
+                                "switch '%s' '%s'",
+                                spath, dst_path);
+        else
+          action = apr_psprintf(resource->pool,
+                                "diff-or-merge '%s' '%s'",
+                                spath, dst_path);          
+      }
+
+    /* Otherwise, it must be checkout, export, or update. */
+    else
+      {
+        /* svn_client_checkout() creates a single root directory, then
+           reports it (and it alone) to the server as being empty. */
+        if (entry_counter == 1 && entry_is_empty)
+          action = apr_psprintf(resource->pool,
+                                "checkout-or-export '%s'",
+                                svn_path_uri_encode(spath, resource->pool));
+        else
+          {
+            if (text_deltas)
+              action = apr_psprintf(resource->pool,
+                                    "update '%s'",
+                                    svn_path_uri_encode(spath,
+                                                        resource->pool));
+            else
+              action = apr_psprintf(resource->pool,
+                                    "remote-status '%s'",
+                                    svn_path_uri_encode(spath,
+                                                        resource->pool));
+          }
+      }
+
+    apr_table_set(resource->info->r->subprocess_env, "SVN-ACTION", action);
+  }
+
   /* this will complete the report, and then drive our editor to generate
      the response to the client. */
   serr = svn_repos_finish_report(rbaton, resource->pool);
@@ -1433,7 +1474,7 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
           goto cleanup;
         }
 
-      serr = send_xml(&uc, "<S:resource-walk>" DEBUG_CR);
+      serr = dav_svn__send_xml(uc.bb, uc.output, "<S:resource-walk>" DEBUG_CR);
       if (serr)
         {
           derr = dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
@@ -1461,7 +1502,8 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
           goto cleanup;
         }
           
-      serr = send_xml(&uc, "</S:resource-walk>" DEBUG_CR);
+      serr = dav_svn__send_xml(uc.bb, uc.output, 
+                               "</S:resource-walk>" DEBUG_CR);
       if (serr)
         {
           derr = dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
@@ -1475,7 +1517,7 @@ dav_error * dav_svn__update_report(const dav_resource *resource,
      started in the first place. */
   if (uc.started_update)
     {
-      serr = send_xml(&uc, "</S:update-report>" DEBUG_CR);
+      serr = dav_svn__send_xml(uc.bb, uc.output, "</S:update-report>" DEBUG_CR);
       if (serr)
         {
           derr = dav_svn_convert_err(serr, HTTP_INTERNAL_SERVER_ERROR,
