@@ -104,6 +104,13 @@ def verify_expected_output(diff_output, expected):
   else:
     raise svntest.Failure
 
+def verify_excluded_output(diff_output, excluded):
+  "verify given line does not exist in diff output as diff line"
+  for line in diff_output:
+    if re.match("^(\\+|-)%s" % re.escape(excluded), line):
+      print 'Sought: %s' % excluded
+      print 'Found:  %s' % line
+      raise svntest.Failure
 
 def extract_diff_path(line):
   l2 = line[(line.find("(")+1):]
@@ -139,7 +146,8 @@ def diff_check_repo_subset(wc_dir, repo_subset, check_fn, do_diff_r):
 
 def update_a_file():
   "update a file"
-  svntest.main.file_append(os.path.join('A', 'B', 'E', 'alpha'), "new atext")
+  open(os.path.join('A', 'B', 'E', 'alpha'), 'w').write("new atext")
+  # svntest.main.file_append(, "new atext")
   return 0
 
 def check_update_a_file(diff_output):
@@ -261,9 +269,9 @@ def check_replace_a_file(diff_output):
 
 def update_three_files():
   "update three files"
-  svntest.main.file_append(os.path.join('A', 'D', 'gamma'), "new gamma")
-  svntest.main.file_append(os.path.join('A', 'D', 'G', 'tau'), "new tau")
-  svntest.main.file_append(os.path.join('A', 'D', 'H', 'psi'), "new psi")
+  open(os.path.join('A', 'D', 'gamma'), 'w').write("new gamma")
+  open(os.path.join('A', 'D', 'G', 'tau'), 'w').write("new tau")
+  open(os.path.join('A', 'D', 'H', 'psi'), 'w').write("new psi")
   return 0
 
 def check_update_three_files(diff_output):
@@ -685,7 +693,7 @@ def dont_diff_binary_file(sbox):
   expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
   expected_status.tweak(wc_rev=1)
   expected_status.add({
-    'A/theta' : Item(status='  ', wc_rev=2, repos_rev=2),
+    'A/theta' : Item(status='  ', wc_rev=2),
     })
 
   # Commit the new binary file, creating revision 2.
@@ -704,7 +712,7 @@ def dont_diff_binary_file(sbox):
 
   expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
   expected_status.add({
-    'A/theta' : Item(status='  ', wc_rev=2, repos_rev=2),
+    'A/theta' : Item(status='  ', wc_rev=2),
     })
 
   svntest.actions.run_and_verify_update(wc_dir,
@@ -750,7 +758,7 @@ def dont_diff_binary_file(sbox):
   expected_status = svntest.actions.get_virginal_state(wc_dir, 3)
   expected_status.tweak(wc_rev=2)
   expected_status.add({
-    'A/theta' : Item(status='  ', wc_rev=3, repos_rev=3),
+    'A/theta' : Item(status='  ', wc_rev=3),
     })
 
   svntest.actions.run_and_verify_commit(wc_dir, expected_output,
@@ -806,7 +814,7 @@ def diff_head_of_moved_file(sbox):
   # Modify the file to ensure that the diff is non-empty.
   svntest.main.file_append(new_mu_path, "\nActually, it's a new mu.")
 
-  svntest.actions.run_and_verify_svn(None, SVNAnyOutput, None,
+  svntest.actions.run_and_verify_svn(None, SVNAnyOutput, [],
                                      'diff', '-r', 'HEAD', new_mu_path)
 
 
@@ -827,7 +835,7 @@ def diff_base_to_repos(sbox):
   mu_path = os.path.join(sbox.wc_dir, 'A', 'mu')
 
   # Make changes to iota, commit r2, update to HEAD (r2).
-  svntest.main.file_append(iota_path, "some rev2 iota text.")
+  svntest.main.file_append(iota_path, "some rev2 iota text.\n")
 
   expected_output = svntest.wc.State(wc_dir, {
     'iota' : Item(verb='Sending'),
@@ -844,13 +852,14 @@ def diff_base_to_repos(sbox):
   expected_output = svntest.wc.State(wc_dir, {})
   expected_disk = svntest.main.greek_state.copy()
   expected_disk.tweak ('iota',
-                       contents="This is the file 'iota'.some rev2 iota text.")
+                       contents=\
+                       "This is the file 'iota'.\nsome rev2 iota text.\n")
   expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
   svntest.actions.run_and_verify_update(wc_dir, expected_output,
                                         expected_disk, expected_status)
   
   # Now make another local mod to iota.
-  svntest.main.file_append(iota_path, "an iota local mod.")
+  svntest.main.file_append(iota_path, "an iota local mod.\n")
 
   # If we run 'svn diff -r 1', we should see diffs that include *both*
   # the rev2 changes and local mods.  That's because the working files
@@ -858,12 +867,22 @@ def diff_base_to_repos(sbox):
   diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
                                                         'diff',
                                                         '-r', '1', wc_dir)
-  line1 = "+This is the file 'iota'.some rev2 iota text.an iota local mod.\n"
 
-  for line in diff_output:
-    if (line == line1):
-      break
-  else:
+  # Makes diff output look the same on all platforms.
+  def strip_eols(lines):
+    return [x.replace("\r", "").replace("\n", "") for x in lines]
+
+  expected_output_lines = [
+    "Index: svn-test-work/working_copies/diff_tests-14/iota\n",
+    "===================================================================\n",
+    "--- svn-test-work/working_copies/diff_tests-14/iota\t(revision 1)\n",
+    "+++ svn-test-work/working_copies/diff_tests-14/iota\t(working copy)\n",
+    "@@ -1 +1,3 @@\n",
+    " This is the file 'iota'.\n",
+    "+some rev2 iota text.\n",
+    "+an iota local mod.\n"]
+
+  if strip_eols(diff_output) != strip_eols(expected_output_lines):
     raise svntest.Failure
 
   # If we run 'svn diff -r BASE:1', we should see diffs that only show
@@ -872,12 +891,16 @@ def diff_base_to_repos(sbox):
   diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
                                                         'diff', '-r', 'BASE:1',
                                                         wc_dir)
-  line1 = "-This is the file 'iota'.some rev2 iota text.\n"
+  expected_output_lines = [
+    "Index: svn-test-work/working_copies/diff_tests-14/iota\n",
+    "===================================================================\n",
+    "--- svn-test-work/working_copies/diff_tests-14/iota\t(working copy)\n",
+    "+++ svn-test-work/working_copies/diff_tests-14/iota\t(revision 1)\n",
+    "@@ -1,2 +1 @@\n",
+    " This is the file 'iota'.\n",
+    "-some rev2 iota text.\n"]
 
-  for line in diff_output:
-    if (line == line1):
-      break
-  else:
+  if strip_eols(diff_output) != strip_eols(expected_output_lines):
     raise svntest.Failure
 
   # But that's not all folks... no, no, we're just getting started
@@ -913,13 +936,13 @@ def diff_base_to_repos(sbox):
     raise svntest.Failure
 
   # Now we schedule an addition and a deletion.
-  svntest.main.file_append(newfile_path, "Contents of newfile")
+  svntest.main.file_append(newfile_path, "Contents of newfile\n")
   svntest.main.run_svn(None, 'add', newfile_path)
   svntest.main.run_svn(None, 'rm', mu_path)
 
   expected_output = svntest.actions.get_virginal_state(wc_dir, 2)
   expected_output.add({
-    'A/D/newfile' : Item(status='A ', wc_rev=0, repos_rev=2),
+    'A/D/newfile' : Item(status='A ', wc_rev=0),
     })
   expected_output.tweak('A/mu', status='D ')
   expected_output.tweak('iota', status='M ')
@@ -967,7 +990,7 @@ def diff_base_to_repos(sbox):
   expected_status.tweak('iota', wc_rev=3)
   expected_status.remove('A/mu')
   expected_status.add({
-    'A/D/newfile' : Item(status='  ', wc_rev=3, repos_rev=3),
+    'A/D/newfile' : Item(status='  ', wc_rev=3),
     })
   svntest.actions.run_and_verify_commit(wc_dir, expected_output,
                                         expected_status, None,
@@ -976,14 +999,15 @@ def diff_base_to_repos(sbox):
   expected_output = svntest.wc.State(wc_dir, {})
   expected_disk = svntest.main.greek_state.copy()
   expected_disk.tweak('iota',
-                      contents="This is the file 'iota'.some rev2 iota text.an iota local mod.")
-  expected_disk.add({'A/D/newfile' : Item("Contents of newfile")})
+                      contents="This is the file 'iota'.\n" + \
+                      "some rev2 iota text.\nan iota local mod.\n")
+  expected_disk.add({'A/D/newfile' : Item("Contents of newfile\n")})
   expected_disk.remove ('A/mu')
 
   expected_status = svntest.actions.get_virginal_state(wc_dir, 3)
   expected_status.remove('A/mu')
   expected_status.add({
-    'A/D/newfile' : Item(status='  ', wc_rev=3, repos_rev=3),
+    'A/D/newfile' : Item(status='  ', wc_rev=3),
     })
   svntest.actions.run_and_verify_update(wc_dir, expected_output,
                                         expected_disk, expected_status)
@@ -1029,7 +1053,7 @@ def diff_deleted_in_head(sbox):
   mu_path = os.path.join(sbox.wc_dir, 'A', 'mu')
 
   # Make a change to mu, commit r2, update.
-  svntest.main.file_append(mu_path, "some rev2 mu text.")
+  svntest.main.file_append(mu_path, "some rev2 mu text.\n")
 
   expected_output = svntest.wc.State(wc_dir, {
     'A/mu' : Item(verb='Sending'),
@@ -1045,7 +1069,7 @@ def diff_deleted_in_head(sbox):
   expected_output = svntest.wc.State(wc_dir, {})
   expected_disk = svntest.main.greek_state.copy()
   expected_disk.tweak ('A/mu',
-                       contents="This is the file 'mu'.some rev2 mu text.")
+                       contents="This is the file 'mu'.\nsome rev2 mu text.\n")
   expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
   svntest.actions.run_and_verify_update(wc_dir, expected_output,
                                         expected_disk, expected_status)
@@ -1454,7 +1478,7 @@ def diff_renamed_file(sbox):
   os.chdir(sbox.wc_dir)
 
   
-  svntest.main.file_append(os.path.join('A', 'D', 'G', 'pi'), "more pi")
+  open(os.path.join('A', 'D', 'G', 'pi'), 'w').write("new pi")
   
   svntest.actions.run_and_verify_svn(None, None, [],
                                      'ci', '-m', 'log msg')
@@ -1534,7 +1558,7 @@ def diff_within_renamed_dir(sbox):
   svntest.main.run_svn(None, 'mv', os.path.join('A', 'D', 'G'),
                                    os.path.join('A', 'D', 'I'))
   # svntest.main.run_svn(None, 'ci', '-m', 'log_msg')
-  svntest.main.file_append(os.path.join('A', 'D', 'I', 'pi'), "new pi")
+  open(os.path.join('A', 'D', 'I', 'pi'), 'w').write("new pi")
 
   # Check a repos->wc diff
   diff_output, err_output = svntest.main.run_svn(None, 'diff',
@@ -1571,6 +1595,294 @@ def diff_within_renamed_dir(sbox):
 
   os.chdir(was_cwd)
 
+#----------------------------------------------------------------------
+def diff_prop_on_named_dir(sbox):
+  "diff a prop change on a dir named explicitly"
+
+  # Diff of a property change or addition should contain a "+" line.
+  # Diff of a property change or deletion should contain a "-" line.
+  # On a diff between repository revisions (not WC) of a dir named
+  # explicitly, the "-" line was missing.  (For a file, and for a dir
+  # recursed into, the result was correct.)
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  current_dir = os.getcwd()
+  os.chdir(sbox.wc_dir)
+  try:
+    svntest.actions.run_and_verify_svn(None, None, [],
+                                       'propset', 'p', 'v', 'A')
+    svntest.actions.run_and_verify_svn(None, None, [],
+                                       'ci', '-m', '')
+
+    svntest.actions.run_and_verify_svn(None, None, [],
+                                       'propdel', 'p', 'A')
+    svntest.actions.run_and_verify_svn(None, None, [],
+                                       'ci', '-m', '')
+
+    diff_output, err_output = svntest.main.run_svn(None,
+                                                   'diff', '-r2:3', 'A')
+    # Check that the result contains a "-" line.
+    verify_expected_output(diff_output, "   - v")
+
+  finally:
+    os.chdir(current_dir)
+
+#----------------------------------------------------------------------
+def diff_keywords(sbox):
+  "ensure that diff won't show keywords"
+
+  sbox.build()
+
+  iota_path = os.path.join(sbox.wc_dir, 'iota')
+  
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ps',
+                                     'svn:keywords',
+                                     'Id Rev Date',
+                                     iota_path)
+
+  fp = open(iota_path, 'w')
+  fp.write("$Date$\n")
+  fp.write("$Id$\n")
+  fp.write("$Rev$\n")
+  fp.write("$Date::%s$\n" % (' ' * 80))
+  fp.write("$Id::%s$\n"   % (' ' * 80))
+  fp.write("$Rev::%s$\n"  % (' ' * 80))
+  fp.close()
+  
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'keywords', sbox.wc_dir)
+
+  svntest.main.file_append(iota_path, "bar\n")
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'added bar', sbox.wc_dir)
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'up', sbox.wc_dir)
+
+  diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                        'diff',
+                                                        '-r', 'prev:head',
+                                                        sbox.wc_dir)
+  verify_expected_output(diff_output, "+bar")
+  verify_excluded_output(diff_output, "$Date:")
+  verify_excluded_output(diff_output, "$Rev:")
+  verify_excluded_output(diff_output, "$Id:")
+  
+  diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                        'diff',
+                                                        '-r', 'head:prev',
+                                                        sbox.wc_dir)
+  verify_expected_output(diff_output, "-bar")
+  verify_excluded_output(diff_output, "$Date:")
+  verify_excluded_output(diff_output, "$Rev:")
+  verify_excluded_output(diff_output, "$Id:")
+
+  # Check fixed length keywords will show up
+  # when the length of keyword has changed
+  fp = open(iota_path, 'w')
+  fp.write("$Date$\n")
+  fp.write("$Id$\n")
+  fp.write("$Rev$\n")
+  fp.write("$Date::%s$\n" % (' ' * 79))
+  fp.write("$Id::%s$\n"   % (' ' * 79))
+  fp.write("$Rev::%s$\n"  % (' ' * 79))
+  fp.close()
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'keywords 2', sbox.wc_dir)
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'up', sbox.wc_dir)
+
+  diff_output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                        'diff',
+                                                        '-r', 'prev:head',
+                                                        sbox.wc_dir)
+  # these should show up
+  verify_expected_output(diff_output, "+$Id:: ")
+  verify_expected_output(diff_output, "-$Id:: ")
+  verify_expected_output(diff_output, "-$Rev:: ")
+  verify_expected_output(diff_output, "+$Rev:: ")
+  verify_expected_output(diff_output, "-$Date:: ")
+  verify_expected_output(diff_output, "+$Date:: ")
+  # ... and these won't
+  verify_excluded_output(diff_output, "$Date: ")
+  verify_excluded_output(diff_output, "$Rev: ")
+  verify_excluded_output(diff_output, "$Id: ")
+
+
+def diff_force(sbox):
+  "show diffs for binary files with --force"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+  
+  iota_path = os.path.join(wc_dir, 'iota')
+  
+  # Append a line to iota and make it binary.
+  svntest.main.file_append(iota_path, "new line")
+  svntest.main.run_svn(None, 'propset', 'svn:mime-type',
+                       'application/octet-stream', iota_path)  
+
+  # Created expected output tree for 'svn ci'
+  expected_output = svntest.wc.State(wc_dir, {
+    'iota' : Item(verb='Sending'),
+    })
+
+  # Create expected status tree
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
+  expected_status.tweak(wc_rev=1)
+  expected_status.add({
+    'iota' : Item(status='  ', wc_rev=2),
+    })
+
+  # Commit iota, creating revision 2.
+  svntest.actions.run_and_verify_commit(wc_dir, expected_output,
+                                        expected_status, None,
+                                        None, None, None, None, wc_dir)
+
+  # Add another line, while keeping he file as binary.
+  svntest.main.file_append(iota_path, "another line")
+
+  # Commit creating rev 3.
+  expected_output = svntest.wc.State(wc_dir, {
+    'iota' : Item(verb='Sending'),
+    })
+
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 3)
+  expected_status.tweak(wc_rev=1)
+  expected_status.add({
+    'iota' : Item(status='  ', wc_rev=3),
+    })
+
+  svntest.actions.run_and_verify_commit(wc_dir, expected_output,
+                                        expected_status, None,
+                                        None, None, None, None, wc_dir)
+
+  # Check that we get diff when the first, the second and both files are
+  # marked as binary.
+
+  re_nodisplay = re.compile('^Cannot display:')
+
+  stdout, stderr = svntest.main.run_svn(None, 'diff', '-r1:2', iota_path,
+                                        '--force')
+
+  for line in stdout:
+    if (re_nodisplay.match(line)):
+      raise svntest.Failure
+
+  stdout, stderr = svntest.main.run_svn(None, 'diff', '-r2:1', iota_path,
+                                        '--force')
+
+  for line in stdout:
+    if (re_nodisplay.match(line)):
+      raise svntest.Failure
+
+  stdout, stderr = svntest.main.run_svn(None, 'diff', '-r2:3', iota_path,
+                                        '--force')
+
+  for line in stdout:
+    if (re_nodisplay.match(line)):
+      raise svntest.Failure
+
+#----------------------------------------------------------------------
+# Regression test for issue #2333: Renaming a directory should produce
+# deletion and addition diffs for each included file.
+def diff_renamed_dir(sbox):
+  "diff a renamed directory"
+
+  sbox.build()
+
+  was_cwd = os.getcwd()
+  os.chdir(sbox.wc_dir)
+
+  svntest.main.run_svn(None, 'mv', os.path.join('A', 'D', 'G'),
+                                   os.path.join('A', 'D', 'I'))
+
+  # Check a repos->wc diff
+  diff_output, err_output = svntest.main.run_svn(None, 'diff',
+                                                 os.path.join('A', 'D'))
+  if check_diff_output(diff_output,
+                       os.path.join('A', 'D', 'G', 'pi'),
+                       'D') :
+    raise svntest.Failure
+  if check_diff_output(diff_output,
+                       os.path.join('A', 'D', 'I', 'pi'),
+                       'A') :
+    raise svntest.Failure
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'ci', '-m', 'log msg')
+
+  # Check repos->wc after commit
+  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r', '1',
+                                                 os.path.join('A', 'D'))
+  if check_diff_output(diff_output,
+                       os.path.join('A', 'D', 'G', 'pi'),
+                       'D') :
+    raise svntest.Failure
+  if check_diff_output(diff_output,
+                       os.path.join('A', 'D', 'I', 'pi'),
+                       'A') :
+    raise svntest.Failure
+
+  # Test a repos->repos diff after commit
+  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r', '1:2')
+  if check_diff_output(diff_output,
+                       os.path.join('A', 'D', 'G', 'pi'),
+                       'D') :
+    raise svntest.Failure
+  if check_diff_output(diff_output,
+                       os.path.join('A', 'D', 'I', 'pi'),
+                       'A') :
+    raise svntest.Failure
+
+  # Test the diff while within the moved directory
+  os.chdir(os.path.join('A','D','I'))
+
+  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r', '1')
+
+  if check_diff_output(diff_output, 'pi', 'A') :
+    raise svntest.Failure
+
+  # Test a repos->repos diff while within the moved directory
+  diff_output, err_output = svntest.main.run_svn(None, 'diff', '-r', '1:2')
+
+  if check_diff_output(diff_output, 'pi', 'A') :
+    raise svntest.Failure
+
+  os.chdir(was_cwd)
+
+
+
+def diff_schedule_delete(sbox):
+  "scheduled deleted"
+  
+  sbox.build()
+  wc_dir = sbox.wc_dir
+  current_dir = os.getcwd()
+  os.chdir(wc_dir)
+
+  try:
+    diff_output, err_output = svntest.main.run_svn(None, 'ci', '-m', 'log msg')
+    if err_output: raise svntest.Failure
+    svntest.main.file_append('foo', "xxx")
+    svntest.main.run_svn(None, 'add', 'foo')
+    diff_output, err_output = svntest.main.run_svn(None, 'ci', '-m', 'log msg')
+    if err_output: raise svntest.Failure
+    svntest.main.run_svn(None, 'rm', 'foo')
+    expected_output = [
+    "Index: foo\n", 
+    "===================================================================\n"
+    ]
+    diff_output, err = svntest.actions.run_and_verify_svn(None, expected_output, [],
+                                                          'diff', '-r', '1' )
+    if err: raise svntest.Failure
+  finally:
+    os.chdir(current_dir)
+
 ########################################################################
 #Run the tests
 
@@ -1600,6 +1912,11 @@ test_list = [ None,
               check_for_omitted_prefix_in_path_component,
               diff_renamed_file,
               diff_within_renamed_dir,
+              diff_prop_on_named_dir,
+              diff_keywords,
+              diff_force,
+              diff_schedule_delete,
+              XFail(diff_renamed_dir),
               ]
 
 if __name__ == '__main__':
