@@ -1,7 +1,7 @@
 /* repos-test.c --- tests for the filesystem
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2005 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -25,8 +25,10 @@
 #include "svn_repos.h"
 #include "svn_path.h"
 #include "svn_delta.h"
-#include "svn_test.h"
-#include "../fs-helpers.h"
+#include "svn_config.h"
+
+#include "../svn_test.h"
+#include "../svn_test_fs.h"
 
 #include "dir-delta-editor.h"
 
@@ -795,7 +797,12 @@ node_locations (const char **msg,
         { 2, "/mu.new" },
         { 0 }
       };
+
+    /* Test this twice, once with a leading slash, once without,
+       because we know that the "without" form has caused us trouble
+       in the past. */
     SVN_ERR (check_locations (fs, info, "/mu.new", 2, pool));
+    SVN_ERR (check_locations (fs, info, "mu.new", 2, pool));
   }
   svn_pool_clear (subpool);
   
@@ -1027,6 +1034,665 @@ rmlocks (const char **msg,
 
   return SVN_NO_ERROR;
 }
+
+
+
+/* Helper for the authz test.  Set *AUTHZ_P to a representation of
+   AUTHZ_CONTENTS, using POOL for temporary allocation. */
+static svn_error_t *
+authz_get_handle (svn_authz_t **authz_p, const char *authz_contents,
+                  apr_pool_t *pool)
+{
+  apr_file_t *authz_file;
+  apr_status_t apr_err;
+  const char *authz_file_path;
+  svn_error_t *err;
+
+  /* Create a temporary file, and fetch its name. */
+  SVN_ERR_W (svn_io_open_unique_file (&authz_file, &authz_file_path,
+                                      "authz_file", "tmp", FALSE, pool),
+             "Opening temporary file");
+
+  /* Write the authz ACLs to the file. */
+  if ((apr_err = apr_file_write_full (authz_file, authz_contents,
+                                      strlen (authz_contents), NULL)))
+    {
+      (void) apr_file_close (authz_file);
+      (void) apr_file_remove (authz_file_path, pool);
+      return svn_error_wrap_apr (apr_err, "Writing test authz file");
+    }
+
+  /* Close the temporary descriptor. */
+  if ((apr_err = apr_file_close (authz_file)))
+    {
+      (void) apr_file_remove (authz_file_path, pool);
+      return svn_error_wrap_apr (apr_err, "Closing test authz file");
+    }
+
+  /* Read the authz configuration back and start testing. */
+  if ((err = svn_repos_authz_read (authz_p, authz_file_path, TRUE, pool)))
+    {
+      (void) apr_file_remove (authz_file_path, pool);
+      return svn_error_quick_wrap (err, "Opening test authz file");
+    }
+
+  /* Delete the file, but ignore the error if we've a more important one. */
+  if ((apr_err = apr_file_remove (authz_file_path, pool)))
+    return svn_error_wrap_apr (apr_err, "Removing test authz file");
+
+  return SVN_NO_ERROR;
+}
+
+
+
+/* Test that authz is giving out the right authorizations. */
+static svn_error_t *
+authz (const char **msg,
+       svn_boolean_t msg_only,
+       svn_test_opts_t *opts,
+       apr_pool_t *pool)
+{
+  const char *contents;
+  svn_authz_t *authz_cfg;
+  svn_error_t *err;
+  svn_boolean_t access_granted;
+  apr_pool_t *subpool = svn_pool_create (pool);
+  int i;
+  /* Definition of the paths to test and expected replies for each. */
+  struct
+  {
+    const char *path;
+    const char *user;
+    const svn_repos_authz_access_t required;
+    const svn_boolean_t expected;
+  } test_set[] = {
+    /* Test that read rules are correctly used. */
+    { "/A", NULL, svn_authz_read, TRUE },
+    { "/iota", NULL, svn_authz_read, FALSE },
+    /* Test that write rules are correctly used. */
+    { "/A", "plato", svn_authz_write, TRUE },
+    { "/A", NULL, svn_authz_write, FALSE },
+    /* Test that pan-repository rules are found and used. */
+    { "/A/B/lambda", "plato", svn_authz_read, TRUE },
+    { "/A/B/lambda", NULL, svn_authz_read, FALSE },
+    /* Test that authz uses parent path ACLs if no rule for the path
+       exists. */
+    { "/A/C", NULL, svn_authz_read, TRUE },
+    /* Test that recursive access requests take into account the rules
+       of subpaths. */
+    { "/A/D", "plato", svn_authz_read | svn_authz_recursive, TRUE },
+    { "/A/D", NULL, svn_authz_read | svn_authz_recursive, FALSE },
+    /* Test global write access lookups. */
+    { NULL, "plato", svn_authz_read, TRUE },
+    { NULL, NULL, svn_authz_write, FALSE },
+    /* Sentinel */
+    { NULL, NULL, svn_authz_none, FALSE }
+  };
+
+  *msg = "test authz access control";
+
+  if (msg_only)
+    return SVN_NO_ERROR;
+
+  /* The test logic:
+   *
+   * 1. Perform various access tests on a set of authz rules.  Each
+   * test has a known outcome and tests different aspects of authz,
+   * such as inheriting parent-path authz, pan-repository rules or
+   * recursive access.  'plato' is our friendly neighborhood user with
+   * more access rights than other anonymous philosophers.
+   *
+   * 2. Load an authz file containing a cyclic dependency in groups
+   * and another containing a reference to an undefined group.  Verify
+   * that svn_repos_authz_read fails to load both and returns an
+   * "invalid configuration" error.
+   *
+   * 3. Regression test for a bug in how recursion is handled in
+   * authz.  The bug was that paths not under the parent path
+   * requested were being considered during the determination of
+   * access rights (eg. a rule for /dir2 matched during a lookup for
+   * /dir), due to incomplete tests on path relations.
+   */
+
+  /* The authz rules for the phase 1 tests. */
+  contents =
+    "[greek:/A]"
+    APR_EOL_STR
+    "* = r"
+    APR_EOL_STR
+    "plato = w"
+    APR_EOL_STR
+    APR_EOL_STR
+    "[greek:/iota]"
+    APR_EOL_STR
+    "* ="
+    APR_EOL_STR
+    APR_EOL_STR
+    "[/A/B/lambda]"
+    APR_EOL_STR
+    "plato = r"
+    APR_EOL_STR
+    "* ="
+    APR_EOL_STR
+    APR_EOL_STR
+    "[greek:/A/D]"
+    APR_EOL_STR
+    "plato = r"
+    APR_EOL_STR
+    "* = r"
+    APR_EOL_STR
+    APR_EOL_STR
+    "[greek:/A/D/G]"
+    APR_EOL_STR
+    "plato = r"
+    APR_EOL_STR
+    "* ="
+    APR_EOL_STR
+    APR_EOL_STR
+    "[greek:/A/B/E/beta]"
+    APR_EOL_STR
+    "* ="
+    APR_EOL_STR
+    APR_EOL_STR;
+
+  /* Load the test authz rules. */
+  SVN_ERR (authz_get_handle (&authz_cfg, contents, subpool));
+
+  /* Loop over the test array and test each case. */
+  for (i = 0; test_set[i].path != NULL; i++)
+    {
+      SVN_ERR (svn_repos_authz_check_access (authz_cfg, "greek",
+                                             test_set[i].path,
+                                             test_set[i].user,
+                                             test_set[i].required,
+                                             &access_granted, subpool));
+
+      if (access_granted != test_set[i].expected)
+        {
+          return svn_error_createf (SVN_ERR_TEST_FAILED, NULL,
+                                    "Authz incorrectly %s %s%s access "
+                                    "to greek:%s for user %s",
+                                    access_granted ?
+                                    "grants" : "denies",
+                                    test_set[i].required
+                                    & svn_authz_recursive ?
+                                    "recursive " : "",
+                                    test_set[i].required
+                                    & svn_authz_read ?
+                                    "read" : "write",
+                                    test_set[i].path,
+                                    test_set[i].user ?
+                                    test_set[i].user : "-");
+        }
+    }
+
+
+  /* The authz rules for the phase 2 tests, first case (cyclic
+     dependency). */
+  contents =
+    "[groups]"
+    APR_EOL_STR
+    "slaves = cooks,scribes,@gladiators"
+    APR_EOL_STR
+    "gladiators = equites,thraces,@slaves"
+    APR_EOL_STR
+    APR_EOL_STR
+    "[greek:/A]"
+    APR_EOL_STR
+    "@slaves = r"
+    APR_EOL_STR;
+
+  /* Load the test authz rules and check that group cycles are
+     reported. */
+  err = authz_get_handle (&authz_cfg, contents, subpool);
+  if (!err || err->apr_err != SVN_ERR_AUTHZ_INVALID_CONFIG)
+    return svn_error_createf (SVN_ERR_TEST_FAILED, err,
+                              "Got %s error instead of expected "
+                              "SVN_ERR_AUTHZ_INVALID_CONFIG",
+                              err ? "unexpected" : "no");
+  svn_error_clear (err);
+
+  /* The authz rules for the phase 2 tests, second case (missing group
+     definition). */
+  contents =
+    "[greek:/A]"
+    APR_EOL_STR
+    "@senate = r"
+    APR_EOL_STR;
+
+  /* Check that references to undefined groups are reported. */
+  err = authz_get_handle (&authz_cfg, contents, subpool);
+  if (!err || err->apr_err != SVN_ERR_AUTHZ_INVALID_CONFIG)
+    return svn_error_createf (SVN_ERR_TEST_FAILED, err,
+                              "Got %s error instead of expected "
+                              "SVN_ERR_AUTHZ_INVALID_CONFIG",
+                              err ? "unexpected" : "no");
+  svn_error_clear (err);
+
+  /* The authz rules for the phase 3 tests */
+  contents =
+    "[/]"
+    APR_EOL_STR
+    "* = rw"
+    APR_EOL_STR
+    APR_EOL_STR
+    "[greek:/dir2/secret]"
+    APR_EOL_STR
+    "* ="
+    APR_EOL_STR;
+
+  /* Load the test authz rules. */
+  SVN_ERR (authz_get_handle (&authz_cfg, contents, subpool));
+
+  /* Verify that the rule on /dir2/secret doesn't affect this
+     request */
+  SVN_ERR (svn_repos_authz_check_access (authz_cfg, "greek",
+                                         "/dir", NULL,
+                                         (svn_authz_read
+                                          | svn_authz_recursive),
+                                         &access_granted, subpool));
+  if (!access_granted)
+    return svn_error_create (SVN_ERR_TEST_FAILED, NULL,
+                             "Regression: incomplete ancestry test "
+                             "for recursive access lookup.");
+
+  /* That's a wrap! */
+  svn_pool_destroy (subpool);
+  return SVN_NO_ERROR;
+}
+
+
+
+/* Callback for the commit editor tests that relays requests to
+   authz. */
+static
+svn_error_t *commit_authz_cb (svn_repos_authz_access_t required,
+                              svn_boolean_t *allowed,
+                              svn_fs_root_t *root,
+                              const char *path,
+                              void *baton,
+                              apr_pool_t *pool)
+{
+  svn_authz_t *authz_file = baton;
+
+  return svn_repos_authz_check_access (authz_file, "test", path,
+                                       "plato", required, allowed,
+                                       pool);
+}
+
+
+
+/* Test that the commit editor is taking authz into account
+   properly */
+static svn_error_t *
+commit_editor_authz  (const char **msg,
+                      svn_boolean_t msg_only,
+                      svn_test_opts_t *opts,
+                      apr_pool_t *pool)
+{
+  svn_repos_t *repos;
+  svn_fs_t *fs;
+  svn_fs_txn_t *txn;
+  svn_fs_root_t *txn_root;
+  svn_revnum_t youngest_rev;
+  void *edit_baton;
+  void *root_baton, *dir_baton, *dir2_baton, *file_baton;
+  svn_error_t *err;
+  const svn_delta_editor_t *editor;
+  svn_authz_t *authz_file;
+  apr_pool_t *subpool = svn_pool_create (pool);
+  const char *authz_contents;
+
+  *msg = "test authz in the commit editor";
+
+  if (msg_only)
+    return SVN_NO_ERROR;
+
+  /* The Test Plan
+   *
+   * We create a greek tree repository, then create a commit editor
+   * and try to perform various operations that will run into authz
+   * callbacks.  Check that all operations are properly
+   * authorized/denied when necessary.  We don't try to be exhaustive
+   * in the kinds of authz lookups.  We just make sure that the editor
+   * replies to the calls in a way that proves it is doing authz
+   * lookups.
+   *
+   * Note that this use of the commit editor is not kosher according
+   * to the generic editor API (we aren't allowed to continue editing
+   * after an error, nor are we allowed to assume that errors are
+   * returned by the operations which caused them).  But it should
+   * work fine with this particular editor implementation.
+   */
+
+  /* Create a filesystem and repository. */
+  SVN_ERR (svn_test__create_repos (&repos, "test-repo-commit-authz",
+                                   opts->fs_type, subpool));
+  fs = svn_repos_fs (repos);
+
+  /* Prepare a txn to receive the greek tree. */
+  SVN_ERR (svn_fs_begin_txn (&txn, fs, 0, subpool));
+  SVN_ERR (svn_fs_txn_root (&txn_root, txn, subpool));
+  SVN_ERR (svn_test__create_greek_tree (txn_root, subpool));
+  SVN_ERR (svn_repos_fs_commit_txn (NULL, repos, &youngest_rev, txn, subpool));
+
+  /* Load the authz rules for the greek tree. */
+  authz_contents =
+    APR_EOL_STR
+    APR_EOL_STR
+    "[/]"
+    APR_EOL_STR
+    "plato = r"
+    APR_EOL_STR
+    APR_EOL_STR
+    "[/A]"
+    APR_EOL_STR
+    "plato = rw"
+    APR_EOL_STR
+    APR_EOL_STR
+    "[/A/alpha]"
+    APR_EOL_STR
+    "plato = "
+    APR_EOL_STR
+    APR_EOL_STR
+    "[/A/C]"
+    APR_EOL_STR
+    APR_EOL_STR
+    "plato = "
+    APR_EOL_STR
+    APR_EOL_STR
+    "[/A/D]"
+    APR_EOL_STR
+    "plato = rw"
+    APR_EOL_STR
+    APR_EOL_STR
+    "[/A/D/G]"
+    APR_EOL_STR
+    "plato = r"
+    ;
+
+  SVN_ERR (authz_get_handle (&authz_file, authz_contents, subpool));
+
+  /* Create a new commit editor in which we're going to play with
+     authz */
+  SVN_ERR (svn_repos_get_commit_editor3 (&editor, &edit_baton, repos,
+                                         NULL, "file://test", "/",
+                                         "plato", "test commit", NULL,
+                                         NULL, commit_authz_cb, authz_file,
+                                         subpool));
+
+  /* Start fiddling.  First get the root, which is readonly.  All
+     write operations fail because of the root's permissions. */
+  SVN_ERR (editor->open_root (edit_baton, 1, subpool, &root_baton));
+
+  /* Test denied file deletion. */
+  err = editor->delete_entry ("/iota", SVN_INVALID_REVNUM, root_baton, subpool);
+  if (err == SVN_NO_ERROR || err->apr_err != SVN_ERR_AUTHZ_UNWRITABLE)
+    return svn_error_createf (SVN_ERR_TEST_FAILED, err,
+                              "Got %s error instead of expected "
+                              "SVN_ERR_AUTHZ_UNWRITABLE",
+                              err ? "unexpected" : "no");
+  svn_error_clear (err);
+
+  /* Test authorized file open. */
+  SVN_ERR (editor->open_file ("/iota", root_baton, SVN_INVALID_REVNUM,
+                              subpool, &file_baton));
+
+  /* Test unauthorized file prop set. */
+  err = editor->change_file_prop (file_baton, "svn:test",
+                                  svn_string_create ("test", subpool),
+                                  subpool);
+  if (err == SVN_NO_ERROR || err->apr_err != SVN_ERR_AUTHZ_UNWRITABLE)
+    return svn_error_createf (SVN_ERR_TEST_FAILED, err,
+                              "Got %s error instead of expected "
+                              "SVN_ERR_AUTHZ_UNWRITABLE",
+                              err ? "unexpected" : "no");
+  svn_error_clear (err);
+
+  /* Test denied file addition. */
+  err = editor->add_file ("/alpha", root_baton, NULL, SVN_INVALID_REVNUM,
+                          subpool, &file_baton);
+  if (err == SVN_NO_ERROR || err->apr_err != SVN_ERR_AUTHZ_UNWRITABLE)
+    return svn_error_createf (SVN_ERR_TEST_FAILED, err,
+                              "Got %s error instead of expected "
+                              "SVN_ERR_AUTHZ_UNWRITABLE",
+                              err ? "unexpected" : "no");
+  svn_error_clear (err);
+
+  /* Test denied file copy. */
+  err = editor->add_file ("/alpha", root_baton, "file://test/A/B/lambda",
+                          youngest_rev, subpool, &file_baton);
+  if (err == SVN_NO_ERROR || err->apr_err != SVN_ERR_AUTHZ_UNWRITABLE)
+    return svn_error_createf (SVN_ERR_TEST_FAILED, err,
+                              "Got %s error instead of expected "
+                              "SVN_ERR_AUTHZ_UNWRITABLE",
+                              err ? "unexpected" : "no");
+  svn_error_clear (err);
+
+  /* Test denied directory addition. */
+  err = editor->add_directory ("/I", root_baton, NULL,
+                               SVN_INVALID_REVNUM, subpool, &dir_baton);
+  if (err == SVN_NO_ERROR || err->apr_err != SVN_ERR_AUTHZ_UNWRITABLE)
+    return svn_error_createf (SVN_ERR_TEST_FAILED, err,
+                              "Got %s error instead of expected "
+                              "SVN_ERR_AUTHZ_UNWRITABLE",
+                              err ? "unexpected" : "no");
+  svn_error_clear (err);
+
+  /* Test denied directory copy. */
+  err = editor->add_directory ("/J", root_baton, "file://test/A/D",
+                               youngest_rev, subpool, &dir_baton);
+  if (err == SVN_NO_ERROR || err->apr_err != SVN_ERR_AUTHZ_UNWRITABLE)
+    return svn_error_createf (SVN_ERR_TEST_FAILED, err,
+                              "Got %s error instead of expected "
+                              "SVN_ERR_AUTHZ_UNWRITABLE",
+                              err ? "unexpected" : "no");
+  svn_error_clear (err);
+
+  /* Open directory /A, to which we have read/write access. */
+  SVN_ERR (editor->open_directory ("/A", root_baton,
+                                   SVN_INVALID_REVNUM,
+                                   pool, &dir_baton));
+
+  /* Test denied file addition.  Denied because of a conflicting rule
+     on the file path itself. */
+  err = editor->add_file ("/A/alpha", dir_baton, NULL,
+                          SVN_INVALID_REVNUM, subpool, &file_baton);
+  if (err == SVN_NO_ERROR || err->apr_err != SVN_ERR_AUTHZ_UNWRITABLE)
+    return svn_error_createf (SVN_ERR_TEST_FAILED, err,
+                              "Got %s error instead of expected "
+                              "SVN_ERR_AUTHZ_UNWRITABLE",
+                              err ? "unexpected" : "no");
+  svn_error_clear (err);
+
+  /* Test authorized file addition. */
+  SVN_ERR (editor->add_file ("/A/B/theta", dir_baton, NULL,
+                             SVN_INVALID_REVNUM, subpool,
+                             &file_baton));
+
+  /* Test authorized file deletion. */
+  SVN_ERR (editor->delete_entry ("/A/mu", SVN_INVALID_REVNUM, dir_baton,
+                                 subpool));
+
+  /* Test authorized directory creation. */
+  SVN_ERR (editor->add_directory ("/A/E", dir_baton, NULL,
+                                  SVN_INVALID_REVNUM, subpool,
+                                  &dir2_baton));
+
+  /* Test authorized copy of a tree. */
+  SVN_ERR (editor->add_directory ("/A/J", dir_baton, "file://test/A/D",
+                                  youngest_rev, subpool,
+                                  &dir2_baton));
+
+  /* Test denied access to a directory. */
+  err = editor->open_directory ("/A/C", dir_baton, SVN_INVALID_REVNUM,
+                                subpool, &dir2_baton);
+  if (err == SVN_NO_ERROR || err->apr_err != SVN_ERR_AUTHZ_UNREADABLE)
+    return svn_error_createf (SVN_ERR_TEST_FAILED, err,
+                              "Got %s error instead of expected "
+                              "SVN_ERR_AUTHZ_UNREADABLE",
+                              err ? "unexpected" : "no");
+  svn_error_clear (err);
+
+  /* Open /A/D.  This should be granted. */
+  SVN_ERR (editor->open_directory ("/A/D", dir_baton, SVN_INVALID_REVNUM,
+                                   subpool, &dir_baton));
+
+  /* Test denied recursive deletion. */
+  err = editor->delete_entry ("/A/D/G", SVN_INVALID_REVNUM, dir_baton,
+                              subpool);
+  if (err == SVN_NO_ERROR || err->apr_err != SVN_ERR_AUTHZ_UNWRITABLE)
+    return svn_error_createf (SVN_ERR_TEST_FAILED, err,
+                              "Got %s error instead of expected "
+                              "SVN_ERR_AUTHZ_UNWRITABLE",
+                              err ? "unexpected" : "no");
+  svn_error_clear (err);
+
+  /* Test authorized recursive deletion. */
+  SVN_ERR (editor->delete_entry ("/A/D/H", SVN_INVALID_REVNUM,
+                                 dir_baton, subpool));
+
+  /* Test authorized propset (open the file first). */
+  SVN_ERR (editor->open_file ("/A/D/gamma", dir_baton, SVN_INVALID_REVNUM,
+                              subpool, &file_baton));
+  SVN_ERR (editor->change_file_prop (file_baton, "svn:test",
+                                     svn_string_create("test", subpool),
+                                     subpool));
+
+  /* Done. */
+  editor->abort_edit (edit_baton, subpool);
+  svn_pool_destroy (subpool);
+
+  return SVN_NO_ERROR;
+}
+
+/* This implements svn_commit_callback2_t. */
+static svn_error_t *
+dummy_commit_cb(svn_revnum_t new_revision,
+                const char *date,
+                const char *author,
+                void *baton)
+{
+  return SVN_NO_ERROR;
+}
+
+/* Test using explicit txns during a commit. */
+static svn_error_t *
+commit_continue_txn (const char **msg,
+                     svn_boolean_t msg_only,
+                     svn_test_opts_t *opts,
+                     apr_pool_t *pool)
+{
+  svn_repos_t *repos;
+  svn_fs_t *fs;
+  svn_fs_txn_t *txn;
+  svn_fs_root_t *txn_root, *revision_root;
+  svn_revnum_t youngest_rev;
+  void *edit_baton;
+  void *root_baton, *file_baton;
+  const svn_delta_editor_t *editor;
+  apr_pool_t *subpool = svn_pool_create (pool);
+  const char *txn_name;
+
+  *msg = "test commit with explicit txn";
+
+  if (msg_only)
+    return SVN_NO_ERROR;
+
+  /* The Test Plan
+   *
+   * We create a greek tree repository, then create a transaction and
+   * a commit editor from that txn.  We do one change, abort the edit, reopen
+   * the txn and create a new commit editor, do anyther change and commit.
+   * We check that both changes were done.
+   */
+
+  /* Create a filesystem and repository. */
+  SVN_ERR (svn_test__create_repos (&repos, "test-repo-commit-continue",
+                                   opts->fs_type, subpool));
+  fs = svn_repos_fs (repos);
+
+  /* Prepare a txn to receive the greek tree. */
+  SVN_ERR (svn_fs_begin_txn (&txn, fs, 0, subpool));
+  SVN_ERR (svn_fs_txn_root (&txn_root, txn, subpool));
+  SVN_ERR (svn_test__create_greek_tree (txn_root, subpool));
+  SVN_ERR (svn_repos_fs_commit_txn (NULL, repos, &youngest_rev, txn, subpool));
+
+  SVN_ERR (svn_fs_begin_txn (&txn, fs, youngest_rev, subpool));
+  SVN_ERR (svn_fs_txn_name (&txn_name, txn, subpool));
+  SVN_ERR (svn_repos_get_commit_editor3 (&editor, &edit_baton, repos,
+                                         txn, "file://test", "/",
+                                         "plato", "test commit",
+                                         dummy_commit_cb, NULL, NULL, NULL,
+                                         subpool));
+
+  SVN_ERR (editor->open_root (edit_baton, 1, subpool, &root_baton));
+
+  SVN_ERR (editor->add_file ("/f1", root_baton, NULL, SVN_INVALID_REVNUM,
+                             subpool, &file_baton));
+  SVN_ERR (editor->close_file (file_baton, NULL, subpool));
+  /* This should leave the transaction. */
+  SVN_ERR (editor->abort_edit (edit_baton, subpool));
+
+  /* Reopen the transaction. */
+  SVN_ERR (svn_fs_open_txn (&txn, fs, txn_name, subpool));
+  SVN_ERR (svn_repos_get_commit_editor3 (&editor, &edit_baton, repos,
+                                         txn, "file://test", "/",
+                                         "plato", "test commit",
+                                         dummy_commit_cb,
+                                         NULL, NULL, NULL,
+                                         subpool));
+
+  SVN_ERR (editor->open_root (edit_baton, 1, subpool, &root_baton));
+
+  SVN_ERR (editor->add_file ("/f2", root_baton, NULL, SVN_INVALID_REVNUM,
+                             subpool, &file_baton));
+  SVN_ERR (editor->close_file (file_baton, NULL, subpool));
+
+  /* Finally, commit it. */
+  SVN_ERR (editor->close_edit (edit_baton, subpool));
+  
+  /* Check that the edits really happened. */
+  {
+    static svn_test__tree_entry_t expected_entries[] = {
+      /* path, contents (0 = dir) */
+      { "iota",        "This is the file 'iota'.\n" },
+      { "A",           0 },
+      { "A/mu",        "This is the file 'mu'.\n" },
+      { "A/B",         0 },
+      { "A/B/lambda",  "This is the file 'lambda'.\n" },
+      { "A/B/E",       0 },
+      { "A/B/E/alpha", "This is the file 'alpha'.\n" },
+      { "A/B/E/beta",  "This is the file 'beta'.\n" },
+      { "A/B/F",       0 },
+      { "A/C",         0 },
+      { "A/D",         0 },
+      { "A/D/gamma",   "This is the file 'gamma'.\n" },
+      { "A/D/G",       0 },
+      { "A/D/G/pi",    "This is the file 'pi'.\n" },
+      { "A/D/G/rho",   "This is the file 'rho'.\n" },
+      { "A/D/G/tau",   "This is the file 'tau'.\n" },
+      { "A/D/H",       0 },
+      { "A/D/H/chi",   "This is the file 'chi'.\n" },
+      { "A/D/H/psi",   "This is the file 'psi'.\n" },
+      { "A/D/H/omega", "This is the file 'omega'.\n" },
+      { "f1",          "" },
+      { "f2",          "" }
+    };
+    SVN_ERR (svn_fs_revision_root (&revision_root, fs, 
+                                   2, subpool)); 
+    SVN_ERR (svn_test__validate_tree 
+             (revision_root, expected_entries,
+              sizeof (expected_entries) / sizeof(expected_entries[0]),
+              subpool));
+  }
+
+  svn_pool_destroy (subpool);
+
+  return SVN_NO_ERROR;
+}
+
+
 
 /* The test table.  */
 
@@ -1038,5 +1704,8 @@ struct svn_test_descriptor_t test_funcs[] =
     SVN_TEST_PASS (revisions_changed),
     SVN_TEST_PASS (node_locations),
     SVN_TEST_PASS (rmlocks),
+    SVN_TEST_PASS (authz),
+    SVN_TEST_PASS (commit_editor_authz),
+    SVN_TEST_PASS (commit_continue_txn),
     SVN_TEST_NULL
   };
